Package: snpStats
Title: SnpMatrix and XSnpMatrix classes and methods
Version: 1.28.0
Date: 2015-12-19
Author: David Clayton <dc208@cam.ac.uk> 
Description: Classes and statistical methods for large SNP association studies. This extends the earlier snpMatrix package, allowing for uncertainty in genotypes.
Maintainer: David Clayton <dc208@cam.ac.uk>
Depends: R(>= 2.10.0), survival, Matrix, methods
Imports: graphics, grDevices, stats, utils, BiocGenerics, zlibbioc
Suggests: hexbin
License: GPL-3
Collate: ss.R contingency.table.R convert.R compare.R glm-test.R
        imputation.R indata.R long.R misc.R ld.R mvtests.R pedfile.R
        outdata.R plink.R qc.R qq-chisq.R single.R tdt-single.R
        structure.R xstuff.R
LazyLoad: yes
biocViews: Microarray, SNP, GeneticVariability
Packaged: 2017-10-30 22:50:29 UTC; biocbuild
NeedsCompilation: yes

Package: snpStats
Title: SnpMatrix and XSnpMatrix classes and methods
Version: 1.56.0
Date: 2022-07-27
Author: David Clayton <dc208@cam.ac.uk> 
Description: Classes and statistical methods for large SNP association studies. This extends the earlier snpMatrix package, allowing for uncertainty in genotypes.
Maintainer: David Clayton <dc208@cam.ac.uk>
Depends: R(>= 2.10.0), survival, Matrix, methods
Imports: graphics, grDevices, stats, utils, BiocGenerics, zlibbioc
Suggests: hexbin
License: GPL-3
Collate: ss.R contingency.table.R convert.R compare.R glm-test.R
        imputation.R indata.R long.R misc.R ld.R mvtests.R pedfile.R
        outdata.R plink.R qc.R qq-chisq.R single.R tdt-single.R
        structure.R xstuff.R
LazyLoad: yes
biocViews: Microarray, SNP, GeneticVariability
Packaged: 2024-10-30 02:57:03 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/snpStats
git_branch: RELEASE_3_20
git_last_commit: 92d244d
git_last_commit_date: 2024-10-29
Repository: Bioconductor 3.20
Date/Publication: 2024-10-29
NeedsCompilation: yes

useDynLib("snpStats")
import(methods)
import(zlibbioc)
importFrom(graphics, abline, axis, barplot, legend, lines, text, mtext, par,
           plot, points, polygon)
importFrom(grDevices, heat.colors)
importFrom(stats, model.matrix, model.offset, model.response, contrasts,
           model.weights, pchisq, qbeta, qchisq, terms)
importFrom(survival, cluster, strata, untangle.specials)
importFrom(utils, download.file, flush.console, head, read.table, write.table)
importFrom(BiocGenerics, cbind)
importFrom(BiocGenerics, rbind)
importClassesFrom(Matrix, dsCMatrix, dgCMatrix)

exportClasses(SnpMatrix, XSnpMatrix, ImputationRules,
              SingleSnpTests, SingleSnpTestsScore,
              GlmTests, GlmEstimates)
exportMethods(cbind, chi.squared, deg.freedom, effect.sign, is.na, 
              names, pool2, p.value, rbind, show, summary, switch.alleles,
              plot, sample.size, effective.sample.size,
              "[", "[<-", coerce, initialize)
export(can.impute, convert.snpMatrix, convert.snpMatrix.dir,
       filter.rules, Fst, glm.test.control, g2post, ibsCount, ibsDist,
       impute.snps, imputation.maf, imputation.r2, imputation.nsnp, ld, mean2g,
       misinherits, mvtests, plotUncertainty, pool, post2g, pp, qq.chisq, 
       read.beagle, read.impute, read.long, read.mach, read.pedfile,
       read.plink, read.snps.long, random.snps,
       row.summary, col.summary,
       single.snp.tests, sm.compare,
       snp.cor, snp.imputation, snp.lhs.tests, snp.lhs.estimates,
       snp.post.multiply, snp.pre.multiply,
       snp.rhs.tests, snp.rhs.estimates, tdt.snp,
       test.allele.switch, write.plink, write.SnpMatrix, xxt)

test.allele.switch <- function(snps, snps2=NULL, split=NULL, prior.df=1) {
  if ((class(snps)!="SnpMatrix") && (class(snps)!="XSnpMatrix"))
    stop("argument `snps' is not a SnpMatrix or XSnpMatrix")
  if (is.null(snps2) && is.null(split))
    stop("either second SnpMatrix or split vector must be specified")
  if (is.null(snps2)) {
    if (length(split) != nrow(snps)) 
      stop("argument `split' not equal to number or rows in SnpMatrix")
    if (length(unique(split[!is.na(split)]))!=2)
      stop("argument `split' cannot be coerced into a 2-level factor")
    split <- as.factor(split)
  }
  else {
    if ((class(snps2)!="SnpMatrix") && (class(snps2)!="XSnpMatrix"))
      stop("argument `snps2' is not a SnpMatrix or XSnpMatrix")
    if (ncol(snps)!=ncol(snps2))
      stop("two SnpMatrix objects have different numbers of columns")
  }
  .Call("test_switch", snps, snps2, split, prior.df, PACKAGE="snpStats")
}

single.snp.tests <- function(phenotype, stratum, data=sys.parent(), snp.data,
                             rules=NULL, subset, snp.subset, uncertain=FALSE,
                             score=FALSE) {
  m <- match.call()
  smiss <- missing(stratum)
  ssmiss <- missing(subset)
  if (smiss)
    stratum <- NULL
  if (!is(snp.data, "SnpMatrix"))
    stop("snp.data argument must be of class SnpMatrix")
  nr.snps = nrow(snp.data)
  nc.snps = ncol(snp.data)
  if (missing(data)) { # phenotype and stratum are in calling environment
    phenotype <- as.numeric(phenotype)
    complt <- !is.na(phenotype)
    if (length(phenotype)!=nr.snps)
      stop("incompatible length for phenotype vector")
    if (!smiss) {
      stratum <- as.integer(as.factor(stratum))
      if (length(stratum)!=nr.snps)
        stop("incompatible length for stratum vector")
      complt <- complt & !is.na(stratum)
    }
    if (!ssmiss) {
      if (is.logical(subset)) {
        if (length(subset)!=nr.snps)
          stop("incompatible length for subset vector")
        else
          inss <- subset
      }
      else if (is.integer(subset)) {
        if (length(subset)>nr.snps ||
            any(subset>nr.snps | duplicated(subset)))
          stop("illegal subset vector")
        inss <- rep(FALSE, nr.snps)
        inss[subset] <- TRUE
      }
      else {
        stop("illegal type for subset vector")
      }
      complt <- complt & inss
    }
  }
  else { # phenotype, stratum, and subset  are in data dataframe
    nm.snps <- rownames(snp.data)
    nm.data <- rownames(data)
    nr.data <- nrow(data)
    # which points to rows in the data matrix
    which <- match(nm.snps, nm.data, nomatch=NA)
    phenotype <- as.numeric(eval(m$phenotype, envir=data))[which]
    complt <- !is.na(phenotype) 
    if (!smiss) {
      stratum <- as.integer(as.factor(eval(m$stratum, envir=data)))[which]
      complt <- complt & !is.na(stratum)
    }
    if (!ssmiss) {
      subset <- eval(m$subset, envir=data)
      if (is.logical(subset)) {
        if (length(subset)!=nr.data)
          stop("incompatible length for subset vector")
        inss <- subset
      }
      else {
        if (is.character(subset)) {
          subset <- match(subset, nm.data, nomatch=0)
          if (any(subset==0))
            stop("unmatched name in subset vector")
        }
        else if (is.integer(subset)) {
          if (length(subset)>nr.data ||
              any(subset>nr.data | duplicated(subset)))
            stop("illegal subset vector")
        }
        else {
          stop("illegal type for subset vector")
        }
        inss <- rep(FALSE, nr.data)
        inss[subset] <- TRUE
      }
      inss <- inss[which]
      inss[is.na(inss)] <- FALSE
      complt <- complt & inss
    }
  }
  if (class(snp.data)=="XSnpMatrix" && any(is.na(snp.data@diploid))) {
    warning("There are ", sum(is.na(snp.data@diploid)),
            " subjects with NA in the diploid slot. These were ignored")
    complt <- complt & !is.na(snp.data@diploid)
  }
  subset <- (1:nr.snps)[complt]


  if (missing(snp.subset))
    snp.subset <- NULL
  else {
    if (is.null(rules))
      n.targ <- nc.snps
    else
      n.targ <- length(rules)
    if (is.character(snp.subset)) {
      if (is.null(rules))
        snp.subset <- match(snp.subset, colnames(snp.data))
      else
        snp.subset <- match(snp.subset, names(rules))
      if (any(is.na(snp.subset)))
        stop("unmatched snps in snp.subset argument")
    }
    else if (is.logical(snp.subset)){
      if (length(snp.subset)!=n.targ)
        stop("snp.subset array has incorrect length")
      snp.subset <- (1:n.targ)[snp.subset]
    }
    else if (is.integer(snp.subset)) {
      if (any(snp.subset<1 | snp.subset>n.targ))
        stop("snp.subset element(s) out of range")
    }
    else 
      stop("illegal type for snp.subset")
  }
  scores <- .Call("score_single", phenotype, stratum, snp.data, rules, subset,
                  snp.subset, uncertain, PACKAGE="snpStats")
  chisq <- .Call("chisq_single", scores, PACKAGE="snpStats")
  if (is.null(rules)) {
    if (is.null(snp.subset))
      tested <- colnames(snp.data)
    else
      tested <- colnames(snp.data)[snp.subset]
  } else {
    if (is.null(snp.subset))
      tested <- names(rules)
    else
      tested <- names(rules)[snp.subset]
  }
  if (score)
    res <- new("SingleSnpTestsScore", snp.names=tested,
               chisq=chisq, N=scores$N, N.r2=scores$N.r2,
               U=scores$U, V=scores$V)
  else
    res <- new("SingleSnpTests", snp.names=tested, chisq=chisq,
               N=scores$N,  N.r2=scores$N.r2)
  res
}

read.plink <- function(bed, bim, fam, na.strings=c("0", "-9"), sep=".",
                       select.subjects=NULL, select.snps=NULL) {
  lb <- nchar(bed);
  ext <- substr(bed, lb-3, lb);
  if (ext==".bed") {
    stub <- substr(bed, 1, lb-4)
  } else {
    stub <- bed
    bed <- paste(bed, ".bed", sep="")
  }
  if (missing(bim))
    bim <- paste(stub, ".bim", sep="")
  if (missing(fam))
    fam <- paste(stub, ".fam", sep="")
  df.bim <- read.table(bim, comment.char="", as.is=TRUE, na.strings=na.strings)
  snps <- as.character(df.bim[,2])
  if (!is.null(select.snps)) {
    if (is.character(select.snps)) {
      select.snps <- match(select.snps, snps)
      if (any(is.na(select.snps)))
        stop("unrecognised snp selected")
    }
    else if (is.numeric(select.snps)) {
      if (min(select.snps)<1 || max(select.snps)>nrow(df.bim))
        stop("selected snp subscript out of range")
      select.snps <- as.integer(select.snps)
    }
    else
      stop("unrecognised snp selection")
    if (any(duplicated(select.snps)))
      stop("snp selection contains duplicates")
    select.snps <- sort(select.snps)
    df.bim <- df.bim[select.snps,]
    snps <- snps[select.snps]
  }     
  if (ncol(df.bim)==6) {
    names(df.bim) <- c("chromosome", "snp.name", "cM", "position",
                       "allele.1", "allele.2")
  }
  else {
    warning("non-standard .bim file")
  }
  rownames(df.bim) <- snps
  if (any(duplicated(snps)))
    stop("duplicated snp name(s)")
  df.fam <- read.table(fam, comment.char="", as.is=TRUE, na.strings=na.strings)
  if (!is.null(select.subjects)) {
    if (is.numeric(select.subjects)) {
      if (min(select.snps)<1 || max(select.snps)>nrow(df.fam))
        stop("selected subject subscript out of range")
      select.subjects <- as.integer(select.subjects)
    }
    else
      stop("unrecognised subject selection")
    if (any(duplicated(select.subjects)))
      stop("subject selection contains duplicates")
    select.subjects <- sort(select.subjects)
    df.fam <- df.fam[select.subjects,]
  }     
  ped <- as.character(df.fam[,1])
  mem <- as.character(df.fam[,2])
  if (any(duplicated(ped))) {
    if (any(duplicated(mem))) {
      id <- paste(ped, mem, sep=sep)
      if (any(duplicated(id)))
        stop("couldn't create unique subject identifiers")
    }
    else
      id <- mem
  }
  else
    id <- ped
  names(df.fam) <-  c("pedigree", "member", "father", "mother", "sex",
                      "affected")
  rownames(df.fam) <- id
  gt <- .Call("readbed", bed, id, snps, select.subjects, select.snps,
              PACKAGE="snpStats")
  list(genotypes=gt, fam=df.fam, map=df.bim)
}

write.plink <- function(file.base, snp.major=TRUE, snps,
  subject.data, pedigree, id, father, mother, sex, phenotype,
  snp.data, chromosome, genetic.distance, position, allele.1, allele.2,
                        na.code=0, human.genome=TRUE) {
  
  mcall <- match.call()
  if (!is(snps, "SnpMatrix"))
    stop("snps argument must be of class SnpMatrix")
  X <- is(snps, "XSnpMatrix")
  nr.snps = nrow(snps)
  nc.snps = ncol(snps)
  if (missing(subject.data)) { ## subject data are in calling environment

    if (missing(pedigree))
      pedigree <- rownames(snps)
    else {
      pedigree <- as.character(pedigree)
      len <- length(pedigree)
      if (len != nr.snps)
        stop("length of `pedigree' argument incompatible with `snps'")
      if (any(is.na(pedigree)))
        stop("NA illegal for pedigree identifier")
    }
    
    if (missing(id))
      id <- rep("1", nr.snps)
    else {
      id <- as.character(id)
      if (length(id)!=nr.snps)
        stop("length of `id' argument  incompatible with `snps'")
      if (any(is.na(id)))
        stop("NA illegal for subject identifier within pedigree")
    }
    
    if (missing(father))
      father <- rep(na.code, nr.snps)
    else {
      father <- as.character(father)
      if (length(father)!=nr.snps)
        stop("length of `father' argument incompatible with `snps'")
      father[is.na(father)] = as.character(na.code)
    }
    
    if (missing(mother))
      mother <- rep(na.code, nr.snps)
    else {
      mother <- as.character(mother)
      if (length(mother)!=nr.snps)
        stop("length of `mother' argument incompatible with `snps'")
      mother[is.na(mother)] = as.character(na.code)
    }
    
    if (missing(sex)) {
      if (X)
        sex <- ifelse(snps@diploid, 2, 1)
      else
        sex <- rep(na.code, nr.snps)
    }
    else {
      sex <- as.numeric(sex)
      sex[is.na(sex)] = as.numeric(na.code)
    }

    if (missing(phenotype))
      phenotype <- rep(na.code, nr.snps)
    else {
      phenotype <- as.numeric(phenotype)
      phenotype[is.na(phenotype)] <- as.numeric(na.code)
    }
           
  } else { ## ped data are in data dataframe
    
    subject.data <- as.data.frame(subject.data)
    sord <- match(rownames(snps), rownames(subject.data))
    if (any(is.na(sord)))
      stop("missing entries in `subject.data' frame")

    if (missing(pedigree))
      pedigree <- rownames(snps)
    else {
      pedigree <- as.character(eval(mcall$pedigree, envir=subject.data))[sord]
      if (any(is.na(pedigree)))
        stop("NA illegal for pedigree identifier")
    }
    
    if (missing(id))
      id <-  rownames(snps)
    else {
      id <- as.character(eval(mcall$id, envir=subject.data))[sord]
      if (any(is.na(id)))
        stop("NA illegal for subject identifier within pedigree")
    }
    
    if (missing(father))
      father <- rep(na.code, nr.snps)
    else {
      father <- as.character(eval(mcall$father, envir=subject.data))[sord]
      father[is.na(father)] = as.character(na.code)
    }
    if (missing(mother))
      mother <- rep(na.code, nr.snps)
    else {
      mother <- as.character(eval(mcall$mother, envir=subject.data))[sord]
      mother[is.na(mother)] = as.character(na.code)
    }
    if (missing(sex))
      sex <- rep(na.code, nr.snps)
    else {
      sex <-  as.numeric(eval(mcall$sex, envir=subject.data))[sord]
      sex[is.na(sex)] = as.numeric(na.code)
    }

    if (missing(phenotype))
      phenotype <- rep(na.code, nr.snps)
    else {
      phenotype <- as.numeric(eval(mcall$phenotype, envir=subject.data))[sord]
      phenotype[is.na(phenotype)] = as.character(na.code)
    }
  }

  ## Map file
  
  if (missing(snp.data)) { ## snp data are in calling environment
    if (missing(chromosome)) {
      if (human.genome && X)
        chromosome <- rep("X", nc.snps)
      else
        chromosome <- rep(na.code, nc.snps)
    }
    else {
      len <- length(chromosome)
      if (human.genome) {
        if (is.integer(chromosome)) {
          if (any(chromosome>22 | chromosome<1))
              stop("Invalid chromosome number")
        }
        else {
          chr = match(as.character(chromosome),
                      c(as.character(1:22), "X", "Y", "XY", "MT"))
          if (any(is.na(chr)))
            stop("unrecognized chromosome name")
          if (X && any(chr!=23))
            stop("chromosome argument conflicts with snp data type")
        }
      }
      if (len==1) 
        chromosome <- rep(chromosome, nc.snps)
      else if (len!=nc.snps)
        stop("length of `chromosome' argument incompatible with `snps'")
    }

    if (missing(genetic.distance))
      genetic.distance <- rep(na.code, nc.snps)
    else {
      len <- length(genetic.distance)
      if (len==nc.snps)
        genetic.distance <- as.numeric(genetic.distance)
      else
        stop("length of `genetic.distance' argument incompatible and `snps'")
      genetic.distance[is.na(genetic.distance)] <- as.numeric(na.code)
    }
 
    if (missing(position))
      position <- rep(na.code, nc.snps)
    else {
      len <- length(position)
      if (len==nc.snps)
        position <- as.numeric(position)
      else
        stop("length of `position' argument incompatibe and `snps'")
      position[is.na(position)] <- as.numeric(na.code)
    }
 
    if (missing(allele.1))
      allele.1 <- rep(as.character(na.code), nc.snps)
    else {
      len <- length(allele.1)
      if (len==nc.snps)
        allele.1 <- as.character(allele.1)
      else
        stop("length of `allele.1' argument incompatibe and `snps'")
      allele.1[is.na(allele.1)] <- as.character(na.code)
    }
 
   if (missing(allele.2))
      allele.2 <- rep(as.character(na.code), nc.snps)
    else {
      len <- length(allele.2)
      if (len==nc.snps)
        allele.2 <- as.character(allele.2)
      else
        stop("length of `allele.2' argument incompatibe and `snps'")
      allele.2[is.na(allele.2)] <- as.character(na.code) 
    }
  } else { ## snp data are in data dataframe
    
    snp.data <- as.data.frame(snp.data)
    sord <- match(colnames(snps), rownames(snp.data))
    if (any(is.na(sord)))
      stop("missing entries in `snp.data' frame")

   if (missing(chromosome))
      chromosome <- rep(na.code, nc.snps)
    else {
      chromosome <- as.character(eval(mcall$chromosome, envir=snp.data))[sord]
      chromosome[is.na(chromosome)] <- as.character(na.code)
    }
    
    if (missing(genetic.distance))
      genetic.distance <- rep(na.code, nc.snps)
    else {
      genetic.distance <-
        as.numeric(eval(mcall$genetic.distance, envir=snp.data))[sord]
      genetic.distance[is.na(genetic.distance)] <- as.numeric(na.code)
    }
    
    if (missing(position))
      position <- rep(na.code, nc.snps)
    else {
      position <- as.numeric(eval(mcall$position, envir=snp.data))[sord]
      position[is.na(position)] <- as.numeric(na.code)
    }
    
    if (missing(allele.1))
      allele.1 <- rep(as.character(na.code), nc.snps)
    else {
      allele.1 <- as.character(eval(mcall$allele.1, envir=snp.data))[sord]
      allele.1[is.na(allele.1)] <- as.character(na.code)
    }
                                   
    if (missing(allele.2))
      allele.2 <- rep(as.character(na.code), nc.snps)
    else {
      allele.2 <- as.character(eval(mcall$allele.2, envir=snp.data))[sord]
      allele.2[is.na(allele.2)] <- as.character(na.code)
    }
  }

  ## Write files

  famdf <- data.frame(pedigree, id, father, mother, sex, phenotype)
  famfn <- paste(file.base, "fam", sep=".")
  cat("Writing FAM file to", famfn, "\n")  
  write.table(famdf, file=famfn, row.names=FALSE, col.names=FALSE,
              quote=FALSE, sep="\t")

  
  mapdf <- data.frame(chromosome, colnames(snps), genetic.distance,
                      position, allele.1, allele.2)
  mapfn <- paste(file.base, "bim", sep=".")
  cat("Writing extended MAP file to", mapfn, "\n")
  write.table(mapdf, file=mapfn, row.names=FALSE, col.names=FALSE,
              quote=FALSE, sep="\t")

  bedfn <- paste(file.base, "bed", sep=".")
  cat("Writing BED file to ", bedfn, " (", sep="")
  if (snp.major)
    cat("SNP-major mode)\n")
  else
    cat("Subject-major mode)\n")
  .Call("writebed", snps, bedfn, snp.major, PACKAGE="snpStats")
}

  

row.summary <- function(object) {
   if (inherits(object, "SnpMatrix"))
     .Call("row_summary", object, PACKAGE="snpStats")
   else
     stop("not a SnpMatrix object")
}

col.summary <- function(object, rules=NULL, uncertain=TRUE) {
   if (inherits(object, "SnpMatrix")) {
     if (inherits(object, "XSnpMatrix")) 
       .Call("X_snp_summary", object, rules, uncertain, PACKAGE="snpStats")
     else  
       .Call("snp_summary", object, rules, uncertain, PACKAGE="snpStats")
   }
   else
     stop("not a SnpMatrix object")
 }

Fst <- function(snps, group, pairwise=FALSE) {
  .Call("Fst", snps, as.factor(group), pairwise, PACKAGE="snpStats")
}

plotUncertainty <- function(snp, nlevels = 10,
                           color.palette = heat.colors(nlevels)) {
  codes <- factor(as.integer(snp), levels=1:253)
  freq <- table(codes)
  tot <- sum(freq, na.rm=TRUE)
  if (tot>0) {
    reord <- c(1, 253, 22, 2:21, 23:252)
    mass <- numeric(253)
    mass[reord] <- freq
    bym <- cut(mass, nlevels)
    h <- sqrt(3)/2
    d <- 1/(2*h)
    plot(c(0,21), c(0,23*h), type="n", xaxt="n", yaxt="n", xlab="", ylab="",
         bty="n", asp=1)
    lines(c(0, 21, 10.5, 0), c(0, 0, 21*h, 0))
    mtext("AA", side=1, line=0, at=0, adj=1, cex=1.5)
    mtext("BB", side=1, line=0, at=21, adj=0, cex=1.5)
    text(10.5, 22.3*h, "AB", cex=1.5, adj=0.5)
    ij <- 0
    for (i in 0:21) {
      yc <- i*h
      yll <- yc-d
      yl <- yc-0.5*d
      yh <- yc+0.5*d
      yhh <- yc+d
      xo <- i/2
      for (j in i:21) {
        ij <- ij+1
        xc <- j - xo
        xl <- xc-0.5   
        xr <- xc+0.5
        lev <- as.numeric(bym[ij])
        x <- c(xc, xl, xl, xc, xr, xr)
        y <- c(yll, yl, yh, yhh, yh, yl)
        mij <- mass[ij]
        if (mij>0) {
          polygon(x, y, col=color.palette[nlevels+1-lev], border=NA)
          text(xc, yc, as.character(mij), cex=0.6)
        }
      }
    }
  }
  else {
    "No data to plot"
  }
}
      
  
pp <- function(x, transpose=FALSE) {
  .Call("pp", as.raw(x), transpose, PACKAGE="snpStats")
}

mean2g <- function(m, maxE=FALSE) .Call("Rmean2g", m, maxE, PACKAGE="snpStats")

g2post <- function(g, transpose=FALSE) .Call("Rg2post", g, transpose,
                       PACKAGE="snpStats")

post2g <- function(p, transpose=FALSE) .Call("Rpost2g", p, transpose,
                       PACKAGE="snpStats")

 qq.chisq <-
  function(x, df=1, x.max,
    main="QQ plot", 
    sub=paste("Expected distribution: chi-squared (",df," df)", sep=""), 
    xlab="Expected", ylab="Observed",
    conc=c(0.025, 0.975), overdisp=FALSE, trim=0.5,  
    slope.one=FALSE, slope.lambda=FALSE, pvals=FALSE,
    thin=c(0.25,50), oor.pch=24, col.shade="gray", ...) {

    # Function to shade concentration band
    
    shade <- function(x1, y1, x2, y2, color=col.shade) {
      n <- length(x2)
      polygon(c(x1, x2[n:1]), c(y1, y2[n:1]), border=NA, col=color)
    }
    
    # Sort values and see how many out of range
    
    obsvd <- sort(x, na.last=NA)
    N <- length(obsvd)
    top <- obsvd[N]
    if (missing(x.max)) {
      Np <- N
    }
    else {
      Np <- sum(obsvd<=abs(x.max))
      if (Np<N || x.max<0) 
        top <- abs(x.max)
    }
    if(Np==0)
      stop("Nothing to plot")

    # Expected values
    
    if (df==2) {
      expctd <- 2*cumsum(1/(N:1))
    }
    else {
      expctd <- qchisq(p=(1:N)/(N+1), df=df)
    }

    # Concentration bands
    
    if (!is.null(conc)) {
      if(conc[1]>0) {
        e.low <- qchisq(p=qbeta(conc[1], 1:N, N:1), df=df)
      }
      else {
        e.low <- rep(0, N)
      }
      if (conc[2]<1) {
        e.high <- qchisq(p=qbeta(conc[2], 1:N, N:1), df=df)
      }
      else {
        e.high <- 1.1*rep(max(x),N)
      }
    }
    
    
    right <- expctd[N]
    par(mar=c(5,4,4,2)+0.1, las=1)
    if (pvals){
      mlp <- floor(-log10(pchisq(top, df=df, lower.tail=FALSE)))
      if (mlp>0) {
        gap <- ceiling(mlp/5)
        lp.vals <- seq(mlp, 1, -gap)
        chi2.vals <- qchisq(10^(-lp.vals), df=df, lower.tail=FALSE)
        par(mar=c(5,4,4,4)+0.1, las=1)
      }
      else 
        pvals <- FALSE
    }
    
    plot(c(0, right), c(0, top), type="n", xlab=xlab, ylab=ylab,
         main=main, sub=sub)

    # log p axis

    if (pvals) {
      nvals <- length(lp.vals)
      for (i in 1:nvals) 
        axis(side=4, at=chi2.vals[i],
             labels=substitute(10^{a}, list(a=-lp.vals[i])),  xaxt="n")
      mtext("P-value", side=4, line=3, las=0, padj=0)
    }
    
    
    # Thinning
    
    if (is.na(thin[1])) {
      show <- 1:Np
    }
    else if (length(thin)!=2 || thin[1]<0 || thin[1]>1 || thin[2]<1) {
      warning("invalid thin parameter; no thinning carried out")
      show <- 1:Np
    }
    else {
      space <- right*thin[1]/floor(thin[2])
      iat <- round((N+1)*pchisq(q=(1:floor(thin[2]))*space, df=df))
      if (max(iat)>thin[2]) 
        show <- unique(c(iat, (1+max(iat)):Np))
      else 
        show <- 1:Np
    }
    Nu <- floor(trim*N)
    if (Nu>0) 
      lambda <- mean(obsvd[1:Nu])/mean(expctd[1:Nu])
    if (!is.null(conc)) {
      if (Np<N)
        vert <- c(show, (Np+1):N)
      else
        vert <- show
      if (overdisp) 
        shade(expctd[vert], lambda*e.low[vert],
              expctd[vert], lambda*e.high[vert])
      else 
        shade(expctd[vert], e.low[vert], expctd[vert], e.high[vert])
    }
    points(expctd[show], obsvd[show], ...)
    # Overflow
    if (Np<N) {
      over <- (Np+1):N
      points(expctd[over], rep(top, N-Np), pch=oor.pch)
    }
    # Lines
    line.types <- c("solid", "dashed", "dotted")
    key <- NULL
    txt <- NULL
    if (slope.one) {
      key <- c(key, line.types[1])
      txt <- c(txt, "y = x")
      abline(a=0, b=1, lty=line.types[1])
    }
    if (slope.lambda && Nu>0) {
      key <- c(key, line.types[2])       
      txt <- c(txt, paste("y = ", format(lambda, digits=4), "x", sep=""))
      if (!is.null(conc)) {
        if (Np<N)
          vert <- c(show, (Np+1):N)
        else
          vert <- show
      }    
      abline(a=0, b=lambda, lty=line.types[2])
    }

    # Returned value
    
    if (!is.null(key)) 
       legend(0, top, legend=txt, lty=key)
    c(N=N, omitted=N-Np, lambda=lambda)

  }

    

# Routines for structure analyses

xxt <- function(snps, strata=NULL, correct.for.missing=FALSE,
                lower.only=FALSE, uncertain = FALSE) {
  if (!is.null(strata) && !is.factor(strata))
    strata <- as.factor(strata)
  .Call("xxt", snps, strata, correct.for.missing, lower.only, uncertain,
        PACKAGE="snpStats")
}

ibsCount <- function(snps, uncertain=FALSE) {
  .Call("ibs_count", snps, uncertain, PACKAGE="snpStats")
}

ibsDist <- function(counts) {
  .Call("ibs_dist", counts, PACKAGE="snpStats")
}

snp.pre.multiply <- function(snps,  mat, frequency=NULL, uncertain=FALSE ) {
  .Call("snp_pre", snps, mat, frequency, uncertain, PACKAGE="snpStats")
}

snp.post.multiply <- function(snps,  mat, frequency=NULL, uncertain=FALSE) {
  .Call("snp_post", snps, mat, frequency, uncertain, PACKAGE="snpStats")
}

snp.cor <- function(x, y, uncertain=FALSE) {
  .Call("corsm", x, as.matrix(y), uncertain, PACKAGE="snpStats")
}

snp.imputation<- function(X, Y, pos.X, pos.Y, phase=FALSE, try=50,
                          stopping=c(0.95, 4, 0.05),
                          use.hap = c(1.0, 0.0),
                          em.cntrl=c(50, 0.01, 10, .01),
                          minA=5) {
  same <- missing(Y)
  if (same) {
    Y <- X
    if (missing(pos.X))
      pos.X <- pos.Y <- 1:ncol(X)
  }
  if (phase)
    stop("phase=TRUE option not yet implemented")
  if (!(is(X, "SnpMatrix") && is(Y, "SnpMatrix")))
    stop("X and Y arguments must be of class SnpMatrix or XSnpMatrix")
  if (class(X)[1]!=class(Y)[1])
    stop("X and Y arguments must be of the same class")
  if (nrow(X)!=nrow(Y) || length(pos.X)!=ncol(X) || length(pos.Y)!=ncol(Y))
    stop("Inconsistent argument dimensions")
  order.X <- order(pos.X)
  order.Y <- order(pos.Y)
  if (try>ncol(X))
    try <- ncol(X)
  .Call("snp_impute", X, Y, order.X, order.Y,
        as.double(pos.X[order.X]), as.double(pos.Y[order.Y]),
        as.logical(phase), as.integer(try),
        stopping, use.hap, em.cntrl, as.double(minA), as.logical(same),
        PACKAGE="snpStats")
}

impute.snps <- function(rules, snps, subset=NULL, as.numeric=TRUE) {
  if (class(rules)!= "ImputationRules")
    stop("incorrect class for `rules' argument")
  if (class(snps)!= "SnpMatrix" && class(snps)!="XSnpMatrix")
    stop("incorrect class for `snps' argument")
  if (!is.null(subset)) {
    nr <- nrow(snps)
    if (is.logical(subset)) {
      if (length(subset)!=nr)
        stop("incompatible dimensions for snps and subset")
      subset <- (1:nr)[subset]
    }
    subset <- as.integer(subset);
    if (any((subset<1) | (subset>nr)))
      stop("row number out of range in subset argument")
  }
  .Call("impute_snps", rules, snps, subset, as.numeric, PACKAGE="snpStats")
}

# Returns names of rules (imputed snps) which feature excluded SNPs

filter.rules <- function(rules, snps.excluded, exclusions=TRUE) {
  if (class(rules)!= "ImputationRules")
    stop("incorrect class for `rules' argument")
  if (!is.character(snps.excluded))
    stop("excluded SNPs must be referenced by name")
  if.excl <- sapply(rules, FUN=function(rule, exc){
                             any(rule$snps %in% exc)
                           }, snps.excluded)
  if (exclusions)
    names(rules)[if.excl]
  else
    names(rules)[!if.excl]
}

# List extraction functions

can.impute <- function(rules) !sapply(rules, is.null)

imputation.maf <- function(rules) sapply(rules,
                    function(x) if (is.null(x$maf)) NA else x$maf)

imputation.r2 <- function(rules) sapply(rules,
                    function(x) if (is.null(x$r.squared)) NA else x$r.squared)

imputation.nsnp <- function(rules) sapply(rules,
                    function(x) if (is.null(x$snps)) NA else length(x$snps))

read.snps.long <-function(files, sample.id=NULL, snp.id=NULL, diploid=NULL,
                         fields=c(sample=1, snp=2, genotype=3, confidence=4),
                         codes=c("0", "1", "2"), threshold=0.9, lower=TRUE,
                         sep=" ", comment="#",  skip=0,
                         simplify=c(FALSE, FALSE),
                         verbose=FALSE, in.order=TRUE, every=1000) {
  if (any(duplicated(files)))
    stop("duplicated input file name(s)")
  if (any(duplicated(sample.id)))
    stop("duplicated target sample identifier(s)")
  if (any(duplicated(snp.id)))
    stop("duplicated target SNP identifier(s)")
  if (!is.null(diploid) && any(is.na(diploid)))
    stop("diploid argument contains one or more NA's")
  if (length(diploid)==1)
    diploid <- rep(diploid, length(sample.id))
  if (!is.integer(fields))
    mode(fields) <- "integer"
  .Call("insnp_new", files, sample.id, snp.id, diploid, fields,
        codes, threshold, lower, sep, comment, as.integer(skip),
        simplify, verbose, in.order, as.integer(every), PACKAGE="snpStats")
}

read.mach <- function(file, colnames=NULL, nrow=NULL) {
  .Call("read_mach", file, colnames, nrow, PACKAGE="snpStats")
}

read.impute <- function(file, rownames=NULL, nsnp=NULL, snpcol=2) {
  .Call("read_impute", file, rownames, nsnp, snpcol, FALSE, PACKAGE="snpStats")
}
read.beagle <- function(file, rownames=NULL, nsnp=NULL, header=TRUE) {
  .Call("read_impute", file, rownames, nsnp, 0, header, PACKAGE="snpStats")
}

random.snps <- function(nrows, ncols) {
    .Call("snp_big", as.integer(nrows), as.integer(ncols), PACKAGE="snpStats")
}

#
# Much duplicated code between tdt.snp() and misinherits()
# Could/should be tidied up

tdt.snp <- function(ped, id, father, mother, affected,
                         data=sys.parent(), snp.data, rules=NULL,
                         snp.subset=NULL,
                         check.inheritance=TRUE, robust=FALSE, uncertain=FALSE,
                         score=FALSE) {
  if (!is.null(rules) || !check.inheritance) 
    robust <- TRUE
  mcall <- match.call()
  if (!is(snp.data, "SnpMatrix"))
    stop("snp.data argument must be of class SnpMatrix")
  if (missing(data)) { # ped data are in calling environment
    ped <- as.character(ped)
    nped <- length(ped)
    if (nped != nrow(snp.data))
      stop("length of `ped' argument incompatible with `snp.data'")
    id <- as.character(id)
    if (length(id)!=nped)
      stop("incompatible length for `id' and `ped' arguments")
    father <- as.character(father)
    if (length(father)!=nped)
      stop("incompatible length for `father' and `ped' arguments")
    mother <- as.character(mother)
    if (length(mother)!=nped)
      stop("incompatible length for `mother' and `ped' arguments")
    affected <- as.logical(affected)
    if (length(affected)!=nped)
      stop("incompatible length for `affected' and `ped' arguments")
               
    # Correspondence between ped data and snp data

    subject.names <- rownames(snp.data)
    in.snp <- 1:nped
    have.snps <- rep(TRUE, nped)
    
  } else { # ped data are in data dataframe
    data <- as.data.frame(data)
    nped <- nrow(data)
    subject.names <- rownames(data)

    if (missing(ped))
      ped <- as.character(data[,1])
    else
      ped <- as.character(eval(mcall$ped, envir=data))
    if (is.null(ped))
      stop("pedigree identifiers not found in data frame")

    if (missing(id))
      id <- as.character(data[,2])
    else
      id <- as.character(eval(mcall$id, envir=data))
    if(is.null(id))
      stop("subject identifiers not found in data frame")   

    if (missing(father))
      father <- as.character(data[,3])
    else
      father <- as.character(eval(mcall$father, envir=data))
    if(is.null(father))
      stop("father identifiers not found in data frame")   
    
    if (missing(mother))
      mother <- as.character(data[,4])
    else
      mother <- as.character(eval(mcall$mother, envir=data))
    if(is.null(mother))
      stop("mother identifiers not found in data frame")   
    
    if (missing(affected))
      affected <- (data[,6]==2)
    else
      affected <- as.logical(eval(mcall$affected, envir=data))
    if(is.null(affected))
      stop("disease status not found in data frame")   
  
    # Correspondence between ped data and snp data

    in.snp <- match(subject.names, rownames(snp.data))
    have.snps <- !is.na(in.snp)
  }

  # Treat subjects with "affected" missing as not affected
  
  affected[is.na(affected)] <- FALSE


  # Father and mother locations in ped file

  s.unique <- paste(ped, id, sep=":")
  if (any(duplicated(s.unique)))
    warning("Combination of pedigree ID and ID within pedigree does not generate unique IDs")
  f.unique <-  paste(ped, father, sep=":")
  fpos <- match(f.unique, s.unique)
  m.unique <-  paste(ped, mother, sep=":")
  mpos <- match(m.unique, s.unique)

  # Potentially complete trios
  
  trio <- have.snps & affected & (!is.na(fpos)) & (have.snps[fpos]) &
                                 (!is.na(mpos)) & (have.snps[mpos])
  ntrio <- sum(trio, na.rm=TRUE)
  if (ntrio==0) {
    cat("No potentially complete trios to analyse\n")
    return(NULL)
  }
  pd.snps <- in.snp[trio] # Proband rows in SnpMatrix
  fr.snps <- in.snp[fpos[trio]] # Fathers' rows in SnpMatrix
  mr.snps <- in.snp[mpos[trio]] # Mothers' rows in SnpMatrix
  
  clust <-   as.integer(factor(ped[trio]))
  cord <- order(clust)
  cat("Analysing", ntrio, "potentially complete trios in", max(clust),
      "different pedigrees\n")

  # Calculate scores and score variances

  scores <- .Call("score_tdt", pd.snps[cord], fr.snps[cord], mr.snps[cord],
                  clust[cord], snp.data, rules, snp.subset,
                  check.inheritance, robust, uncertain, PACKAGE="snpStats")
  chisq <- .Call("chisq_single", scores, PACKAGE="snpStats")
  if (is.null(rules)) {
    if (is.null(snp.subset))
      tested <- colnames(snp.data)
    else
      tested <- colnames(snp.data)[snp.subset]
  } else {
    if (is.null(snp.subset))
      tested <- names(rules)
    else
      tested <- names(rules)[snp.subset]
  }
  if (score)
    res <- new("SingleSnpTestsScore", snp.names=tested,
               chisq=chisq, N=scores$N, N.r2=scores$N.r2,
               U=scores$U, V=scores$V)
  else
    res <- new("SingleSnpTests", snp.names=tested, chisq=chisq,
               N=scores$N,  N.r2=scores$N.r2)
  res
}

misinherits <- function(ped, id, father, mother, data=sys.parent(), snp.data){

  non.mendel <- !as.logical(c(
     1,1,1,1, 1,1,1,0, 1,1,1,1, 1,0,1,1,
     1,1,1,0, 1,1,0,0, 1,1,1,0, 1,0,1,0,
     1,1,1,1, 1,1,1,0, 1,1,1,1, 1,0,1,1,
     1,0,1,1, 1,0,1,0, 1,0,1,1, 1,0,0,1,
     1,1,0,1, 1,1,0,0, 1,1,0,1, 1,0,0,1))

  mcall <- match.call()
  if (!is(snp.data, "SnpMatrix"))
    stop("snp.data argument must be of class SnpMatrix")
  X <- is(snp.data, "XSnpMatrix")
  if (missing(data)) { # ped data are in calling environment
    ped <- as.character(ped)
    nped <- length(ped)
    if (nped != nrow(snp.data))
      stop("length of `ped' argument incompatible with `snp.data'")
    id <- as.character(id)
    if (length(id)!=nped)
      stop("incompatible length for `id' and `ped' arguments")
    father <- as.character(father)
    if (length(father)!=nped)
      stop("incompatible length for `father' and `ped' arguments")
    mother <- as.character(mother)
    if (length(mother)!=nped)
      stop("incompatible length for `mother' and `ped' arguments")
    subject.names <- id
           
    # Correspondence between ped data and snp data

    subject.names <- rownames(snp.data)
    in.snp <- 1:nped
    have.snps <- rep(TRUE, nped)
           
  } else { # ped data are in data dataframe
    data <- as.data.frame(data)
    nped <- nrow(data)
    subject.names <- rownames(data)
    if (missing(ped))
      ped <- as.character(data[,1])
    else
      ped <- as.character(eval(mcall$ped, envir=data))
    
    if (missing(id))
      id <- as.character(data[,2])
    else
      id <- as.character(eval(mcall$id, envir=data))
    
    if (missing(father))
      father <- as.character(data[,3])
    else
      father <- as.character(eval(mcall$father, envir=data))
    
    if (missing(mother))
      mother <- as.character(data[,4])
    else
      mother <- as.character(eval(mcall$mother, envir=data))

    # Correspondence between ped data and snp data

    in.snp <- match(subject.names, rownames(snp.data))
    have.snps <- !is.na(in.snp)
  }

  # Father and mother locations in ped file

  s.unique <- paste(ped, id, sep=":")
  if (any(duplicated(s.unique)))
    warning("Combination of pedigree ID and ID within pedigree does not generate unique IDs")
  f.unique <-  paste(ped, father, sep=":")
  fpos <- match(f.unique, s.unique)
  m.unique <-  paste(ped, mother, sep=":")
  mpos <- match(m.unique, s.unique)

  # Potentially complete trios
  
  trio <- have.snps & (!is.na(fpos)) & (have.snps[fpos]) &
                                 (!is.na(mpos)) & (have.snps[mpos])
  ntrio <- sum(trio)
  if (ntrio==0) {
    cat("No potentially complete trios to analyse\n")
    return(NULL)
  }
  pd.snps <- in.snp[trio] # Proband rows in SnpMatrix
  fr.snps <- in.snp[fpos[trio]] # Fathers' rows in SnpMatrix
  mr.snps <- in.snp[mpos[trio]] # Mothers' rows in SnpMatrix

  fr.raw <- as.raw(snp.data[fr.snps,])
  if (X)
    fr.raw[!snp.data@diploid[pd.snps]] <- as.raw(4)
  mr.raw <- as.raw(snp.data[mr.snps,])
  pd.raw <- as.raw(snp.data[pd.snps,])
  ## Treat any uncertain genotypes as missing 
  fr.raw[fr.raw>3] <- as.raw(0)
  mr.raw[mr.raw>3] <- as.raw(0)
  pd.raw[pd.raw>3] <- as.raw(0)
  code <- 1 + as.numeric(
                rawShift(fr.raw, 4) |
                rawShift(mr.raw, 2) |
                pd.raw)
  error <- non.mendel[code]
  error[is.na(snp.data[pd.snps,])] <- NA
  error <- matrix(error, nrow=ntrio,
                  dimnames=list(subject.names[trio], colnames(snp.data)))
  erows <- apply(error, 1, any, na.rm=TRUE)
  ecols <- apply(error, 2, any, na.rm=TRUE)
  error[erows, ecols, drop=FALSE]
}

.guessPloidy <- function(snps, known.diploid=NULL) {
  het <- row.summary(snps)$Heterozygosity
  N <- length(het)
  if (is.null(known.diploid)) {
    to.guess <- rep(TRUE, N)
    known.diploid <- logical(N)
  }
  else 
    to.guess <- is.na(known.diploid)
  dip <- ifelse(to.guess, het>0.1, known.diploid)
  repeat {
    diploid <- dip
    hp <- mean(het[diploid], na.rm=TRUE)
    dp <- mean(het[!diploid], na.rm=TRUE)
    if (is.na(dp) || is.na(hp))
      break
    hd <- (het-hp)^2/(hp*(1-hp))
    dd <- (het-dp)^2/((dp+0.001)*(1-dp))
    dip <- ifelse(to.guess, hd<dd, known.diploid)
    if (all(dip==diploid, na.rm=T))
      break
  }
  list(diploid=diploid, Heterozygosity=het)
}

.forceHom <- function(xsnps, diploid) {
  .Call("force_hom", xsnps, diploid, PACKAGE="snpStats")
}

ld <- function(x, y=NULL, depth=NULL, stats, symmetric=FALSE) {
  if (missing(stats))
    stop("LD stats must be specified")
  cstats <- c("LLR", "OR", "Q", "Covar", "D.prime", "R.squared", "R")%in%stats
  if (!any(cstats))
    stop("nothing to calculate")
  if (is.null(y)) {
    if (is.null(depth))
      stop("depth argument must be supplied")
    dmax <- ncol(x)-1
    if (depth>dmax) {
      depth <- as.integer(dmax)
      warning("depth too large; it has been reset to ", dmax)
    }
  }
  .Call("ld", x, y, as.integer(depth), cstats, symmetric, PACKAGE="snpStats");
}

read.pedfile <- function(file, n, snps, which, split="\t| +", sep=".",
                        na.strings="0", lex.order=FALSE) {
  ## Constants
  r0 <- as.raw(0)
  r1 <- as.raw(1)
  r2 <- as.raw(2)
  r3 <- as.raw(3)
  ## Input file
  con <- gzfile(file)
  open(con)
  ## If no line count, find out
  if (missing(n)) {
    n <- 0
    repeat {
      line <- readLines(con, n=1)
      if (length(line)==0) break;
      n  <- n+1;
    }
    if (n==0)
      stop("Nothing read")
    seek(con, 0)
  }
  ## Find snp names
  gen <- missing(snps)
  map <- NULL
  if (!gen) {
    m <- length(snps)
    if (m==1) {
      map <- read.table(snps, comment.char="")
      m <- nrow(map)
      if (missing(which)) {
        which <- 1
        repeat {
          snps <- map[,which]
          if (!any(duplicated(snps)))
            break
          if (which==ncol(map))
            stop("No unambiguous snp names found on file")
          which <- which+1
        }
      }
      else {
        snps <- map[,which]
      }
    }
  }
  else {
    line <- readLines(con, n=1)
    fields <- strsplit(line, split)[[1]]
    if (m%%2!=0)
      stop("Odd number of fields")
    m <- (length(fields) - 6)/2
    seek(con, 0)
  }
  nf <- 6+2*m
  ## Generate empty matrix
  result <- matrix(raw(n*m), nrow=n)
  ## Columns of subject dataframe
  ped <- character(n)
  mem <- character(n)
  pa <- character(n)
  ma <- character(n)
  sex <- numeric(n)
  aff <- numeric(n)
  rownms <- character(n)
  a1 <- a2 <- rep(NA, m)
  a1m <- a2m <- rep(TRUE, m)
  mallelic <- rep(FALSE, m) ## Multiallelic?
  for (i in 1:n) {
    line <- readLines(con, n=1)
    fields <- strsplit(line, "\t| +")[[1]]
    to.na <- fields %in% na.strings
    fields[to.na] <- NA
    ped[i] <- fields[1]
    mem[i] <- fields[2]
    pa[i] <- fields[3]
    ma[i] <- fields[4]
    sex[i] <- as.numeric(fields[5])
    aff[i] <- as.numeric(fields[6])
    alleles <- matrix(fields[7:nf], byrow=TRUE, ncol=2)
    one <- two <- rep(FALSE, m)
    for (k in 1:2) {
      ak <- alleles[,k]
      akm <- is.na(ak) 
      br1 <- !akm & a1m
      a1[br1] <- ak[br1]
      a1m[br1] <- FALSE
      br2 <- !akm & (a1==ak)
      one[br2] <- TRUE
      br3 <- !akm & !a1m & (a1!=ak)
      br4 <- br3 & a2m
      a2[br4] <- ak[br4]
      a2m[br4] <- FALSE
      br5 <- br3 & (a2==ak)
      two[br5] <- TRUE
      mallelic <- mallelic | !(akm|one|two)
    }
    gt <- rep(r0, m)
    gt[one&!two] <- r1
    gt[one&two] <- r2
    gt[two&!one] <- r3
    result[i,] <- gt
  }
  close(con)
  ## Warnin messages
  if (any(a1m))
    warning("no data for ", sum(a1m), " loci")
  mono <-(a2m & !a1m) 
  if (any(mono))
    warning(sum(mono), " loci were monomorphic")
  if (any(mallelic)) {
    result[,mallelic] <- r0;
    warning(sum(mallelic), " loci were multi-allelic --- set to NA")
  }
  ## SnpMatrix result
  if (gen)
    snps <- paste("locus", 1:m, sep=sep)
  if (any(duplicated(ped))) {
    if (any(duplicated(mem))) {
      rnames <- paste(ped, mem, sep=sep)
      if (any(duplicated(rnames)))
        stop("could not create unique subject identifiers")
    }
    else 
      rnames <- mem
  } 
  else
    rnames <- ped
  dimnames(result) <- list(rnames, snps)
  result <- new("SnpMatrix", result)
  ## Switch alleles if requested and necessary
  if (lex.order) {
    swa<- (!(is.na(a1)|is.na(a2)) & (a1>a2))
    switch.alleles(result, swa)
    a1n <- a1
    a1n[swa] <- a2[swa]
    a2[swa] <- a1[swa]
    a1 <- a1n
  }
  ## Subject support file
  fam <- data.frame(row.names=rnames, pedigree=ped, member=mem,
                    father=pa, mother=ma, sex=sex, affected=aff)
  ## map data frame 
  if (is.null(map))
    map <- data.frame(row.names=snps, snp.name=snps,
                      allele.1=a1, allele.2=a2)
  else {
    map$allele.1 <- a1
    map$allele.2 <- a2
    names(map)[which] <- "snp.names"
  }
  list(genotypes=result, fam=fam, map=map)    
}

  

read.long <-
function(file, samples, snps,
         fields = c(snp = 1, sample = 2, genotype = 3, confidence = 4,
           allele.A=NA, allele.B=NA),
         split="\t| +", gcodes, no.call="", threshold=NULL,
         lex.order=FALSE, verbose=FALSE) {
  ## Number of ignored lines to list if verbose operation
  list.ignore <- 50
  ## Input file(s)
  if (length(file)>1) {
    if (missing(samples) || missing(snps))
      stop("When input is from multiple files, sample and snp arguments must be supplied")
    if (verbose)
      cat("Data to be read from multiple files", file[1], "...\n")
    con <- file(file[1], open="rt")
  } else {
    if (verbose)
      cat("Data to be read from the file", file, "\n")
    con <- file(file, open="rt")
  }
  ifile <- 1
  ## Sample and snp fields
  sampf <- fields["sample"]
  snpf <- fields["snp"]
  ## Size
  nsamp <- 0
  nsnp <- 0
  if (!missing(samples)) {
    if (is.character(samples)) {
      nsamp <- length(samples)
      if (verbose)
        cat("Names of", nsamp, "samples specified\n")
    }
    else if (is.numeric(samples) && length(samples)==1) {
      nsamp <- samples
      samples <- NULL
      if (verbose)
        cat("Number of samples to be read:", nsamp, "(names unspecified)\n")
    }
    else
      stop("illegal argument: samples")
  } else {
    samples <- NULL
  }
  if (!missing(snps)) {
    if (is.character(snps)) {
      nsnp <- length(snps)
      if (verbose)
        cat("Names of", nsnp, "snps specified\n")
    }    
    else if (is.numeric(snps) && length(snps)==1) {
      nsnp <- snps
      snps <- NULL
      if (verbose)
        cat("Number of snps to be read:", nsnp, "(names unspecified)\n")
    }
    else
      stop("illegal argument: snps")
  } else {
    snps <- NULL
  }

  ## Remaining argument checks
  
  cf <- fields["confidence"]
  if (is.null(threshold)) {
    if (!is.na(cf))
      stop("confidence field read but no thresholds set")
    if (verbose)
      cat("No confidence thresholds specified\n")
  } else {
    if (is.na(cf))
      stop("thresholds set but no confidence field read")
    low <- threshold[1]
    high <- threshold[2]
    if (verbose) {
      cat("Call rejected if confidence score ")
      if (!is.na(low))
        cat("is less than", low)
      if (length(threshold)>1)
        cat(" or ")
      if (!is.na(high))
        cat("is greater than", high)
      cat("\n")
    }       
  }
  af1 <- fields["allele.A"]
  af2 <- fields["allele.B"]
  gf <- fields["genotype"]
  if ((is.na(af1)+is.na(af2))==1)
    stop("only one allele field specified in fields argument")
  aread <- !is.na(af1) && !is.na(af2)
  gread <- !is.na(gf)
  if (aread) {
    if (gread)
      stop("both genotype and allele fields specified")
    if (missing(gcodes))
      gcodes <- NULL
    else
      stop("gcodes argument set when data are read as alleles")
    if (verbose)
      cat("Alleles are read from two different fields\n")
  } else {
    if (!gread)
      stop("neither allele or genotype fields specified")
    if (verbose)
      cat("Genotype read as a single field ")
    if (missing(gcodes)) {
      ## 2-character allele coding
      if (verbose)
        cat("of two characters (which specify the alleles)\n")
      gcodes <- NULL
    } else {
      if (length(gcodes)==3) {
        ## genotype coding
        gcodes <- as.character(gcodes)
        if (verbose)
          cat("matching one of", gcodes, "\n")
      } else {
        ## single field to be split in two?
        if (!is.character(gcodes) || length(gcodes)!=1)
          stop("invalid value passed for gcodes argument")
        else if (verbose)
          cat("to be split into alleles by the regexp", gcodes, "\n")
      }
    }	
  } 

  ## If missing size, read ahead

  if (nsamp==0 || nsnp==0) {
    if (verbose)
      cat("Initial scan of file\n")
    last.samp <- ""
    last.snp <- ""
    nline <- 0
    repeat {
      line <- readLines(con, n=1)
      if (length(line)==0 || nchar(line)==0) {
        if (nsamp==0) {
          nsamp <- length(samples)
          if (verbose)
            cat("Last sample:", last.samp, "\n")
        }
        if (nsnp==0) {
          nsnp <- length(snps)
          if (verbose)
            cat("Last snp:", last.snp, "\n")
        }
        break
      }
      nline <- nline+1
      fields <- strsplit(line, split)[[1]]
      this.samp <- fields[sampf]
      if (is.na(this.samp) || this.samp=="")
        stop("empty or NA sample field at line ", nline) 
      if (nsamp==0 && this.samp!=last.samp) {
        if (this.samp %in% samples) {
          if (verbose)
            cat("Last sample:", last.samp, "\n")
          nsamp = length(samples) ## Finished finding new sample names 
        } else {
          if (verbose && is.null(samples))
            cat("First sample:", this.samp, "\n")
          samples <- c(samples, this.samp)
          last.samp <- this.samp
        }
      }
      this.snp <- fields[snpf]
      if (is.na(this.snp) || this.snp=="")
        stop("empty or NA snp field at line ", nline)
      if (nsnp==0 && this.snp!=last.snp) {
        if (this.snp %in% snps) {
          if (verbose)
            cat("Last snp:", last.snp, "\n")
          nsnp = length(snps) ## Finished finding new snp names
        } else {
          if (verbose && is.null(snps))
            cat("First snp:", this.snp, "\n")
          snps <- c(snps, this.snp)
          last.snp <- this.snp
        }
      }
      if (nsamp>0 && nsnp>0)
        break;
    }
    if (nsamp==0 || nsnp==0)
      stop("Nothing read")
    seek(con, 0)  
  }
  
  ## Read data

  if (verbose)
    cat(nsamp, "x", nsnp, " matrix to be read\n", sep="")
  size <- nsamp*nsnp
  if (size>(2^31-1))
    stop("This is larger than the maximum size of a single object in R")
  genotypes <- matrix(as.raw(0), nrow=nsamp, ncol=nsnp)

  if (length(gcodes)==3)
    alleles <- NULL
  else
    alleles <- vector("list", nsnp)
  n.missing <- 0
  n.ignore <- 0
  n.reject <- 0
  n.found <- 0
  last.samp <- ""
  last.snp <- ""
  nline <- 0

  if (verbose) {
    cat("Reading genotypes from file\n ")
    for (i in 1:5) cat("       ", 2*i, "0%", sep="")
    cat("\n")
    for (i in 1:5) cat(".........|")
    cat("\n")
    bite <- size/50
    writ <- 0
  }
  
  while(n.found<size) {

    line <- readLines(con, n=1)
    if (length(line)==0 || nchar(line)==0) {
      if (ifile==length(file)) {
        break;
      } else {
        ifile <- ifile+1
        close(con)
        con <- file(file[ifile], open="rt")
        line <- readLines(con, n=1)
        if (length(line)==0 || nchar(line)==0)
          stop("Empty file: ", file[ifile])
      }
    }
    nline <- nline+1
    fields <- strsplit(line, split)[[1]]

    ## Sample
    
    this.samp <- fields[sampf]
    if (this.samp=="")
      stop("sample field empty at line ", nline)
    if (this.samp != last.samp) {
      i <- match(this.samp, samples)
      if (is.na(i)) {
        if (length(samples)<nsamp) {
          samples <- c(samples, this.samp)
          i <- length(samples)
        } else {
          n.ignore <- n.ignore+1
          next
        }
      }
      last.samp <- this.samp
    }

    ## Snp
    
    this.snp <- fields[snpf]
    if (this.snp=="")
      stop("snp field empty at line ", nline)
    if (this.snp != last.snp) {
      j <- match(this.snp, snps)
      if (is.na(j)) {
        if (length(snps)<nsnp) {
          snps <- c(snps, this.snp)
          j <- length(snps)
        } else {
          n.ignore <- n.ignore+1
          next
        }
      }
      last.snp <- this.snp
    }

    n.found <- n.found+1
    if (verbose && n.found>writ) {
      cat("-")
      writ <- writ+bite
    }
    ## Confidence filter

    if (!is.na(cf)) {
      fc <- fields[cf]
      if ((!is.na(low) && fc<low) || (!is.na(high) && fc>high)) {
        n.reject <- n.reject+1
        next
      }
    }

    ## Get genotype
    
    if (aread) {
      a12 <- c(fields[af1], fields[af2])
      if (a12[1]==no.call || a12[2]==no.call) {
        n.missing <- n.missing+1
        next
      }
    } else {
      fg <- fields[gf]
      if (fg==no.call) {
        n.missing <- n.missing+1
        next
      }
      if (is.null(gcodes)) {
        if (nchar(fg)!=2)
          stop("at line ", nline, ": ", fg,
               " (expecting a 2-character genotype field)")
        a12 <- c(substr(fg, 1, 1), substr(fg, 2, 2))
      } else if (length(gcodes)==3) {
        gtype <- match(fg, gcodes)
        genotypes[i,j] <- as.raw(gtype)
        next
      } else {
        a12 <- strsplit(fg, gcodes[1])
      }
    }
    aj <- alleles[[j]]
    if (length(aj)<2) {
      aj <- union(aj, a12)
      if (length(aj)>2)
        stop("on sample ", i, " snp ",  j, ":  not a SNP")
      alleles[[j]] <- aj
    }
    ia <- match(a12, aj)
    if (is.na(ia[1]) || is.na(ia[2]))
      stop("on sample ", i, " snp ",  j, ":  not a SNP")
    genotypes[i,j] <- as.raw(ia[1]+ia[2]-1)
  }
  if (verbose)
    cat("\n")
  close(con)
        
  if (n.ignore>0)
    warning(n.ignore, " lines were ignored")
  if (n.missing>0)
    warning(n.missing, " genotypes were not called")
  if (n.reject>0)
    warning(n.reject, " genotype calls did not pass confidence thresholds")
  
  ## Check that array is full
  
  msamp <- length(samples)
  msnp <- length(snps)

  if (msamp<nsamp || msnp<nsnp) {
    warning("Matrix was not filled; ", msamp, "x", nsnp, " matrix returned")
    genotypes <- genotypes[1:msamp, 1:msnp]
  }

  ## Make SnpMatrix object
  
  dimnames(genotypes) <- list(samples, snps)
  genotypes <- new("SnpMatrix", genotypes)

  ## Alleles frame, switching if necessary
  
  if (is.null(alleles)) {
    if (verbose)
      cat("Returning a SnpMatrix object\n")
    return(genotypes)
  } else {
    if (verbose)
      cat("Returning a list containing a SnpMatrix object and a dataframe listing the alleles\n")
    extract <- function(x, i) {
      if (is.null(x) || length(x)<i)
        return(as.character(NA))
      else
        return(x[i])
    }
    aA <- vapply(alleles, extract, character(1), 1)
    aB <- vapply(alleles, extract, character(1), 2)
    unseen <- is.na(aA) & is.na(aB)
    mono <- is.na(aB) & !unseen
    if (any(unseen))
      warning(sum(unseen), " snps had no acceptable calls")
    if (any(mono))
      warning(sum(mono), " snps were monomorphic")
    if (lex.order) {
      swa<- (!(is.na(aA)|is.na(aB)) & (aA>aB))
      if (any(swa)) {
        genotypes <- switch.alleles(genotypes, swa)
        aAsw <- aA[swa]
        aA[swa] <- aB[swa]
        aB[swa] <- aAsw
      }
    }
    alleles <- data.frame(allele.A=aA, allele.B=aB,
                          row.names=snps)
    return(list(genotypes=genotypes, alleles=alleles))
  }
}

#setOldClass(c("haplotype", "genotype"), test=TRUE)

#content should be raw - TODO: put restriction 
setClass("SnpMatrix", contains="matrix")

setClass("XSnpMatrix", representation("SnpMatrix", diploid="logical"),
         contains="SnpMatrix")
 
# constructor 
setMethod("initialize", 
          "SnpMatrix",
          function(.Object, ...) {
            if(is.null(.Object)) {
              stop("object is null to constructor");
            }
            .Object <- callNextMethod()
            # Please don't remove the tests - they are to do with promises and not redundant.
            # The accessor methods transparently passes to .Data, the assignment methods are different...
            if(!is.null(dim(.Object)) && (dim(.Object)[1] > 0) && (dim(.Object)[2] > 0)) {
              if (mode(.Object) != "raw") {
                cat("coercing object of mode ", mode(.Object), " to SnpMatrix\n")
                mode(.Object@.Data) <- "raw"
              }
              if (is.null(dimnames(.Object))) {
                cat("object has no names - using numeric order for row/column names\n")
                dimnames(.Object@.Data) <- c(list(as.character(1:(dim(.Object)[1]))), list(as.character(1:(dim(.Object)[2]))))
              }
            }
            .Object
          })

setMethod("initialize", 
          "XSnpMatrix", 
          function(.Object, ...) {
            .Object <- callNextMethod()
            if (length(.Object@diploid)==0){
              warning("diploid slot not supplied; it has been estimated from heterozygosity")
              guess <- .guessPloidy(.Object)
              .Object@diploid <- guess$diploid
              het <- guess$Heterozygosity
            }
            else {
              lend <- length(.Object@diploid)
              if (lend==1)
                .Object@diploid <- rep(.Object@diploid, nrow(.Object))
              else if (lend!=nrow(.Object))
                stop("diploid argument incorrect length")
              dpna <- is.na(.Object@diploid)
              if (any(dpna)) {
                warning("diploid argument contains NAs; these have been replaced by guesses based on heterozygosity")
                guess <- .guessPloidy(.Object, .Object@diploid)
                .Object@diploid <- guess$diploid
                het <- guess$Heterozygosity
              }
              else
                het <- row.summary(.Object)$Heterozygosity               
            }
            hetm <- !.Object@diploid & (!is.na(het)&(het>0))
            if (any(hetm)){
              warning(sum(hetm, na.rm=TRUE),
                      " heterozygous calls for haploid genotypes; these were set to NA")
              .Object@.Data <- .forceHom(.Object@.Data, .Object@diploid)
            }
            .Object
          })

setMethod("[", signature(x="SnpMatrix",i="ANY",j="ANY",drop="ANY"),
          function(x, i, j, drop=FALSE) {
            if (drop!=FALSE)
              stop("dimensions cannot be dropped from a SnpMatrix object")
            
            if (missing(i)) i <- integer(0)
            else if (is.character(i)) {
              i <- match(i, rownames(x))
              if (any(is.na(i)))
                stop("No match for one or more row selections")
            }
            else if (is.numeric(i)) {
              if (any(i<0)) 
                  i <- (1:nrow(x))[i]
              if (min(i)<1 || max(i)>nrow(x)) 
                stop("One or more row selections out of range")
            }
            else if (is.logical(i)) {
              if (length(i)!=nrow(x))
                stop("logical row selection vector is wrong length")
              i <- (1:nrow(x))[i]
            }
            else
              stop("Illegal type for row selection, i")
            i <- as.integer(i)
            if (any(is.na(i)))
              stop("NAs in row selection vector")
            
            if (missing(j)) j <- integer(0)
            else if (is.character(j)) {
              j <- match(j, colnames(x))
              if (any(is.na(j)))
                stop("No match for one or more column selections")
            }
            else if (is.numeric(j)) {
              if (any(j<0))
                j <- (1:ncol(x))[j]
              if (min(j)<1 || max(j)>ncol(x)) 
                stop("One or more column selections out of range")
            }
            else if (is.logical(j)) {
              if (length(j)!=ncol(x))
                stop("logical column selection vector is wrong length")
              j <- (1:ncol(x))[j]
            }
            else
              stop("Illegal type for column selection, j")
            j <- as.integer(j)
            if (any(is.na(j)))
              stop("NAs in column selection vector")
            
            .Call("subset", x, i, j, PACKAGE="snpStats")
          }
)

setMethod("[", signature(x="XSnpMatrix",i="ANY",j="ANY",drop="ANY"),
          function(x, i, j, drop=FALSE) {
            if (drop!=FALSE)
              stop("dimensions cannot be dropped from an XSnpMatrix object")  
            callNextMethod()
           })

# The sub-assignment methods

setMethod("[<-", signature(x="XSnpMatrix",i="ANY",j="ANY",value="XSnpMatrix"),
          function(x, i, j, value) {
            # The raw sub assignment should *just work*:
            if (!missing(i) & !missing(j)) {
              x@.Data[i,j] <- value
            } else if (missing(i) & missing(j)) {
              x@.Data[,] <- value
            } else if (missing(i)) {
              x@.Data[,j] <- value
            } else if (missing(j)) {
              x@.Data[i,] <- value
            }
            # All we want to do is to shuffle the rows
            # if a row index is passed
            if(!missing(i)) {
              slot(x, "diploid")[i] <- slot(value, "diploid")
            }
            x
          })

setAs("SnpMatrix", "numeric",
      function(from) {
        .Call("asnum", from, PACKAGE="snpStats")
      })

setAs("SnpMatrix", "character",
      function(from) {
        df <- dim(from)
        dnames <- dimnames(from)
        from <- 1+as.integer(from)
        to <- ifelse(from<5, c("NA", "A/A", "A/B", "B/B")[from], "Uncertain")
        dim(to) <- df
        dimnames(to) <- dnames
        to
      })

setAs("SnpMatrix", "XSnpMatrix",
      function(from) {
        new("XSnpMatrix", from)
      })

setAs("XSnpMatrix", "character",
      function(from) {
        df <- dim(from)
        ifr <- 1 + as.integer(from)
        offset <-  4*rep(from@diploid, ncol(from))
        to <- ifelse(ifr<5, c("NA", "A", "Error", "B",
                               "NA", "A/A", "A/B", "B/B")[ifr + offset],
                     "Uncertain")
        dim(to) <- df
        dimnames(to) <- dimnames(from)
        to
      })

setAs("matrix", "SnpMatrix",
      function(from) {
        if (is.numeric(from)) {
          valid  <- (from==0 | from==1 | from==2)
          if(!all(is.na(from) | valid)) 
            warning("values other than 0, 1 or 2 set to NA")
          to <- from+1
          to[is.na(from)|!valid] <- 0
          mode(to) <- "raw"
        }
        else if (is.raw(from)) {
          not.valid  <- (from>3)
          if (any(not.valid))
            warning("values other than 01, 02 or 03 set to NA (00)")
          to <- from
          to[not.valid] <- as.raw(0)
        }
        else 
          stop("Can only convert `numeric' or `raw' matrices") 
        new("SnpMatrix", to)
      })
           
# Summary of a SnpMatrix

setGeneric("summary")

setMethod("summary", "SnpMatrix",
   function(object) {
     list(rows = summary(row.summary(object)),
          cols = summary(col.summary(object, uncertain=TRUE)))
   })

setMethod("summary", "XSnpMatrix",
   function(object) {
     tab <- table(object@diploid)
     names(tab) <- ifelse(as.logical(names(tab)), "diploid", "haploid")
     list(sex = tab,
          rows = summary(row.summary(object)),
          cols = summary(col.summary(object, uncertain=TRUE)))
   })

# Imputation

setClass("ImputationRules", contains="list")

setMethod("summary", "ImputationRules",
   function(object) {
     info <- .Call("r2_impute", object, PACKAGE="snpStats")
     i2 <- info[,2]
     levs <- sort(unique(i2))
     labs <- paste(levs, "tags")
     size <- factor(i2, levels= levs, labels=labs)
     r2 <- info[,1]
     r2[r2>1] <- 1
     byr2 <- cut(r2, c((0:9)/10, 0.95, 0.99, 1.0), right=FALSE,
                 include.lowest=TRUE)
     table(byr2, size, dnn=c("R-squared", "SNPs used"), useNA="ifany")
   })

setMethod("[",
       signature(x="ImputationRules", i="ANY", j="missing", drop="missing"),
       function(x, i){
         if (is.character(i))
           i <- match(i, names(x), nomatch=0)
         ss <- x@.Data[i]
         names(ss) <- names(x)[i]
         res <- new("ImputationRules", ss)
         attr(res, "Max.predictors") <-
           max(sapply(res, function(rule) length(rule$snps)))
         res
       })


# Show and plot methods

setMethod("show", "ImputationRules",
   function(object) {
     to <- names(object)
     for (i in 1:length(object)) {
       if (is.null(object[[i]]$snps))
         cat(to[i], "~ No imputation available\n")
       else {
         if (is.null(object[[i]]$hap.probs)) 
           stop("Old imputation rule - not supported by this version")
         else 
           cat(to[i], " ~ ", paste(object[[i]]$snps, collapse="+"))
         cat(" (MAF = ", object[[i]]$maf, 
             ", R-squared = ", object[[i]]$r.squared,
             ")\n", sep="")
       }
     }
   })

setMethod("plot", signature(x="ImputationRules", y="missing"),
   function(x, y, ...) {
     mat <- summary(x)
     n <- nrow(mat)
     m <- ncol(mat)
     if (is.na(rownames(mat)[n])) {
       mat <- mat[-n,-m]
       n <- n-1
       m <- m-1
     }
     val <- barplot(t(mat[n:1,]), legend.text=TRUE, 
                    beside=F, col=heat.colors(m),
                    xlab=expression(r^2), ylab="Number of imputed SNPs",
                    names.arg=rep("", n), cex.lab=1.3, ...)
     mtext(rownames(mat)[n:1], at=val, side=1, las=2, line=0.5, cex=0.8)
    })

setMethod("show", "XSnpMatrix",
   function(object) {
     nr <- nrow(object)
     nc <- ncol(object) 
     dip <- object@diploid
     cat("An XSnpMatrix with ", nr, " rows (", sum(!dip), " haploid and ",
         sum(dip), " diploid), and ", nc, " columns\n", sep="")
     if (nr>1)
       cat("Row names: ", rownames(object)[1],"...", rownames(object)[nr],"\n")
     else
       cat("Row name: ", rownames(object)[1], "\n")
     if (nc>1)
       cat("Col names: ", colnames(object)[1],"...", colnames(object)[nc],"\n")
     else
       cat("Col name: ", colnames(object)[1],"\n")
       
   })

setMethod("show", "SnpMatrix",
   function(object) {
     nr <- nrow(object)
     nc <- ncol(object) 
     cat("A SnpMatrix with ", nr, "rows and ", nc,
         "columns\n")
     if (nr>1)
       cat("Row names: ", rownames(object)[1],"...", rownames(object)[nr],"\n")
     else
       cat("Row name: ", rownames(object)[1],"\n")
     if (nc>1)
       cat("Col names: ", colnames(object)[1],"...", colnames(object)[nc],"\n")
     else
       cat("Col name: ", colnames(object)[1],"\n")
   })


setMethod("is.na", "SnpMatrix", function(x){ x==0})

snp.rbind <- function(...){
  .External("snp_rbind", ..., PACKAGE="snpStats")
}
snp.cbind <- function(...){
  .External("snp_cbind", ..., PACKAGE="snpStats")
}

setMethod("rbind", signature(...="SnpMatrix"), snp.rbind)
setMethod("cbind", signature(...="SnpMatrix"), snp.cbind)



# Tests 

setClass("SingleSnpTests", 
         representation(snp.names="character", chisq="matrix",
                        N="integer", N.r2="numeric"))
setClass("SingleSnpTestsScore", 
         representation("SingleSnpTests", U="matrix", V="matrix"),
         contains="SingleSnpTests")
setClass("GlmTests",
         representation(snp.names="ANY", var.names="character",
                        chisq="numeric", df="integer",
                        N="integer"))
setClass("GlmTestsScore", representation("GlmTests", score="list"),
         contains="GlmTests")

              
setMethod("[",
          signature(x="SingleSnpTests", i="ANY",
                    j="missing", drop="missing"),
          function(x, i, j, drop) {
            if (is.character(i)) {
              i <- match(i, x@snp.names)
              if (any(is.na(i))) {
                warning(sum(is.na(i)),
                        " SNPs couldn't be found in SingleSnpTests object\n")
                i <- i[!is.na(i)]
              }
            } else if (is.logical(i)) {
              if (length(i)!=length(x@snp.names))
                stop("logical selection array has incorrect length")
              i <- (1:length(i))[i]
            } else if (is.numeric(i)) {
              if (min(i)<0 || max(i)>length(x@snp.names))
                stop("selection index out of range")
            } else {
              stop("illegal selection array")
            }
            if (length(x@N.r2)>0)
              N.r2 <- x@N.r2[i]
            else
              N.r2 <- numeric(0)
            new("SingleSnpTests",
              snp.names=x@snp.names[i], chisq=x@chisq[i,, drop=FALSE],
                N=x@N[i], N.r2=N.r2)
          })

setMethod("[",
          signature(x="SingleSnpTestsScore", i="ANY",
                    j="missing", drop="missing"),
          function(x, i, j, drop) {
            if (is.character(i)) {
              i <- match(i, x@snp.names)
              if (any(is.na(i))) {
                warning(sum(is.na(i)),
                        " SNPs couldn't be found in SingleSnpTests object\n")
                i <- i[!is.na(i)]
              }
            } else if (is.logical(i)) {
              if (length(i)!=length(x@snp.names))
                stop("logical selection array has incorrect length")
              i <- (1:length(i))[i]
            } else if (is.numeric(i)) {
              if (min(i)<0 || max(i)>length(x@snp.names))
                stop("selection index out of range")
            } else {
              stop("illegal selection array")
            }
            if (length(x@N.r2)>0)
              N.r2 <- x@N.r2[i]
            else
              N.r2 <- numeric(0)
            new("SingleSnpTestsScore",
              snp.names=x@snp.names[i], chisq=x@chisq[i,,drop=FALSE],
                N=x@N[i], N.r2=N.r2,
                U=x@U[i,,drop=FALSE], V=x@V[i,,drop=FALSE])
          })

setMethod("[",
          signature(x="GlmTests", i="ANY",
                    j="missing", drop="missing"),
           function(x, i, j, drop) {
             if (is.character(i))
               i <- match(i, names(x), nomatch=0)
             new("GlmTests",
                 snp.names = x@snp.names[i],
                 var.names = x@var.names,
                 chisq = x@chisq[i],
                 df = x@df[i],
                 N = x@N[i])
          })

setMethod("[",
          signature(x="GlmTestsScore", i="ANY",
                    j="missing", drop="missing"),
          function(x, i, j, drop) {
            if (is.character(i)) 
              i <- match(i, names(x), nomatch=0)
            new("GlmTestsScore",
                snp.names = x@snp.names[i],
                var.names = x@var.names,
                chisq = x@chisq[i],
                df = x@df[i],
                N = x@N[i],
                score = x@score[i])
          })
                   
setAs("SingleSnpTests", "data.frame",
      function(from) {
        if (length(from@N.r2)>0)
          data.frame(N=from@N, N.r2=from@N.r2,
                     Chi.squared=from@chisq,
                     P.1df=pchisq(from@chisq[,1], df=1, lower.tail=FALSE),
                     P.2df=pchisq(from@chisq[,2], df=2, lower.tail=FALSE),
                     row.names=from@snp.names)
        else
          data.frame(N=from@N,
                     Chi.squared=from@chisq,
                     P.1df=pchisq(from@chisq[,1], df=1, lower.tail=FALSE),
                     P.2df=pchisq(from@chisq[,2], df=2, lower.tail=FALSE),
                     row.names=from@snp.names)
      })

setMethod("summary", "SingleSnpTests",
          function(object) {
            summary(as(object, 'data.frame'))
          })

setMethod("show", "SingleSnpTests",
          function(object) {
            print(as(object, 'data.frame'))
          })

setAs("GlmTests", "data.frame",
      function(from) {
            chi2 <- from@chisq
            df <- from@df
            p <- pchisq(chi2, df=df, lower.tail=FALSE)
            N <- from@N
            data.frame(row.names=names(from),
                               Chi.squared=chi2, Df=df, p.value=p)
          })

setMethod("summary", "GlmTests",
          function(object) {
            summary(as(object, "data.frame"))
          })

setMethod("show", "GlmTests",
          function(object) {
            print(as(object, "data.frame"))
          })

# There are no standard generics for these, but the call to standardGeneric
# avoids a warning message 
  
setGeneric("p.value", function(x, df) standardGeneric("p.value"),
           useAsDefault=FALSE)

setGeneric("chi.squared", function(x, df) standardGeneric("chi.squared"),
           useAsDefault=FALSE)

setGeneric("deg.freedom", function(x) standardGeneric("deg.freedom"),
           useAsDefault=FALSE)

setGeneric("effect.sign", function(x, simplify) standardGeneric("effect.sign"),
           useAsDefault=FALSE)

setGeneric("sample.size", function(x) standardGeneric("sample.size"),
           useAsDefault=FALSE)

setGeneric("effective.sample.size",
           function(x) standardGeneric("effective.sample.size"),
           useAsDefault=FALSE)

setGeneric("pool2", function(x, y, score) standardGeneric("pool2"),
           useAsDefault=FALSE)

setGeneric("names")

setMethod("p.value", signature(x="SingleSnpTests", df="numeric"),
          function(x, df) {
            if (df>2)
              stop("df must be 1 or 2")
            p <- pchisq(x@chisq[,df], df=df, lower.tail=FALSE)
            names(p) <- x@snp.names
            p
          })

setMethod("p.value", signature(x="GlmTests", df="missing"),
          function(x, df) {
            p <- pchisq(q=x@chisq, x@df, lower.tail=FALSE)
            p
          })


setMethod("chi.squared", signature(x="SingleSnpTests", df="numeric"),
          function(x, df) {
            if (df>2)
              stop("df must be 1 or 2")
            chi2 <- x@chisq[,df]
            names(chi2) <- x@snp.names
            chi2
          })

setMethod("chi.squared", signature(x="GlmTests", df="missing"),
          function(x, df) {
            chi2 <- x@chisq
            chi2
          })


setMethod("deg.freedom", signature(x="GlmTests"),
          function(x) {
            df <- x@df
            df
          })

setMethod("effect.sign", signature(x="GlmTests", simplify="logical"),
          function(x, simplify=TRUE) {
            res <- sapply(x@score, function(x) sign(x$U))
            res
         })

setMethod("effect.sign",
          signature(x="SingleSnpTestsScore", simplify="missing"),
          function(x, simplify) {
            res <- sign(x@U[,1])
            names(res) <- x@snp.names
            res
          })

setMethod("sample.size", signature(x="GlmTests"),
          function(x) {
            res <- x@N
            res
          })

setMethod("sample.size", signature(x="SingleSnpTests"),
          function(x) {
            res <- x@N
            names(res) <- x@snp.names
            res
          })

setMethod("effective.sample.size", signature(x="SingleSnpTests"),
          function(x) {
            if (length(x@N.r2)==0)
              res <- x@N
            else
              res <- x@N.r2
            names(res) <- x@snp.names
            res
          })

setMethod("names", signature(x="SingleSnpTests"), function(x) x@snp.names)

setMethod("names", "GlmTests",
          function(x) {
            objn <- x@snp.names
            if (is.list(objn)) {
              nobjn <- attr(objn, "names")
              if (!is.null(nobjn))
                nobjn
              else
                vapply(objn, function(x){
                  paste(x[1],x[length(x)], sep="...")}, "character")
            }
            else
              as.character(objn)
          })

setMethod("pool2",
          signature(x="SingleSnpTestsScore",y="SingleSnpTestsScore",
                    score="logical"),
          function(x, y, score) {
            all.snps <- union(x@snp.names, y@snp.names)
            can.pool <- intersect(x@snp.names, y@snp.names)
            x.only <- setdiff(x@snp.names, can.pool)
            y.only <- setdiff(y@snp.names, can.pool)
            if (length(can.pool)>0) {
              xsel <- match(can.pool, x@snp.names)
              ysel <- match(can.pool, y@snp.names)
              N <- x@N[xsel] + y@N[ysel]
              U <- x@U[xsel,,drop=FALSE] + y@U[ysel,,drop=FALSE]
              V <- x@V[xsel,,drop=FALSE] + y@V[ysel,,drop=FALSE]
              if (length(x@N.r2)>0) {
                if (length(y@N.r2)>0)
                  Nr2 <- x@N.r2[xsel] + y@N.r2[ysel]
                else
                  Nr2 <- x@N.r2[xsel] + y@N[ysel]
              } else {
                if (length(y@N.r2)>0)
                  Nr2 <- x@N[xsel]+  y@N.r2[ysel]
                else
                  Nr2 <- numeric(0)
              }
            } else {
              N <- NULL
              U <- NULL
              V <- NULL
              Nr2 <- numeric(0)
            }
            if (length(x.only)>0) {
              xsel <- match(x.only, x@snp.names)
              U <- rbind(U, x@U[xsel,,drop=FALSE])
              V <- rbind(V, x@V[xsel,,drop=FALSE])
              if (length(Nr2>0)) {
                if (length(x@N.r2)>0) 
                  Nr2 <- c(Nr2, x@N.r2[xsel])
                else
                  Nr2 <- c(Nr2, x@N[xsel])
              } else {
                if (length(x@N.r2)>0)
                  Nr2 <- c(N, x@N.r2[xsel])
              }
              N <- c(N, x@N[xsel])
            }
            if (length(y.only)>0) {
              ysel <- match(y.only, y@snp.names)
              U <- rbind(U, y@U[ysel,,drop=FALSE])
              V <- rbind(V, y@V[ysel,,drop=FALSE])
              if (length(Nr2>0)) {
                if (length(y@N.r2)>0) 
                  Nr2 <- c(Nr2, y@N.r2[ysel])
                else
                  Nr2 <- c(Nr2, y@N[ysel])
              } else {
                if (length(y@N.r2)>0)
                  Nr2 <- c(N, y@N.r2[ysel])
              }
              N <- c(N, y@N[ysel])
            }
            chisq <- .Call("chisq_single", list(U=U, V=V, N=N),
                           PACKAGE="snpStats")
            if (score)
              res <- new("SingleSnpTestsScore",
                         snp.names=c(can.pool, x.only, y.only),
                         chisq=chisq, N=N, U=U, V=V, N.r2=Nr2)
            else
              res <- new("SingleSnpTests",
                         snp.names=c(can.pool, x.only, y.only),
                         chisq=chisq, N=N, N.r2=Nr2)
            res
          })

# need to edit this

setMethod("pool2",
          signature(x="GlmTestsScore",y="GlmTestsScore",
                    score="logical"),
          function(x, y, score) {
            if (!is.null(x@var.names) && !is.null(y@var.names) &&
                (any(x@var.names!=y@var.names)))
              warning("tests to be pooled have non-matching var.names slots") 
            nm.x <- names(x)
            nm.y <- names(y)
            if (any(duplicated(nm.x))||any(duplicated(nm.y)))
              stop("At least one object has duplicated test names")
            to.pool <- intersect(nm.x, nm.y)
            if (length(to.pool)>0) {
              res <- .Call("pool2_glm",
                           x[to.pool],
                           y[to.pool], score, 
                           PACKAGE="snpStats")
 
            }
            else {
              res <- NULL
            }
            x.only <- setdiff(nm.x, to.pool)
            y.only <- setdiff(nm.y, to.pool)
            if (length(x.only)==0 && length(y.only)==0)
              return(res);
            ix <- match(x.only, nm.x, nomatch=0)
            iy <- match(y.only, nm.y, nomatch=0)
            if (is.null(res)) {
              res.chisq <- NULL
              res.df <- NULL
              res.N <- NULL
              res.score <- NULL
            }
            else {
              res.chisq <- res@chisq
              res.df <- res@df
              res.N <- res@N
              if (score)
                res.score <- res@score
            }
            if (score)
              res <- new("GlmTestsScore",
                         snp.names=c(to.pool, x.only, y.only),
                         var.names=x@var.names,
                         chisq=c(res.chisq, x@chisq[ix], y@chisq[iy]),        
                         df=c(res.df, x@df[ix], y@df[iy]),        
                         N=c(res.N, x@N[ix], y@N[iy]),        
                         score=append(res.score,
                           append(x@score[ix], y@score[iy])))
            else
               res <- new("GlmTests",
                          snp.names=c(to.pool, x.only, y.only),
                          var.names=x@var.names,
                          chisq=c(res.chisq, x@chisq[ix], y@chisq[iy]),        
                          df=c(res.df, x@df[ix], y@df[iy]),        
                          N=c(res.N, x@N[ix], y@N[iy]))       
            res
          })

# Switch allele methods 

setGeneric("switch.alleles",
           function(x, snps) standardGeneric("switch.alleles"),
           useAsDefault=FALSE)

setMethod("switch.alleles", signature(x="SnpMatrix", snps="ANY"),
          function(x, snps) {
            if (is.character(snps)) 
              snps <- match(snps, colnames(x))
            else if (is.logical(snps)) {
              if (length(snps)!=ncol(x))
                stop("logical snp selection vector is wrong length")
              snps <- (1:ncol(x))[snps]
            }
            else if (!is.integer(snps))
              stop("snp selection must be character, logical or integer")
            if (any(is.na(snps) | snps>ncol(x) | snps<1))
              stop("illegal snp selection")
            .Call("smat_switch", x, snps, PACKAGE="snpStats")
          })

setMethod("switch.alleles", signature(x="SingleSnpTestsScore", snps="ANY"),
          function(x, snps) {
            if (is.character(snps)) {
              snps <- match(snps, x@snp.names)
              if (any(is.na(snps)))
                warning(sum(is.na(snps)),
                        " SNP names were not found in tests object")
              snps <- snps[!is.na(snps)]
            } 
            ntest <- length(x@snp.names)
            if (is.logical(snps)) {
              if (length(snps)!=ntest)
                stop("incompatible arguments")
              if (sum(snps)==0)
                return(x)
            } else if (is.numeric(snps)) {
              if (length(snps)==0)
                return(x)
              if (max(snps)>ntest || min(snps)<1)
                stop("incompatible arguments")
            } else {
              stop("SNPs to be switched must be indicated by name, position, or by a logical vector")
            }
            res <- x
            res@U[snps,1] <- -x@U[snps,1]
            if (ncol(x@U)==3) {
              res@U[snps,2] <- -x@U[snps,2]
              res@U[snps,3] <- x@U[snps,3] - x@U[snps,2]
              res@V[snps,4] <- x@V[snps,4] - 2*x@V[snps,3] +
                x@V[snps,2]
              res@V[snps,3] <- x@V[snps,3] - x@V[snps,2]
            } else {
              res@U[snps,2] <- x@U[snps,2] - x@U[snps,1]
              res@V[snps,3] <- x@V[snps,3] - 2*x@V[snps,2] +
                x@V[snps,1]
              res@V[snps,2] <- x@V[snps,2] - x@V[snps,1]
            }
            res
          })

## This next will be slow but needed rarely

setMethod("switch.alleles", signature(x="GlmTestsScore",
                                      snps="character"),
          function(x, snps) {
            if (is.list(x@snp.names))
              switch <- lapply(x@snp.names, function(x, y) x%in%y, snps)
            else
              switch <- x@snp.names %in% snps
            N <- length(x@score)
            new.score <- vector("list", N)
            for (i in 1:N) {
              Ui <- x$U
              Vi <- x$V
              Si <- switch[i]
              lu <- length(Ui)
              ls <- length(Si)
              if (lu!=ls) {
                if (lu%%ls)
                  stop("error in GlmTest object")
                Si <- rep(Si, rep(lu/ls, ls))
              }
              new.score[[i]] <- list(U=ifelse(Si, -Ui, Ui), V=Vi)
            }
            new("GlmTestsScore", snp.names=x@snp.names, var.names=x@var.names,
                chisq=x@chisq, df=x@df, N=x@N,
                score=new.score)
          })


pool <- function(..., score=FALSE) {
  argl <- list(...)
  na <- length(argl)
  while (na > 0 && is.null(argl[[na]])) {
    argl <- argl[-na]
    na <- na - 1
  }
  if (na<2)
    stop("need at least two test sets to pool")
  if (na>2) {
    p2 <- pool2(..1, ..2, score=TRUE)
    r <- do.call(pool, c(p2, argl[3:na], score=score))
  }
  else
    r <- pool2(..1, ..2, score=score)
  r
}


## Parameter estimates

setClass("GlmEstimates", contains="list")

setMethod("[", signature("GlmEstimates", i="ANY", j="missing",
                         drop="missing"),
          function(x, i) {
            if (is.character(i))
              i <- match(i, names(x))
            res <- x@.Data[i, drop=FALSE]
            if (length(res)==0)
              return(NULL)
            names(res) <- names(x)[i]
            new("GlmEstimates", res)
           })

setMethod("show", "GlmEstimates",
          function(object) {
            len0 <- max(5, nchar(names(object)))
            len1 <- max(10,
                        sapply(object, function(x)
                               max(if(is.null(x)) 0 else nchar(x$Y.var))))
            len2 <- max(9,
                        sapply(object, function(x)
                               max(if(is.null(x)) 0 else nchar(names(x$beta)))))
            space <- 2
            nwidth <- 10
            di <- 5
            divide <- rep("-", len0+len1+len2+3*nwidth+5*space)
            cat("\n", format("Model", justify="right", width=len0),
                format("Y-variable", justify="right", width=len1+space),
                format("Parameter", justify="right", width=len2+space),
                format("Estimate", justify="right", width=nwidth+space),
                format("S.E.", justify="right", width=nwidth+space),
                format("z-value", justify="right", width=nwidth+space),"\n",
                divide, "\n", sep="")
            labs <- names(object)
            for (j in 1:length(object)) {
              labj <- labs[j]
              x <- object[[j]]
              if (is.null(x)) {
                cat(format(labj, justify="right", width=len1),
                    "- No estimates available\n") 
              }
              else {
                nyj <- x$Y.var
                nb <- length(x$beta)
                namep <- names(x$beta)
                ii <- 0
                for (i in 1:nb){
                  ii <- ii+i
                  bi <- x$beta[i]
                  si <- sqrt(x$Var.beta[ii])
                  zi <- bi/si
                  cat(format(labj, justify="right", width=len0), sep="")
                  cat(format(nyj, justify="right", width=len1+space), sep="")
                  labj <- ""
                  nyj <- ""
                  cat(format(namep[i], justify="right", width=len2+space),
                      format(bi, justify="right", width=nwidth+space,
                             digits=di),
                      format(si, justify="right", width=nwidth+space,
                             digits=di),
                      formatC(zi, format="f", width=nwidth+space, digits=3),
                      "\n", sep="")
                }
              }
              cat(divide, "\n", sep="")
            }
          }
          )

## Wald test

setAs("GlmEstimates", "GlmTests",
      function(from) {
        .Call("wald", from, package="snpStats")
      }
      )
        

## To do

## names method for snp and X.snp

# This routine takes a whole row of summary() into input
.contingency.table <- function(sum.case, sum.control) {
  OR <- sum.case$MAF / (1-sum.case$MAF) * (1-sum.control$MAF) /sum.control$MAF
  cat("OR: ", OR, "\n")
  
  case    <- c((1-sum.case$MAF), sum.case$MAF) * 2 * sum.case$Calls / sum.case$Call.rate
  control <- c((1-sum.control$MAF), sum.control$MAF) * 2 * sum.control$Calls / sum.control$Call.rate
  
  x <- cbind(control, case)
  # We'll just do fisher test, but it only does integers
  # print(fisher.test(x))
  x
}

single.snp.tests <- function(phenotype, stratum, data=sys.parent(), snp.data,
                             rules=NULL, subset, snp.subset, uncertain=FALSE,
                             score=FALSE) {
  m <- match.call()
  smiss <- missing(stratum)
  ssmiss <- missing(subset)
  if (smiss)
    stratum <- NULL
  if (!is(snp.data, "SnpMatrix"))
    stop("snp.data argument must be of class SnpMatrix")
  nr.snps = nrow(snp.data)
  nc.snps = ncol(snp.data)
  if (missing(data)) { # phenotype and stratum are in calling environment
    phenotype <- as.numeric(phenotype)
    complt <- !is.na(phenotype)
    if (length(phenotype)!=nr.snps)
      stop("incompatible length for phenotype vector")
    if (!smiss) {
      stratum <- as.integer(as.factor(stratum))
      if (length(stratum)!=nr.snps)
        stop("incompatible length for stratum vector")
      complt <- complt & !is.na(stratum)
    }
    if (!ssmiss) {
      if (is.logical(subset)) {
        if (length(subset)!=nr.snps)
          stop("incompatible length for subset vector")
        else
          inss <- subset
      }
      else if (is.integer(subset)) {
        if (length(subset)>nr.snps ||
            any(subset>nr.snps | duplicated(subset)))
          stop("illegal subset vector")
        inss <- rep(FALSE, nr.snps)
        inss[subset] <- TRUE
      }
      else {
        stop("illegal type for subset vector")
      }
      complt <- complt & inss
    }
  }
  else { # phenotype, stratum, and subset  are in data dataframe
    nm.snps <- rownames(snp.data)
    nm.data <- rownames(data)
    nr.data <- nrow(data)
    # which points to rows in the data matrix
    which <- match(nm.snps, nm.data, nomatch=NA)
    nnomatch <- sum(is.na(which))
    if (nnomatch>0)
      warning(nnomatch, " rows in the SnpMatrix have no matching row in the phen
otype data matrix")
    phenotype <- as.numeric(eval(m$phenotype, envir=data))[which]
    complt <- !is.na(phenotype) 
    if (!smiss) {
      stratum <- as.integer(as.factor(eval(m$stratum, envir=data)))[which]
      complt <- complt & !is.na(stratum)
    }
    if (!ssmiss) {
      subset <- eval(m$subset, envir=data)
      if (is.logical(subset)) {
        if (length(subset)!=nr.data)
          stop("incompatible length for subset vector")
        inss <- subset
      }
      else {
        if (is.character(subset)) {
          subset <- match(subset, nm.data, nomatch=0)
          if (any(subset==0))
            stop("unmatched name in subset vector")
        }
        else if (is.integer(subset)) {
          if (length(subset)>nr.data ||
              any(subset>nr.data | duplicated(subset)))
            stop("illegal subset vector")
        }
        else {
          stop("illegal type for subset vector")
        }
        inss <- rep(FALSE, nr.data)
        inss[subset] <- TRUE
      }
      inss <- inss[which]
      inss[is.na(inss)] <- FALSE
      complt <- complt & inss
    }
  }
  if (class(snp.data)=="XSnpMatrix" && any(is.na(snp.data@diploid))) {
    warning("There are ", sum(is.na(snp.data@diploid)),
            " subjects with NA in the diploid slot. These were ignored")
    complt <- complt & !is.na(snp.data@diploid)
  }
  subset <- (1:nr.snps)[complt]


  if (missing(snp.subset))
    snp.subset <- NULL
  else {
    if (is.null(rules))
      n.targ <- nc.snps
    else
      n.targ <- length(rules)
    if (is.character(snp.subset)) {
      if (is.null(rules))
        snp.subset <- match(snp.subset, colnames(snp.data))
      else
        snp.subset <- match(snp.subset, names(rules))
      if (any(is.na(snp.subset)))
        stop("unmatched snps in snp.subset argument")
    }
    else if (is.logical(snp.subset)){
      if (length(snp.subset)!=n.targ)
        stop("snp.subset array has incorrect length")
      snp.subset <- (1:n.targ)[snp.subset]
    }
    else if (is.integer(snp.subset)) {
      if (any(snp.subset<1 | snp.subset>n.targ))
        stop("snp.subset element(s) out of range")
    }
    else 
      stop("illegal type for snp.subset")
  }
  scores <- .Call("score_single", phenotype, stratum, snp.data, rules, subset,
                  snp.subset, uncertain, PACKAGE="snpStats")
  chisq <- .Call("chisq_single", scores, PACKAGE="snpStats")
  if (is.null(rules)) {
    if (is.null(snp.subset))
      tested <- colnames(snp.data)
    else
      tested <- colnames(snp.data)[snp.subset]
  } else {
    if (is.null(snp.subset))
      tested <- names(rules)
    else
      tested <- names(rules)[snp.subset]
  }
  if (score)
    res <- new("SingleSnpTestsScore", snp.names=tested,
               chisq=chisq, N=scores$N, N.r2=scores$N.r2,
               U=scores$U, V=scores$V)
  else
    res <- new("SingleSnpTests", snp.names=tested, chisq=chisq,
               N=scores$N,  N.r2=scores$N.r2)
  res
}

write.SnpMatrix <-

function(x, file, as.alleles=FALSE, append=FALSE, quote=TRUE, sep=" ", eol="\n",
         na="NA", row.names=TRUE, col.names=TRUE) {
  if (!is(x, "SnpMatrix"))
    stop("argument must be a SnpMatrix object")
  if (append && col.names)
    stop("col.names option is illegal in append mode")
  res <- .C("write_as_matrix", as.character(file),
     x@.Data, as.integer(nrow(x)), as.integer(ncol(x)),
     rownames(x), colnames(x), as.logical(as.alleles), as.logical(append),
     as.logical(quote), as.character(sep), as.character(eol),
     as.character(na), as.logical(row.names), as.logical(col.names),
     logical(1), PACKAGE="snpStats")
  error <- res[[15]]
  if (error==1)
    stop("Couldn't open output file")
  else
    c(nrow(x), ncol(x))
}

sm.compare <- function(obj1, obj2, row.wise=TRUE, col.wise=TRUE) {
  if (!row.wise && !col.wise)
    stop("nothing to do")
  subjects <- intersect(rownames(obj1), rownames(obj2))
  if (length(subjects)==0)
    stop("objects have no rows in common")
  if (length(subjects)!=nrow(obj1) || length(subjects)!=nrow(obj2))
    warning("non-comformant rows; proceeding with ", length(subjects),
            " which are in common")
  snps <- intersect(colnames(obj1), colnames(obj2))
  if (length(snps)==0)
    stop("objects have no columns in common")
  if (length(snps)!=ncol(obj1) || length(snps)!=ncol(obj2))
    warning("non-comformant columns; proceeding with ", length(snps),
            " which are in common")
  ## Make into 3-D array, with object as last dimension
  x <- array(c(obj1[subjects,snps], obj2[subjects,snps]),
             dim=c(length(subjects), length(snps), 2))
  ## Compare function 
  comp <- function(mat) {
    c(sum(mat[,1]==mat[,2]),
      sum(mat[,1]!=mat[,2]),
      sum(mat[,1]==00 & mat[,2]==00),
      sum((mat[,1]==00 & mat[,2]!=00)|(mat[,1]!=00 & mat[,2]==00)),
      sum((mat[,1]==01 & mat[,2]==01)|(mat[,1]==03 & mat[,2]==03)),
      sum((mat[,1]==01 & mat[,2]==03)|(mat[,1]==03 & mat[,2]==01)),
      sum(mat[,1]==02 & mat[,2]==02),
      sum((mat[,1]==02 & mat[,2]!=02)|(mat[,1]!=02 & mat[,2]==02))
    )
  }
  snames <-  c("Agree", "Disagree", "NA.agree", "NA.disagree", "Hom.agree",
                    "Hom.switch", "Het.agree", "Het.Hom")
  ## Row compare
  if (row.wise) {
    rc <- t(apply(x, 1, comp))
    colnames(rc) <- snames
    rownames(rc) <- subjects
    if (!col.wise)
      return(rc)
  }
  ## Column compare
  if (col.wise) {
    cc <- t(apply(x, 2, comp))
    colnames(cc) <- snames
    rownames(cc) <- snps
    if (!row.wise)
      return(cc)
  }
  ## Both comparisons carried out
  list(row.wise=rc, col.wise=cc)
}
  

  
  

convert.snpMatrix <- function(object) {
  cl <- class(object)
  pkg <- attr(cl, "package")
  newpkg<- "snpStats"
  old.classes <- c("snp.matrix", "X.snp.matrix", "single.snp.tests",
                   "single.snp.tests.score", "snp.tests.glm",
                   "snp.tests.glm.score", "snp.estimates.glm",
                   "imputation.rules")
  new.classes <- c("SnpMatrix", "XSnpMatrix", "SingleSnpTests",
                   "SingleSnpTestsScore", "GlmTests", "GlmTestsScore",
                   "GlmEstimates", "ImputationRules")
  icl <- match(cl[1], old.classes)
  if (is.na(icl)) {
    warning("unrecognized old snpMatrix class:", cl)
    return(object) ## return unchanged
  }
  else {
    if (icl==2) { ## X.snp.matrix
      object <- unclass(object)
      dip <- attr(object, "Female")
      object <- new("XSnpMatrix", object, diploid=dip)
      return(object)
    }
    else if (icl==5) { ## snp.tests.glm 
      object <-  new("GlmTests", snp.names=object@test.names,
                    var.names="Phenotype",
                    chisq=object@chisq, df=object@df, N=object@N)
      return(object)
    }
    else if (icl==6) { ## snp.tests.glm.score
      object <-  new("GlmTestsScore", snp.names=object@test.names,
                    var.names="Phenotype",                   
                    chisq=object@chisq, df=object@df, N=object@N,
                    score=object@score)
      return(object)
    }
    if (icl==8 && (is.null(pkg) || pkg!=newpkg)) 
      warning("Old snpMatrix imputation.rules will probably need to be regenerated")
    newcl <- new.classes[icl]
    attr(newcl, "package") <- newpkg
    class(object) <- newcl
    if (!isS4(object))
      object <- asS4(object)
    return(object)
  }
}

convert.snpMatrix.dir  <- function(dir=".", ext="RData") {
  files <- dir(dir)
  with.ext <- grep(paste(".", ext, "$", sep=""), files)
  if (length(with.ext)==0)
    cat("No files to be converted\n")
  else {
    toconvert <- files[with.ext]
    for (file in toconvert) {
      cat("Converting objects in file", file, "...")
      objects <- load(file)
      for (object in objects)
        assign(object, convert.snpMatrix(get(object)))
      cat("Saving")
      save(list=objects, file=file)
      cat("\n")
      remove(list=objects)
    }
  }
}




  

read.pedfile <- function(file, n, snps, which, split="\t| +", sep=".",
                        na.strings="0", lex.order=FALSE) {
  ## Constants
  r0 <- as.raw(0)
  r1 <- as.raw(1)
  r2 <- as.raw(2)
  r3 <- as.raw(3)
  ## Input file
  con <- gzfile(file)
  open(con)
  ## If no line count, find out
  if (missing(n)) {
    n <- 0
    repeat {
      line <- readLines(con, n=1)
      if (length(line)==0) break;
      n  <- n+1;
    }
    if (n==0)
      stop("Nothing read")
    seek(con, 0)
  }
  ## Find snp names
  gen <- missing(snps)
  map <- NULL
  if (!gen) {
    m <- length(snps)
    if (m==1) {
      map <- read.table(snps, comment.char="")
      m <- nrow(map)
      if (missing(which)) {
        which <- 1
        repeat {
          snps <- map[,which]
          if (!any(duplicated(snps)))
            break
          if (which==ncol(map))
            stop("No unambiguous snp names found on file")
          which <- which+1
        }
      }
      else {
        snps <- map[,which]
      }
    }
  }
  else {
    line <- readLines(con, n=1)
    fields <- strsplit(line, split)[[1]]
    nf <- length(fields)
    if (nf%%2!=0)
      stop("Odd number of fields")
    m <- (nf - 6)/2
    seek(con, 0)
  }
  nf <- 6+2*m
  ## Generate empty matrix
  result <- matrix(raw(n*m), nrow=n)
  ## Columns of subject dataframe
  ped <- character(n)
  mem <- character(n)
  pa <- character(n)
  ma <- character(n)
  sex <- numeric(n)
  aff <- numeric(n)
  rownms <- character(n)
  a1 <- a2 <- rep(NA, m)
  a1m <- a2m <- rep(TRUE, m)
  mallelic <- rep(FALSE, m) ## Multiallelic?
  for (i in 1:n) {
    line <- readLines(con, n=1)
    fields <- strsplit(line, "\t| +")[[1]]
    to.na <- fields %in% na.strings
    fields[to.na] <- NA
    ped[i] <- fields[1]
    mem[i] <- fields[2]
    pa[i] <- fields[3]
    ma[i] <- fields[4]
    sex[i] <- as.numeric(fields[5])
    aff[i] <- as.numeric(fields[6])
    alleles <- matrix(fields[7:nf], byrow=TRUE, ncol=2)
    one <- two <- rep(FALSE, m)
    for (k in 1:2) {
      ak <- alleles[,k]
      akm <- is.na(ak) 
      br1 <- !akm & a1m
      a1[br1] <- ak[br1]
      a1m[br1] <- FALSE
      br2 <- !akm & (a1==ak)
      one[br2] <- TRUE
      br3 <- !akm & !a1m & (a1!=ak)
      br4 <- br3 & a2m
      a2[br4] <- ak[br4]
      a2m[br4] <- FALSE
      br5 <- br3 & (a2==ak)
      two[br5] <- TRUE
      mallelic <- mallelic | !(akm|one|two)
    }
    gt <- rep(r0, m)
    gt[one&!two] <- r1
    gt[one&two] <- r2
    gt[two&!one] <- r3
    result[i,] <- gt
  }
  close(con)
  ## Warnin messages
  if (any(a1m))
    warning("no data for ", sum(a1m), " loci")
  mono <-(a2m & !a1m) 
  if (any(mono))
    warning(sum(mono), " loci were monomorphic")
  if (any(mallelic)) {
    result[,mallelic] <- r0;
    warning(sum(mallelic), " loci were multi-allelic --- set to NA")
  }
  ## SnpMatrix result
  if (gen)
    snps <- paste("locus", 1:m, sep=sep)
  if (any(duplicated(ped))) {
    if (any(duplicated(mem))) {
      rnames <- paste(ped, mem, sep=sep)
      if (any(duplicated(rnames)))
        stop("could not create unique subject identifiers")
    }
    else 
      rnames <- mem
  } 
  else
    rnames <- ped
  dimnames(result) <- list(rnames, snps)
  result <- new("SnpMatrix", result)
  ## Switch alleles if requested and necessary
  if (lex.order) {
    swa<- (!(is.na(a1)|is.na(a2)) & (a1>a2))
    switch.alleles(result, swa)
    a1n <- a1
    a1n[swa] <- a2[swa]
    a2[swa] <- a1[swa]
    a1 <- a1n
  }
  ## Subject support file
  fam <- data.frame(row.names=rnames, pedigree=ped, member=mem,
                    father=pa, mother=ma, sex=sex, affected=aff)
  ## map data frame 
  if (is.null(map))
    map <- data.frame(row.names=snps, snp.name=snps,
                      allele.1=a1, allele.2=a2)
  else {
    map$allele.1 <- a1
    map$allele.2 <- a2
    names(map)[which] <- "snp.names"
  }
  list(genotypes=result, fam=fam, map=map)    
}

  

# Set control parameters 

glm.test.control <-
function(maxit=20, epsilon=1.e-5, R2Max=0.99) 
  list(maxit=as.integer(maxit),
       epsilon=as.double(epsilon),
       R2Max=as.double(R2Max))

# Logit tests with snps on LHS */


snp.lhs.tests <-
  
function(snp.data, base.formula, add.formula, subset, snp.subset,
         data=sys.parent(), robust=FALSE, uncertain=FALSE,
         control=glm.test.control(),
         score=FALSE)
{   
  if (is(snp.data,"SnpMatrix")) 
    snames <- rownames(snp.data)
  else
    stop("snp.data must be stored as a SnpMatrix object")
  
  call <- match.call()
  temp <- c("", "base.formula", "subset", "data")
  m <- call[match(temp, names(call), nomatch=0)]
  data.missing <- is.null(m$data)
  special <- c("strata", "cluster")
  Terms <- terms(base.formula, special, data=data)

  # Separate intercept term from rest of model formula
  # Note as of now I ignore this and fit an intercept anyway!

  INTERCEPT <- attr(Terms, "intercept")
  if (!INTERCEPT) # put one back temporarily
    attr(Terms, "intercept") <- 1

  # Model frame for base model

  names(m)[2] <- "formula"
  m[[1]] <- as.name("model.frame")
  m[[2]] <- Terms
  m <- as.call(c(as.list(m), list(na.action=as.symbol("na.omit"))))
  m <- eval(m)
  bnames <- rownames(m)

  # Base model matrix -- minus any cluster or strata variables

  strats <- attr(Terms, "specials")$strata
  clust <- attr(Terms, "specials")$cluster
  dropx <- NULL
  if (!is.null(clust)) {
    # Some check on valid clusters?
    if (missing(robust))
      robust <- TRUE
    tempc <- untangle.specials(Terms, "cluster", 1)
    clust <- as.integer(cluster(m[, tempc$vars]))  # unused 'shortlabel=TRUE' was here - possibly a mistake
    dropx <- tempc$terms
  }
  if (!is.null(strats)) {
    temps <- untangle.specials(Terms, "strata", 1)
    strats <- as.integer(strata(m[, temps$vars], shortlabel=TRUE))
    dropx <- c(dropx, temps$terms)
  }

  if (length(dropx)) 
    newTerms <- Terms[-dropx]
  else
    newTerms <- Terms
  X <- model.matrix(newTerms, m)

  # Remove unit column vector
  
  ones <- match("(Intercept)", colnames(X), nomatch=0)
  if (ones)
    X <- X[,-ones, drop=FALSE]
  
  # Additional model matrix
  
  temp <- c("", "add.formula", "subset", "data")
  m <- call[match(temp, names(call), nomatch=0)]
  Terms <- terms(add.formula, data=data)
     
  names(m)[2] <- "formula"
  m[[1]] <- as.name("model.frame")
  m <- as.call(c(as.list(m), list(na.action=as.symbol("na.omit"))))
  m <- eval(m)
  anames <- rownames(m) 
  Z <- model.matrix(Terms, m)

  # If there is an intercept term in the model already, remove this from Z

  ones <- match("(Intercept)", colnames(Z), nomatch=0)
  if (ones)
    Z <- Z[,-ones, drop=FALSE]
 
  
  # Find common rows and mapping to line up  snp.data

  common <- intersect(anames, bnames)
  if (!data.missing) {
    common <- intersect(common, snames)
    ix <- match(common, bnames)
    iz <- match(common, anames)
    iy <- match(common, rownames(snp.data))
  }
  else {
    ix <- match(common, bnames)
    iz <- match(common, anames)
    iy <- as.numeric(common) # common will refer to row numbers in snp.data  
  }
  N <- as.integer(length(common))
  if (N==0)
    stop("no observations")
  else {
    snp.data <- snp.data[iy,]
    X <- X[ix,, drop=FALSE]
    if (!is.null(strats))
      strats <- strats[ix]
    if (!is.null(clust))
      clust <- clust[ix]
    Z <- Z[iz,, drop=FALSE]
  }

  if (missing(snp.subset)) {
    snp.subset <- NULL
  }
  else {
    if (is.character(snp.subset)) {
      snp.subset <- match(snp.subset, colnames(snp.data))
      if (any(is.na(snp.subset)))
        stop("unmatched snps in snp.subset argument")
    }
    else if (is.logical(snp.subset)){
      if (length(snp.subset)!=ncol(snp.data))
        stop("snp.subset element(s) out of range")
      snp.subset <- (1:ncol(snp.data))[snp.subset]
    }
    else if (is.integer(snp.subset)) {
      if (any(snp.subset<1 | snp.subset>ncol(snp.data)))
        stop("snp.subset element(s) out of range")
    }
    else 
      stop("illegal type for snp.subset")
  }
  .Call("snp_lhs_score",
        snp.data, X, strats, Z, snp.subset, robust, clust, uncertain,
        control, as.logical(score), PACKAGE="snpStats")
}

# GLM tests with SNPs on RHS

snp.rhs.tests <-
function(formula, family="binomial", link, weights, subset,
          data=parent.frame(), snp.data, rules=NULL, 
          tests=NULL, robust=FALSE, uncertain=FALSE, 
          control=glm.test.control(),
          allow.missing=0.01, score=FALSE) {
  
  call <- match.call()

  # Family and link function
  
  fam <- pmatch(tolower(family),
                c("binomial", "poisson", "gaussian", "gamma"))
  if (is.na(fam))
    stop("Unrecognized family argument")
  if (fam==0)
    stop("Ambiguous family argument")

  if (missing(link))
    lnk <- fam # Canonical link is default
  else {
    lnk <- pmatch(tolower(link), c("logit", "log", "identity", "inverse"))
    if (is.na(fam))
      stop("Unrecognized link argument")
    if (lnk==0)
      stop("Ambiguous link argument")
  }
  no.data <- missing(data)
  if (no.data) {
    data <- environment(formula)
  }
  temp <- c("", "formula", "data", "weights", "subset")
  m <- call[match(temp, names(call), nomatch=0)]
  special <- c("strata", "cluster")
    
  Terms <- if (no.data)
    terms(formula, special)
  else terms(formula, special, data=data)

  # Separate intercept term from rest of model formula

  INTERCEPT <- attr(Terms, "intercept")
  if (!INTERCEPT) # put one back temporarily
    attr(Terms, "intercept") <- 1

  # Model frame for null model

  m$formula <- Terms
  if (no.data)
    m$rows <- 1:nrow(snp.data)
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
        
  # Find common rows and mapping to line up  snp.data

  if (is(snp.data, "SnpMatrix")) 
    snames <- rownames(snp.data)
  else
    stop("snp.data must be stored as a SnpMatrix")

  if (no.data) {
    map <- m[["(rows)"]]
  }
  else {
    mnames <- rownames(m)
    map <- match(mnames, snames, nomatch=0)
  }
  use <- map>0
  N <- sum(use)
  if (N==0)
    stop("no observations")
  

  # Response vector

  if(is.factor(m[[1]])){
    # This is just to avoid the warning that conversion
    # to numeric(=double) from factor is ignored.
    Y <- model.response(m, "any")
  } else {
    Y <- model.response(m, "numeric")
  }
  if (fam==1) { # Binomial family
    Y <- as.factor(Y)
    if (nlevels(Y)==2)
      Y <- as.numeric(Y)-1
    else
      stop("Response is not on two levels")
  }

  # Model matrix -- minus any cluster or strata variables

  strats <- attr(Terms, "specials")$strata
  clust <- attr(Terms, "specials")$cluster
  dropx <- NULL
  if (length(clust)) {
    # Some check on valid clusters?
    if (missing(robust))
      robust <- TRUE
    tempc <- untangle.specials(Terms, "cluster", 1)
    clust <- strata(m[, tempc$vars], shortlabel=TRUE)
    dropx <- tempc$terms
  }
  else
    clust <- NULL
  if (length(strats)) {
    temps <- untangle.specials(Terms, "strata", 1)
    strats <- strata(m[, temps$vars], shortlabel=TRUE)
    if (fam==1) {
      ## Check for saturated response
      levs <- levels(strats)
      for (s in 1:nlevels(strats)) {
        ins <- use & (as.numeric(strats)==s)
        if (all(Y[ins]==1)) {
          use[ins] <- FALSE
          warning("100 percent response in stratum ", levs[s],
                "; ", sum(ins), " observations dropped")
        } else if (all(Y[ins]==0)) {
          use[ins] <- FALSE
          warning("0 percent response in stratum ", levs[s],
                "; ", sum(ins), " observations dropped")
        }   
      }
    }
    dropx <- c(dropx, temps$terms)
  }
  else
    strats <- NULL

  if (length(dropx)) 
    newTerms <- Terms[-dropx]
  else
    newTerms <- Terms
  X <- model.matrix(newTerms, m)

  # Remove unit column vector
  
  ones <- match("(Intercept)", colnames(X), nomatch=0)
  if (ones)
    X <- X[, -ones, drop=FALSE]
  if (ncol(X) == 0)
    X <- NULL
  
  # Prior weights

  if (!is.null(model.weights(m))) {
    cat("One\n")
    weights <- model.weights(m)
    if (any(weights<0))
      stop("negative prior weights not allowed")
  }
  else
    weights <- NULL

  # Offsets

  offset <- model.offset(m)
  if(!is.null(offset)) {
    offset <- offset
    stop("Can't deal with offsets at the moment")
  }

  # Sort and reshape arrays if necessary

  if (!all(use)) {
    Y <- Y[use]
    if (!is.null(X))
      X <- X[use,,drop=FALSE]
    if (!is.null(weights))
      weights <- weights[use]
    if (!is.null(strats))
      strats <- strats[use]
    if (!is.null(clust))
      clust <- clust[use]
    map <- map[use]
    snp.data <- snp.data[map,]
  }
  else if (any(map != 1:N))
    snp.data <- snp.data[map,]
    
  # Imputation

  if (!is.null(rules) && class(rules)!="ImputationRules")
    stop("illegal class for `rules' argument")


  # Coerce tests argument to correct form #

  if(is.null(tests)) {
    if (is.null(rules)) {
      tests <- as.integer(1:ncol(snp.data))
    }
    else {
      tests <- as.integer(-(1:length(rules)))
    }
  }
  else if (is.logical(tests)) {
    tests <- as.integer(1:ncol(snp.data))[tests]
  }    
  else if (is.character(tests)) {
    tests <- .col.numbers(tests, colnames(snp.data), names(rules))
  }
  else if (is.numeric(tests)) {
    tests <- as.integer(tests)
  }
  else if(is.list(tests)) {
    tests <- lapply(tests, .col.numbers, colnames(snp.data), names(rules))
  } else {
    stop("illegal tests argument")
  }

  .Call("snp_rhs_score", Y, fam, lnk, X, strats, snp.data, rules,
        weights, tests, robust, clust, uncertain, control, allow.missing,
        as.logical(score),
        PACKAGE="snpStats")
}  

# Private utility function to 
# match character array  to list of (first) names
# or (optionally) to second names if that fails
  
.col.numbers <- function(inc, first.names, second.names=NULL) {
  if (is.character(inc)) {
    res <- match(inc, first.names, nomatch=NA)
    missing <- is.na(res)
    if (any(missing)) {
      if (!is.null(second.names)) {
        res[missing] <- - match(inc[missing], second.names, nomatch=NA)
        missing <- is.na(res)
        if (any(missing)) {
          warning("Unmatched SNP names in tests argument")
          res <- res[!missing]
        }
      } else {
        warning("Unmatched SNP names in tests argument")
        res <- res[!missing]
      }
    }
  }
  else if (is.numeric(inc)) {
    res <- inc
  }
  else stop("Illegal multiple SNP test definition")
  
  as.integer(res)
}

## Estimation functions

## SNP on LHS  of equation


snp.lhs.estimates <-
  
function(snp.data, base.formula, add.formula, subset, snp.subset,
         data=sys.parent(), robust=FALSE, uncertain=FALSE,
         control=glm.test.control())
{   
  if (is(snp.data,"SnpMatrix")) 
    snames <- rownames(snp.data)
  else
    stop("snp.data must be stored as a SnpMatrix object")
  
  call <- match.call()
  temp <- c("", "base.formula", "subset", "data")
  m <- call[match(temp, names(call), nomatch=0)]
  data.missing <- is.null(m$data)
  special <- c("strata", "cluster")
  Terms <- terms(base.formula, special, data=data)

  # Separate intercept term from rest of model formula
  # Note as of now I ignore this and fit an intercept anyway!

  INTERCEPT <- attr(Terms, "intercept")
  if (!INTERCEPT) # put one back temporarily
    attr(Terms, "intercept") <- 1

  # Model frame for base model

  names(m)[2] <- "formula"
  m[[1]] <- as.name("model.frame")
  m[[2]] <- Terms
  m <- as.call(c(as.list(m), list(na.action=as.symbol("na.omit"))))
  m <- eval(m)
  bnames <- rownames(m)
 
  # Base model matrix -- minus any cluster or strata variables

  strats <- attr(Terms, "specials")$strata
  clust <- attr(Terms, "specials")$cluster
  dropx <- NULL
  if (!is.null(clust)) {
    # Some check on valid clusters?
    if (missing(robust))
      robust <- TRUE
    tempc <- untangle.specials(Terms, "cluster", 1)
    clust <- as.integer(cluster(m[, tempc$vars]))  # unused 'shortlabel=TRUE' was here - possibly a mistake
    dropx <- tempc$terms
  }
  if (!is.null(strats)) {
    temps <- untangle.specials(Terms, "strata", 1)
    strats <- as.integer(strata(m[, temps$vars], shortlabel=TRUE))
    dropx <- c(dropx, temps$terms)
  }

  if (length(dropx)) 
    newTerms <- Terms[-dropx]
  else
    newTerms <- Terms
  X <- model.matrix(newTerms, m)

  # Remove unit column vector
  
  ones <- match("(Intercept)", colnames(X), nomatch=0)
  if (ones)
    X <- X[,-ones, drop=FALSE]
  
  # Additional model matrix
  
  temp <- c("", "add.formula", "subset", "data")
  m <- call[match(temp, names(call), nomatch=0)]
  Terms <- terms(add.formula, data=data)
     
  names(m)[2] <- "formula"
  m[[1]] <- as.name("model.frame")
  m <- as.call(c(as.list(m), list(na.action=as.symbol("na.omit"))))
  m <- eval(m)
  anames <- rownames(m) 
  Z <- model.matrix(Terms, m)

  # If there is an intercept term in the model already, remove this from Z

  ones <- match("(Intercept)", colnames(Z), nomatch=0)
  if (ones)
    Z <- Z[,-ones, drop=FALSE]
 
  
  # Find common rows and mapping to line up  snp.data

  common <- intersect(anames, bnames)
  if (!data.missing) {
    common <- intersect(common, snames)
    ix <- match(common, bnames)
    iz <- match(common, anames)
    iy <- match(common, rownames(snp.data))
  }
  else {
    ix <- match(common, bnames)
    iz <- match(common, anames)
    iy <- as.numeric(common) # common will refer to row numbers in snp.data  
  }
  N <- as.integer(length(common))
  if (N==0)
    stop("no observations")
  else {
    snp.data <- snp.data[iy,]
    X <- X[ix,, drop=FALSE]
    if (!is.null(strats))
      strats <- strats[ix]
    if (!is.null(clust))
      clust <- clust[ix]
    Z <- Z[iz,, drop=FALSE]
  }

  if (missing(snp.subset)) {
    snp.subset <- NULL
  }
  else {
    if (is.character(snp.subset)) {
      snp.subset <- match(snp.subset, colnames(snp.data))
      if (any(is.na(snp.subset)))
        stop("unmatched snps in snp.subset argument")
    }
    else if (is.logical(snp.subset)){
      if (length(snp.subset)!=ncol(snp.data))
        stop("snp.subset element(s) out of range")
      snp.subset <- (1:ncol(snp.data))[snp.subset]
    }
    else if (is.integer(snp.subset)) {
      if (any(snp.subset<1 | snp.subset>ncol(snp.data)))
        stop("snp.subset element(s) out of range")
    }
    else 
      stop("illegal type for snp.subset")
  }
  .Call("snp_lhs_estimate",
        snp.data, cbind(X, Z), strats, snp.subset, ncol(Z), robust, clust,
        uncertain, control, PACKAGE="snpStats")
}
  
## SNP on RHS of equation

snp.rhs.estimates <-
  
function(formula, family="binomial", link, weights, subset,
        data=parent.frame(), snp.data, rules=NULL, sets=NULL, robust=FALSE, uncertain=FALSE,
        control=glm.test.control()) {
  
  call <- match.call()

  # Family and link function
  
  fam <- pmatch(tolower(family),
                c("binomial", "poisson", "gaussian", "gamma"))
  if (is.na(fam))
    stop("Unrecognized family argument")
  if (fam==0)
    stop("Ambiguous family argument")

  if (missing(link))
    lnk <- fam # Canonical link is default
  else {
    lnk <- pmatch(tolower(link), c("logit", "log", "identity", "inverse"))
    if (is.na(fam))
      stop("Unrecognized link argument")
    if (lnk==0)
      stop("Ambiguous link argument")
  }
  no.data <- missing(data)
  if (no.data) {
    data <- environment(formula)
  }
  temp <- c("", "formula", "data", "weights", "subset")
  m <- call[match(temp, names(call), nomatch=0)]
  special <- c("strata", "cluster")
    
  Terms <- if (no.data)
    terms(formula, special)
  else terms(formula, special, data=data)

  # Separate intercept term from rest of model formula

  INTERCEPT <- attr(Terms, "intercept")
  if (!INTERCEPT) # put one back temporarily
    attr(Terms, "intercept") <- 1

  # Model frame for null model

  m$formula <- Terms
  if (no.data)
    m$rows <- 1:nrow(snp.data)
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
        
  # Find common rows and mapping to line up  snp.data

  if (is(snp.data, "SnpMatrix")) 
    snames <- rownames(snp.data)
  else
    stop("snp.data must be stored as a SnpMatrix")

  if (no.data) {
    map <- m[["(rows)"]]
  }
  else {
    mnames <- rownames(m)
    map <- match(mnames, snames, nomatch=0)
  }
  use <- map>0
  N <- sum(use)
  if (N==0)
    stop("no observations")
  

  # Response vector

  if(is.factor(m[[1]])){
    # This is just to avoid the warning that conversion
    # to numeric(=double) from factor is ignored.
    Y <- model.response(m, "any")
  } else {
    Y <- model.response(m, "numeric")
  }
  if (fam==1) { # Binomial family
    Y <- as.factor(Y)
    if (nlevels(Y)==2)
      Y <- as.numeric(Y)-1
    else
      stop("Response is not on two levels")
  }
  Yname <- as.character(Terms, "variables")[[2]]

  # Model matrix -- minus any cluster or strata variables

  strats <- attr(Terms, "specials")$strata
  clust <- attr(Terms, "specials")$cluster
  dropx <- NULL
  if (length(clust)) {
    # Some check on valid clusters?
    if (missing(robust))
      robust <- TRUE
    tempc <- untangle.specials(Terms, "cluster", 1)
    clust <- strata(m[, tempc$vars], shortlabel=TRUE)
    dropx <- tempc$terms
  }
  else
    clust <- NULL
  if (length(strats)) {
    temps <- untangle.specials(Terms, "strata", 1)
    strats <- strata(m[, temps$vars], shortlabel=TRUE)
    dropx <- c(dropx, temps$terms)
  }
  else
    strats <- NULL

  if (length(dropx)) 
    newTerms <- Terms[-dropx]
  else
    newTerms <- Terms
  X <- model.matrix(newTerms, m)

  # Remove unit column vector
  
  ones <- match("(Intercept)", colnames(X), nomatch=0)
  if (ones)
    X <- X[, -ones, drop=FALSE]
  if (ncol(X) == 0)
    X <- NULL

  # Prior weights

  if (!is.null(model.weights(m))) {
    cat("One\n")
    weights <- model.weights(m)
    if (any(weights<0))
      stop("negative prior weights not allowed")
  }
  else
    weights <- NULL

  # Offsets

  offset <- model.offset(m)
  if(!is.null(offset)) {
    offset <- offset
    stop("Can't deal with offsets at the moment")
  }

  # Sort and reshape arrays if necessary

  if (N != length(use)) {
    Y <- Y[use]
    if (!is.null(X))
      X <- X[use,,drop=FALSE]
    if (!is.null(weights))
      weights <- weights[use]
    if (!is.null(strats))
      strats <- strats[use]
    if (!is.null(clust))
      clust <- clust[use]
    snp.data <- snp.data[map,]
  }
  else if (any(map != 1:N))
    snp.data <- snp.data[map,]
    
  
  # Coerce sets argument to correct form #

  if(is.null(sets)) {
    if (is.null(rules))
      sets <- as.integer(1:ncol(snp.data))
    else
      sets <- as.integer(-(1:length(rules)))
  }
  else if (is.logical(sets)) {
    sets <- as.integer(1:ncol(snp.data))[sets]
  }
  else if (is.character(sets)) {
    sets <- .col.numbers(sets, colnames(snp.data), names(rules))
  }
  else if (is.numeric(sets)) {
    sets <- as.integer(sets)
  }
  else if(is.list(sets)) {
    sets <- lapply(sets, .col.numbers, colnames(snp.data), names(rules))
  } else {
    stop("illegal sets argument")
  }
  attr(Y, "label") <- Yname
  .Call("snp_rhs_estimate", Y, fam, lnk, X, strats, snp.data, rules, 
        weights, sets, robust, clust, uncertain, control, PACKAGE="snpStats")
}  

  

mvtests <- function(phenotype, sets, stratum, data=sys.parent(), snp.data,
                     rules=NULL, complete=TRUE, uncertain=FALSE, score=FALSE) {
  m <- match.call()
  tmiss <- missing(sets)
  if (tmiss)
    sets <- NULL
  smiss <- missing(stratum)
  if (smiss)
    stratum <- NULL
  if (!is(snp.data, "SnpMatrix"))
    stop("snp.data argument must be of class SnpMatrix")
  nr.snps = nrow(snp.data)
  if (missing(data)) { ## phenotype and stratum are in calling environment
    if (is.factor(phenotype)) 
      phenotype <- contrasts(phenotype)[as.numeric(phenotype),, drop=FALSE]
    if (!is.matrix(phenotype))
      stop("Phenotype must be a factor or a matrix")
    if (nrow(phenotype)!=nr.snps)
      stop("incompatible number of rows for phenotype vector")
    if (!smiss) {
      stratum <- as.integer(as.factor(stratum))
      if (length(stratum)!=nr.snps)
        stop("incompatible length for stratum vector")
      sorder <- order(stratum)
      sorder[is.na(stratum)] <- 0
    }
    else {
      sorder <- 1:nr.snps
    }
  }
  else { # phenotype, stratum, and subset  are in data dataframe
    nm.snps <- rownames(snp.data)
    nm.data <- rownames(data)
    nr.snps < nrow(snp.data)
    phenotype <- eval(m$phenotype, envir=data)
    if (is.factor(phenotype)) 
      phenotype <- contrasts(phenotype)[as.numeric(phenotype),]
    if (!is.matrix(phenotype))
      stop("Phenotype must be a factor or a matrix")
    # which points to rows in the data matrix
    which <- match(nm.snps, nm.data, nomatch=NA)
    phenotype <- phenotype[which,]
    if (nrow(phenotype)!=nr.snps)
      stop("incompatible number of rows for phenotype vector")
    if (!smiss) {
      stratum <- as.integer(as.factor(eval(m$stratum, envir=data)))[which]
      sorder <- order(stratum)
      sorder[is.na(stratum)] <- 0
    }
    else {
      sorder <- 1:nr.snps
    }
  }
  if (class(snp.data)=="XSnpMatrix") {
    stop("Not yet implemented for SNPs on X chromosome")
    if(any(is.na(snp.data@diploid))) {
      warning("There are ", sum(is.na(snp.data@diploid)),
              " subjects with NA in the diploid slot. These were ignored")
      sorder[is.na(snp.data@diploid)] <- 0
    }
  }
    # Coerce sets argument to correct form #

  if(is.null(sets)) {
    sets <- as.integer(1:ncol(snp.data))
  } 
  else if (is.character(sets)) {
    sets <- .col.numbers(sets, colnames(snp.data))
  }
  else if (is.numeric(sets)) {
    sets <- as.integer(sets)
  }
  else if(is.list(sets)) {
    sets <- lapply(sets, .col.numbers, colnames(snp.data))
  } else {
    stop("illegal sets argument")
  }
  .Call("mvphen", phenotype, snp.data, rules, stratum, as.integer(sorder),
        sets, complete, uncertain, score, PACKAGE="snpStats")

}



                         

\documentclass[12pt]{article}
\usepackage{fullpage}
\usepackage{graphicx}

\usepackage[pdftex,
bookmarks,
bookmarksopen,
pdfauthor={David Clayton},
pdftitle={Fst calculations}]
{hyperref}

\title{Fst\\The algorithm used in {\tt snpStats}}
\author{David Clayton}
\date{\today}

\usepackage{Sweave}
\SweaveOpts{echo=TRUE, pdf=TRUE, eps=FALSE}

\begin{document}
\setkeys{Gin}{width=1.0\textwidth}

%\VignetteIndexEntry{Fst}
%\VignettePackage{snpStats}

\maketitle

<<lib,echo=FALSE>>=
require(snpStats)
@ 
\section*{$F$ statistics for diversity of groups}
There is a very large literature on this topic and the author does not claim
any great expertise. The purpose of this vignette is simply to document the 
method of calculation implemented in {\tt snpStats}.

We shall start by introducing some notation. Let:\\[5mm]
\begin{tabular}[h]{ll}
  $N_g$ &  Number of chromosomes in group $g$\\
  $N_{sg}$ & Number of chromsomes in group $g$ observed for SNP $s$\\
  $N_s$ & Number of chromosomes observed for SNP $s$, all groups\\
  $p_{sg}$ & Allele (relative) frequency for SNP $s$ in group $g$\\
  $p_s$ & Overall allele frequency for SNP $s$\\
\end{tabular}\\[5mm]
and let:
\begin{eqnarray*}
  Y_s &=& \frac{N_s}{N_s-1}p_s(1-p_s)\\
  X_{sg} &=& \frac{N_{sg}}{N_{sg}-1} p_{sg}(1-p_{sg})\\
  X_g &=& \sum_g W_g X_{sg}
\end{eqnarray*}
where $W_g$ are group-specific weights (see below).

The value returned for the F statistic for SNP $s$ is
\[
F_s = 1 - \frac{X_s}{Y_s} = \frac{Y_s - X_s}{Y_s}.
\]

A natural combined value over all SNPs is obtained by summing numerators and 
denominators of teh SNP-specific values Denoting summation by a $+$ subsscript,
\[
F = \frac{Y_+ - X_+}{Y_+},
\]
which can also be written as a weighted mean of the SNP-specific values, 
with $Y_s$ as weights:
\[
F = \frac{1}{\sum_s Y_s} \sum_s Y_s F_s.
\]

There appear to be two ways of looking at this index, leading to different 
weights when group sizes are unequal. 

\subsection*{Pairwise differences}
One rationale suggests that the index can be written 
\[
\frac{D_T - D_W }{D_T }
\]
where $D_T$ is the probability that two chromosomes, sampled at random
from the total population, differ and $D_W$ is the probability that two
chromosomes, sampled at random from the same subpopulation,
differ. For a single SNP, $s$, $Y_s$ is an unbiased estimator for $D_T$
and $X_{sg}$ is an unbiased estimator for $D_W$ {\em within subgroup $g$}. 
With this rationale, the weights,
$\{W_g\}$ should reflect the numbers of distinct pairwise comparisons within
each group:
\[
W_g = \frac{N_g (N_g-1)}{\sum_g N_g(N_g-1)}
\]
\subsection*{The analysis of variance}
Another rationale would seem to be in terms of partition of the total
variance, specifically the ratio of the between-group variance to the
total variance (this seems to be the thrust of a series of papers by
Cockerham). $Y_s$ is then an unbiased estimate of the total variance of SNP 
$s$ and $X_{sg}$ is an unbiased estimator of its variance in group $g$. But 
the natural weights are then
\[
W_g =  \frac{N_g }{\sum_g N_g}.
\]
(Cockerham also seems to have considered
an unweighted analysis but that could be inefficient if group sizes differ 
substantially).
\section*{An example}
Here we show the results of these calculations using the HapMap data discussed 
in other vignettes. These data were constructed by re-sampling individuals from
two groups of HapMap subjects, the CEU sample (of European
origin) and the JPT$+$CHB sample (of Asian origin), these groups being 
identified by the variable {\tt stratum} in the subject support data frame.

We first use the pair-wise difference weights, first calculating the 
SNP-specific values, followed by the weighted average across all SNPs:
<<pairwise>>=
data(for.exercise)
f1 <- Fst(snps.10, subject.support$stratum, pairwise=TRUE)
weighted.mean(f1$Fst, f1$weight)
@
We now compare this result with that obtained with the alternative (AOV) 
weights: 
<<aov>>=
f2 <- Fst(snps.10, subject.support$stratum, pairwise=FALSE)
weighted.mean(f2$Fst, f2$weight)
@
Here there is little difference between the two values, since the group sizes 
are nearly the same:
<<groupsize>>=
table(subject.support$stratum)
@
In other cases the two weighting schemes could lead to different answers. In 
such situations, the preference of this author is for the analysis of variance 
weights and, accordingly, this has been set as the default action.

\end{document}

%\documentclass[a4paper,12pt]{article}
\documentclass[12pt]{article}
\usepackage{fullpage}
% \usepackage{times}
%\usepackage{mathptmx}
%\renewcommand{\ttdefault}{cmtt}
\usepackage{graphicx}

\usepackage[pdftex,
bookmarks,
bookmarksopen,
pdfauthor={David Clayton},
pdftitle={TDT and snpStats Vignette}]
{hyperref}

\title{TDT vignette\\Use of snpStats in family--based studies}
\author{David Clayton}
\date{\today}

\usepackage{Sweave}
\SweaveOpts{echo=TRUE, pdf=TRUE, eps=FALSE}

\begin{document}
\setkeys{Gin}{width=1.0\textwidth}

%\VignetteIndexEntry{TDT tests}
%\VignettePackage{snpStats}

\maketitle


\section*{Pedigree data}

The {\tt snpStats} package contains some tools for analysis of
family-based studies. These assume that a subject support file
provides the information necessary to reconstruct pedigrees in the
well-known format used in the {\it LINKAGE} package. Each line of the
support file  must contain an identifier of the {\em pedigree} to which
the individual belongs, together with an identifier of subject within pedigree,
and the within-pedigree identifiers for the subject's father and
mother. Usually this information, together with phenotype data, will
be contained in a dataframe with rownames which link to the rownames
of the {\tt SnpMatrix} containing the genotype data. The following
commands read some illustrative data on 3,017 subjects and 43
(autosomal) SNPs\footnote{These data are on a much smaller scale than
  would arise in genome-wide studies, but serve to illustrate the
  available tools. Note, however, that execution speeds are quite adequate for
  genome-wide data.}. The data consist of a dataframe containing the
subject and pedigree information ({\tt pedData}) and a {\tt
  SnpMatrix} containing the genotype data ({\tt genotypes}):
<<family-data>>=
require(snpStats)
data(families)
genotypes
head(pedData)
@ 
The first family comprises four individuals: two parents and two
sibling offspring. The parents are ``founders'' in the pedigree, {\it
  i.e.}  there is no data for their parents, so that their {\tt father}
and {\tt mother} identifiers are set to {\tt NA}. This differs from
the convention in the {\it LINKAGE} package, which would code these as
zero. Otherwise coding is as in {\it LINKAGE}: {\tt sex} is coded 1 for
  male and 2 for female, and disease status ({\tt affected}) is coded
  1 for unaffected and 2 for affected.
  
\section*{Checking for mis-inheritances}

The function {\tt misinherits} counts non-Mendelian inheritances in
the data. It returns a logical matrix with one row for each subject
who has any mis-inheritances and one column for each SNP which was ever
mis-inherited. 
<<mis-inheritances>>=
mis <- misinherits(data=pedData, snp.data=genotypes)
dim(mis)
@ 
Thus, 114 of the subjects and 37 of the SNPs had at least one
mis-inheritance. The following commands count mis-inheritances per
subject and plot its frequency distribution, and similarly,
for mis-inheritances per SNP:

<<per-subj-snp,fig=TRUE>>=
per.subj <- apply(mis, 1, sum, na.rm=TRUE)
per.snp <- apply(mis, 2, sum, na.rm=TRUE)
par(mfrow = c(1, 2))
hist(per.subj,main='Histogram per Subject', xlab='Subject')
hist(per.snp,main='Histogram per SNP', xlab='SNP')
@ 

Note that mis-inheritances must be ascribed to offspring, although the
error may lie with the parent data. The following commands first
extract the pedigree identifiers for mis-inheriting subjects and go on
to chart the numbers of mis-inheritances per family:
<<per-family,fig=TRUE>>=
fam <- pedData[rownames(mis), "familyid"]
per.fam <- tapply(per.subj, fam, sum)
par(mfrow = c(1, 1))
hist(per.fam, main='Histogram per Family', xlab='Family')
@ 
None of the above analyses suggest serious problems with the data,
although there are clearly a few genotyping errors.

\section*{TDT tests}

At present, the package only allows testing of
discrete disease phenotypes in case--parent trios --- basically the
Transmission/Disequilibrium Test (TDT). This is carried out by the
function {\tt tdt.snp}, which returns the same class of object as that
returned by {\tt single.snp.tests}; allelic (1 df) and genotypic
(2~df) tests are computed. The following commands compute the tests,
display the $p$-values, and plot quantile--quantile plots of the 1~df tests
chi-squared statistics:
<<tdt-tests,fig=TRUE,keep.source=TRUE>>=
tests <- tdt.snp(data = pedData, snp.data = genotypes)
cbind(p.values.1df = p.value(tests, 1),
      p.values.2df = p.value(tests, 2))
qq.chisq(chi.squared(tests, 1), df = 1)
@ 
Since these SNPs were all in a region of known association, the
overdispersion of test statistics is not surprising. Note that,
because each family had two affected offspring, there were twice as
many parent-offspring trios as families. In the above tests, the
contribution of the two trios in each family to the test statistic
have been assumed to be independent. When there is {\em linkage}
between the genetic locus and disease trait, this assumption is
incorrect and an alternative variance estimate can be used by
specifying {\tt robust=TRUE} in the call. However, in practice,
linkage is very rarely strong enough to require this correction.
\end{document}

%\documentclass[a4paper,12pt]{article}
\documentclass[12pt]{article}
\usepackage{fullpage}
% \usepackage{times}
%\usepackage{mathptmx}
%\renewcommand{\ttdefault}{cmtt}
\usepackage{graphicx}

\usepackage[pdftex,
bookmarks,
bookmarksopen,
pdfauthor={David Clayton},
pdftitle={Imputed SNP analyses with snpStats}]
{hyperref}

\title{Imputed SNP analyses and meta-analysis with snpStats}
\author{David Clayton}
\date{\today}

\usepackage{Sweave}
\SweaveOpts{echo=TRUE, pdf=TRUE, eps=FALSE}

\begin{document}
\setkeys{Gin}{width=1.0\textwidth}
%\VignetteIndexEntry{Imputation and meta-analysis}
%\VignettePackage{snpStats}

\maketitle

% R code as
%<<label[,fig=TRUE]>>=
%
%@ 

\section*{Getting started}
The need for imputation in SNP analysis studies occurs when we have a
smaller set of samples in which a large number of SNPs have been
typed, and a larger set of samples typed in  
only a subset of the SNPs. 
We use the smaller, complete dataset (which will be termed the
{\em training dataset}) to impute the missing SNPs in the larger,
incomplete dataset (the {\em target dataset}). Examples of such applications 
include:
\begin{itemize}
\item use of HapMap data to impute association tests for a large
  number of SNPs, given data from genome-wide studies using, for
  example, a 500K SNP array, and
\item meta-analyses which seek to combine results from two platforms
  such as the Affymetrix 500K and Illumina 550K platforms.
\end{itemize}
Here we will not use a real example such as the above to explore the
use of {\tt snpStats} for imputation, 
but generate a fictitious example using the
data analysed in earlier exercises. This is particularly artificial in
that we have seen that these data suffer from extreme heterogeneity of 
population structure. 

We start by attaching the required libraries and accessing the data
used in the exercises:

<<init>>=
library(snpStats)
library(hexbin)
data(for.exercise)
@ 

We shall sample 200 subjects in our fictitious study as the
training data set, select every second SNP to be missing 
in the target dataset, and split the training set
into two parts accordingly:

<<select>>= 
training <- sample(1000, 200)
select <- seq(1, ncol(snps.10),2) 
missing <- snps.10[training, select]
present <- snps.10[training, -select] 
missing 
present 
@
Thus the training dataset consists of the objects {\tt missing} and
{\tt present}. The target dataset holds a subset of the SNPs 
for the remaining 800 subjects. 

<<target>>=
target <- snps.10[-training, -select]
target
@ 
But, in order to see how successful we
have been with imputation, we will also save the SNPs we have removed from the
target dataset
<<>>=
lost <- snps.10[-training, select]
lost
@ 
We also need to know where the SNPs are on the chromosome in order to
avoid having to search the entire chromosome for suitable predictors
of a missing SNP:

<<positions>>=
pos.miss <- snp.support$position[select]
pos.pres <- snp.support$position[-select]
@ 

\section*{Calculating the imputation rules}

The next step is to calculate a set of rules which
for imputing the {\tt missing} SNPs from the {\tt
  present} SNPs. This is carried out by the function 
  {\tt snp.imputation}\footnote{Sometimes this command generates a
    warning message concerning the maximum number of EM iterations. If this 
    only concerns a small proportion of the SNPs to be imputed it can be 
    ignored.}:

<<rules>>=
rules <- snp.imputation(present, missing, pos.pres, pos.miss)
@ 

This step executes remarkably quickly  when we
consider what the function has done. For each of the \Sexpr{length(select)} 
SNPs in
the ``missing'' set, the function has performed a forward step-wise
regression on the 50 nearest SNPs in the ``present'' set, stopping each
search either when the $R^2$ for prediction exceeds 0.95, or after
including 4 SNPs in the regression, or until $R^2$ is not improved by 
at least 0.05. The figure 50 is the default value of the {\tt try} argument 
of the function, while the values 0.95, 4 and 0.05 together make up the 
default value of the {\tt stopping} argument. After the predictor, or
``tag'' SNPs
have been chosen, the haplotypes of the target SNP plus tags was phased
 and haplotype frequencies calculated using the EM algorithm. These
 frequencies were then stored in the {\tt rules} object.
\footnote{For imputation from small samples, some smoothing of these
  haplotype frequencies would be advantageous and some ability to do
  this has been included. The {\tt use.haps} argument to {\tt
    snp.imputation} controls this. But invoking this option 
  slows down the algorithm and it is not
  advised other than for very small sample sizes.}

A short listing of the first 10 rules follows:
<<rule1>>=
rules[1:10]
@ 
The rules are also selectable by SNP name for detailed examination:
<<rule2>>=
rules[c('rs11253563', 'rs2379080')]
@
Rules are shown with a {\tt +} symbol separating
predictor SNPs. (It is important to know which SNPs were used for each
imputation when checking imputed test results for artifacts.)

A summary table of all the 14,251 rules is generated by 

<<summary>>=
summary(rules)
@ 

Columns represent the number of tag SNPs while rows
represent grouping on $R^2$. The last column (headed {\tt <NA>})
represents SNPs for which an imputation rule could  not be computed,
either because they were monomorphic or because there was insufficient
data (as determined by the {\tt minA} optional argument in the call to
{\tt snp.imputation}). The same information may be displayed graphically by

<<ruleplot,fig=TRUE>>=
plot(rules)
@ 


\section*{Carrying out the association tests}

The association tests for imputed SNPs can be carried out using the
function {\tt single.snp.tests}. 

<<imptest>>=
imp <- single.snp.tests(cc, stratum, data=subject.support,
                        snp.data=target, rules=rules)
@ 

Using the observed data in the matrix {\tt target} and the set of
imputation rules stored in {\tt rules}, the above
command  imputes each of the imputed SNPs, carries out 1- and 2-df
single locus tests for association,  returns the results in the object {\tt
  imp}. To see how successful imputation has been, we can carry out
the same tests using the {\em true} data in {\tt missing}:

<<realtest>>= 
obs <- single.snp.tests(cc, stratum, data=subject.support, snp.data=lost)
@ 

The next commands extract the $p$-values for the 1-df tests, using both
the  imputed and the true ``missing'' data, and plot one against the
other (using the {\tt hexbin} plotting package for clarity):

<<compare,fig=TRUE>>=
logP.imp <- -log10(p.value(imp, df=1))
logP.obs <- -log10(p.value(obs, df=1))
hb <- hexbin(logP.obs, logP.imp, xbin=50)
sp <- plot(hb)
hexVP.abline(sp$plot.vp, 0, 1, col="black")
@ 

As might be expected, the agreement is rather better if we only
compare the results for SNPs that can be computed with high $R^2$. The
$R^2$ value is extracted from the {\tt rules} object, using the
function {\tt imputation.r2} and used to select a subset of rules:

<<best,fig=TRUE>>=
use <- imputation.r2(rules)>0.9
hb <- hexbin(logP.obs[use], logP.imp[use], xbin=50)
sp <- plot(hb)
hexVP.abline(sp$plot.vp, 0, 1, col="black")
@ 

Similarly, the function {\tt imputation.maf} can be used to extract
the minor allele frequencies of the imputed SNP from the {\tt rules}
object. Note that there is a tendency for SNPs with a high minor allele
frequency to be imputed rather more successfully:

<<rsqmaf,fig=TRUE>>=
hb <- hexbin(imputation.maf(rules), imputation.r2(rules), xbin=50)
sp <- plot(hb)
@ 

The function {\tt snp.rhs.glm} also allows testing imputed SNPs. In
its simplest form, it can be used to calculate essentially the same
tests as carried out with 
{\tt single.snp.tests}\footnote{There is a small discrepancy, of the 
  order of $(N-1):N$ .}
(although, being a more flexible function, this will run
  somewhat slower). The next commands recalculate the 1 df tests for
  the imputed SNPs using {\tt snp.rhs.tests}, and plot the results
  against those obtained when values are observed.
<<imptest-rhs,fig=TRUE>>=
imp2 <- snp.rhs.tests(cc~strata(stratum), family="binomial", 
                      data=subject.support, snp.data=target, rules=rules)
logP.imp2 <- -log10(p.value(imp2))
hb <- hexbin(logP.obs, logP.imp2, xbin=50)
sp <- plot(hb)
hexVP.abline(sp$plot.vp, 0, 1, col="black")
@ 

\section*{Storing imputed genotypes}

In the previous two sections we have seen how to (a) generate imputation
rules and, (b) carry out tests on SNPs imputed according to these
rules, but without storing the imputed genotypes. It is also possible
to store imputed SNPs in an object of class {\tt SnpMatrix} (or {\tt
  XSnpMatrix}). The posterior probabilities of assignment of each
individual to the three possible genotypes are stored within a
one byte variable, although obviously not to full accuracy. 

The following command imputes the ``missing'' SNPs using the
``target''  dataset and stores the imputed values in an object of class 
{\tt SnpMatrix}:
<<impstore>>=
imputed <- impute.snps(rules, target, as.numeric=FALSE)
@ 
(If {\tt as.numeric} were set to {\tt TRUE}, the default, the
resulting object would be a simple numeric matrix containing posterior
expectations of the 0, 1, 2 genotype.) A nice graphical description of
how {\tt snpStats} stores uncertain genotypes is provided by the
function {\tt plotUncertainty}. This plots the frequency of the stored
posterior probabilities on an equilateral triangle. The posterior
probabilities are represented by the perpendicular distances from each
side, the vertices of the triangle corresponding to certain
assignments. Thus, the SNP {\tt rs4880568} is accurately imputed
($R^2 = 0.94$)
<<uncert1,fig=TRUE>>=
plotUncertainty(imputed[, "rs4880568"])
@ 
while {\tt rs2050968} is rather less so ($R^2 = 0.77$
<<uncert2,fig=TRUE>>=
plotUncertainty(imputed[, "rs2050968"])
@ 

Tests can be carried out on these uncertainly assigned genotypes. For example
<<imptest2>>=
imp3 <- single.snp.tests(cc, stratum, data=subject.support,
                        snp.data=imputed, uncertain=TRUE)
@ 
The {\tt uncertain=TRUE} argument ensures that uncertaing genotypes
are used in the computations. This should yield nearly the same result 
as before. For the first five SNPs we have 
<<imp3>>=
imp3[1:5]
imp[1:5]
@ 
There are small discrepancies due to the genotype assignment probabilities not
being stored to full accuracy. However these should have little effect
on power of the tests and no effect on the type~1 error rate.

Note that the ability of {\tt snpStats} to store imputed genotypes in
this way allows alternative  programs to be used to generate
the imputed genotypes. For example, the file ``mach1.out.mlprob.gz''
(which is stored in the {\tt extdata} sub-directory of 
the {\tt snpStats} package)
contains imputed SNPs generated by the MACH program, using the {\tt
  --mle} and {\tt --mldetails} options. In the following commands, we
find the full path to this file, read it,  and inspect one the imputed
SNP in column~50:   
<<mach,fig=TRUE>>=
path <- system.file("extdata/mach1.out.mlprob.gz", package="snpStats")
mach <- read.mach(path)
plotUncertainty(mach[,50])
@ 
\section*{Meta-analysis}
As stated at the beginning of this document, one of the main reasons
that we need imputation is to perform meta-analyses which bring
together data from genome-wide studies which use different platforms. 
The {\tt snpStats} package includes a number of tools to facilitate
this. All the tests implemented in {\tt snpStats} are ``score''
tests. In the 1 df case we calculate a score defined by 
the first derivative of the log
likelihood function with respect to the association parameter of
interest at the parameter value corresponding to the null hypothesis
of no association. Denote this by $U$. We also calculate an estimate
of its variance,  also under the null hypothesis --- $V$ say. Then
$U^2/V$ provides the chi-squared test on 1~df. This procedure extends
easily to meta-analysis; given two independent studies of the same
hypothesis, we simply add together the two values of $U$ and the two
values of $V$, and then calculate $U^2/V$ as before. These ideas also
extend naturally to tests of several parameters (2 or more df tests).

In {\tt snpStats}, the statistical testing functions can be
called with the option {\tt score=TRUE}, causing an extended object to
be saved. The extended object contains the $U$ and $V$ values, thus
allowing later combination of the evidence from different
studies. We shall first see what sort of object we have calculated
previously using {\tt
  single.snp.tests} {\em without} the {\tt score=TRUE} argument.
<<class-imp-obs>>=
class(imp)
@ 
This object contains the imputed SNP tests in our target set. However,
these SNPs were observed in our training set, so we can test
them. We will also recalculate the imputed tests. In both cases we
will save the score information:
<<save-scores>>=
obs <- single.snp.tests(cc, stratum, data=subject.support, snp.data=missing, 
                        score=TRUE)
imp <- single.snp.tests(cc, stratum, data=subject.support,
                        snp.data=target, rules=rules, score=TRUE)
@
The extended objects have been returned:
<<>>=
class(obs)
class(imp)
@ 
These extended
objects behave in the same way as the original objects, so that the
same functions can be used to extract chi-squared values, $p$-values
etc., but several additional functions, or methods, are now
available. Chief amongst these is {\tt pool}, which combines evidence
across independent studies as described at the beginning of this
section. Although {\tt obs} and {\tt imp} are {\em not} from
independent studies, so that the resulting test would not be valid, we
can use them to demonstrate this:
<<pool>>=
both <- pool(obs, imp)
class(both)
both[1:5]
@ 
Note that if we wished at some later stage to combine the
results in {\tt both} with a further study, we would also need to
specify {\tt score=TRUE} in the call to {\tt pool}:
<<pool-score>>=
both <- pool(obs, imp, score=TRUE)
class(both)
@

Another reason to save the score statistics is that this allows us to
investigate the {\em direction} of findings. These can be extracted
from the extended objects using the function {\tt effect.sign}. For
example, this command tabulates the signs of the associations in {\tt obs}:
<<sign>>=
table(effect.sign(obs))
@ 
In this table, -1 corresponds to tests in which 
effect sizes were negative (corresponding to an
odds ratio less than one), while +1 indicates positive effect sizes
(odds ratio greater than one). Zero sign indicates that 
the effect was {\tt NA} (for example
because the SNP was monomorphic). 
Reversal of sign can be the explanation of a puzzling phenomenon when
two studies give significant results individually, but no significant
association when pooled. Although it is not impossible that such
results are genuine, a more usual explanation is that the two alleles
have been coded differently in the two studies: allele~1 in the first
study is allele~2 in the second study and vice versa. To allow for
this, {\tt snpStats} provides the {\tt switch.alleles} function, which
reverses the coding of specified SNPs. It can be applied to {\tt
  SnpMatrix} objects but, because allele switches are often
discovered quite late on in the analysis and recoding the original
data matrices could have unforeseen consequences, the {\tt
  switch.alleles} function can also be applied to the extended test
output objects. This modifies the saved scores {\em as if} the allele
coding had been switched in the original data. The use of this is
demonstrated below.
<<switch>>=
effect.sign(obs)[1:6]
sw.obs <- switch.alleles(obs, 1:3)
class(sw.obs)
effect.sign(sw.obs)[1:6]
@ 
\end{document}

\documentclass[12pt]{article}
\usepackage{fullpage}
% \usepackage{times}
%\usepackage{mathptmx}
%\renewcommand{\ttdefault}{cmtt}
\usepackage{graphicx}

\usepackage[pdftex,
bookmarks,
bookmarksopen,
pdfauthor={David Clayton},
pdftitle={PCA-snpStats Vignette}]
{hyperref}

\title{PCA vignette\\Principal components analysis with snpStats}
\author{David Clayton}
\date{\today}

\usepackage{Sweave}
\SweaveOpts{echo=TRUE, pdf=TRUE, eps=FALSE}

\newcommand{\tr}{^{\mbox{\scriptsize T}}}

\begin{document}
\setkeys{Gin}{width=1.0\textwidth}

%\VignetteIndexEntry{Principal components analysis}
%\VignettePackage{snpStats}

\maketitle

Principal components analysis has been widely used in population
genetics in order to study population structure in genetically
heterogeneous populations. More recently, it has been proposed as a
method for dealing with the problem of confounding by population
structure in genome-wide association studies. 
\section*{The maths}
Usually, principal components analysis is carried out by calculating
the eigenvalues and eigenvectors of the correlation matrix. With $N$
cases and $P$ variables, if we write $X$ for the $N\times P$ matrix
which has been standardised so that columns have zero mean and unit
standard deviation, we find the eigenvalues and eigenvectors of
the $P\times P$ matrix $X\tr.X$ (which is $N$ or $(N-1)$ times the
correlation matrix depending on which denominator was used when
calculating standard deviations). The first eigenvector gives the
loadings of each variable in the first principal component, the second
eigenvector gives the loadings in the second component, and so
on. Writing the first $C$ component loadings as columns of the $P\times
C$ matrix $B$, the $N\times C$ matrix of subjects' principal component
scores, $S$, is obtained by applying
the factor loadings to the original data matrix, {\it i.e.\,}  $S = X.B$. 
The sum of squares and products matrix, $S\tr.S = D $, is diagonal
with elements equal to the first $C$  
eigenvalues of the  $X\tr.X$ matrix, so that the variances of the
principal components  can obtained by dividing the eigenvalues by 
$N$ or $(N-1)$.

This standard method is rarely feasible for genome-wide data since $P$
is very large indeed and calculating the eigenvectors of $X\tr.X$
becomes impossibly onerous. However, the calculations can also be
carried out by calculating the eigenvalues and eigenvectors of the
$N\times N$ matrix $X.X\tr$. 
The (non-zero) eigenvalues of this matrix are the same as those of
$X\tr.X$, and its eigenvectors are proportional to
the principal component scores defined above; writing the first $C$
eigenvectors of $X.X\tr$ as 
the columns of the $N\times C$ matrix, $U$, then $U =  S.D^{-1/2}$.
Since for many purposes we are not too
concerned about the scaling of the principal components, it will
often be acceptable to use the eigenvectors, $U$, in place of the
more conventionally scaled principal components. However some
attention should be paid to the corresponding eigenvalues since, as
noted above, these are proportional to the variances of the
conventional principle components. The factor loadings may be
calculated by $B = X\tr.U.D^{-1/2}$.

Using this method of
calculation, it is only (!) necessary to find the eigenvalues and eigenvectors
of an $N\times N$ matrix. Current microarray-based genotyping studies
are such that $N$ is typically a few thousands while $P$ may be in
excess of one million.

\section*{An example}

In this exercise, we shall calculate principal component loadings in
controls alone and then apply these loading to the whole data. This is
more complicated than the simpler procedure of calculating principal
components in the entire dataset but avoids component loadings which
unduly reflect case/control differences; using such components to
correct for population structure would seriously reduce the power to
detect association since one would, to some extent, be ``correcting''
for case/control differences\footnote{An alternative approach is to
  standardise the $X$ matrix so that each column has zero mean in
  both cases and controls. This can be achieved by using the {\tt
    strata} argument in the call to {\tt xxt}. Here, however, we have
  used controls only since this reduces the size of the matrix for
  the eigenvalue and vector calculations.
}. We will also ``thin'' the data by taking
only every tenth SNP. We do this mainly to reduce computation time but
thinning is often employed to minimize the impact of linkage
disequilibrium (LD), to reduce the risk that the larger 
components may simply reflect unusually long stretches of LD rather
than population structure. Of course, this would require a more
sophisticated approach to thinning than that used in this demonstration.

In a more sophisticated approach, one might use the output of
{\tt snp.imputation} to eliminate all but one of a groups of SNPs in strong LD
for thinning.

We shall use the data introduced in the main vignette. We shall first
load the data and extract the controls. 
<<get-data>>=
require(snpStats)
data(for.exercise)
controls <- rownames(subject.support)[subject.support$cc==0]
use <- seq(1, ncol(snps.10), 10)
ctl.10 <- snps.10[controls,use]
@ 
The next step is to standardize the data to zero mean and unit
standard deviation and to calculate the $X.X\tr$ matrix. These
operations are carried out using the function {\tt xxt}.
<<xxt-matrix>>=
xxmat <- xxt(ctl.10, correct.for.missing=FALSE)
@ 
The argument {\tt correct.for.missing=FALSE} selects a very simple
missing data treatment, {\it i.e.\,} replacing missing values by their
mean. This is quite adequate when the proportion of missing data is
low. The default method is more sophisticated but introduces
complications later so we will keep it simple.

When performing a genome-wide analysis, it will usually be the case
that all the data cannot be stored   
in a single {\tt SnpMatrix} object. Usually they will
be organized with one matrix for each chromosome. In these cases, it
is straightforward to write a script which carries out the above
calculations for each chromosome in turn, saving the resultant matrix
to disk each time. When all chromosomes have been processed, the
$X.X\tr$ matrices are read and added together.

The next step of the calculations requires us to calculate the
eigenvalues and eigenvectors of the $X.X\tr$ matrix. This can be
carried out using a standard R function. We will save the first five
components. 
<<eigen>>=
evv <- eigen(xxmat, symmetric=TRUE)
pcs <- evv$vectors[,1:5]
evals <- evv$values[1:5]
evals
@ 
Here, {\tt pcs} refers to the scaled principal components ({\it i.e.\,}
to the columns of the matrix $U$ in our mathematical introduction) and
all have the same variance. The eigenvalues give an idea of the
relative magnitude of these sources of variation.
The first principal component has a markedly larger eigenvalue and we
might hope that this reflects population structure. In fact these data
were drawn from two very different populations, as indicated by the
{\tt stratum} variable in the subject support frame. The next set of
commands extract this variable for the controls and plot box plots
for the first two components by stratum.
<<pc-one,fig=TRUE>>=
pop <- subject.support[controls,"stratum"]
par(mfrow=c(1,2))
boxplot(pcs[,1]~pop)
boxplot(pcs[,2]~pop)
@ 
Clearly the first component has captured the difference between the
populations. Equally clearly, the second principal component has not.

The next step in the calculation is to obtain the SNP loadings in the
components. This requires calculation of $B = X\tr.S.D^{-1/2}$. Here
we calculate the transpose of this matrix, $B\tr = D^{-1/2}S\tr.X$,
using the special function {\tt snp.pre.multiply} which pre-multiplies a {\tt
  SnpMatrix} object by a matrix after first standardizing it to zero
mean and unit standard deviation. 
<<pre-multiply>>=
btr <- snp.pre.multiply(ctl.10, diag(1/sqrt(evals)) %*% t(pcs))
@ 
We can now apply these loadings back to the entire dataset (cases as
well as controls) to derive scores that we can use to correct for
population structure. To do that we use the function {\tt snp.post.multiply}
which post-multiplies a {\tt SnpMatrix} by a general matrix, after first
standardizing the columns of the {\tt SnpMatrix} 
to zero mean and unit standard deviation. Note that it
is first necessary to select out those SNPs that we have actually used
in the calculation of components.
<<post-multiply>>=
pcs <- snp.post.multiply(snps.10[,use], t(btr))
@ 

Finally we shall evaluate how successful the first principal component
is in correcting for population structure effects. 
({\tt snp.rhs.tests} return {\tt glm} objects.)
<<testing,fig=TRUE>>=
cc <- subject.support$cc
uncorrected <- single.snp.tests(cc, snp.data=snps.10)
corrected <- snp.rhs.tests(cc~pcs[,1], snp.data=snps.10)
par(mfrow=c(1,2),cex.sub=0.85)
qq.chisq(chi.squared(uncorrected,1), df=1)
qq.chisq(chi.squared(corrected), df=1)
@ 
The use of the first principal component as a covariate has been
quite successful in reducing the serious over-dispersion due to
population structure. Indeed it is just as successful as
stratification by the observed {\tt stratum} variable.
\subsection*{Acknowledgement}
Thanks to David Poznik for pointing out an error in the earlier
versions of this vignette.
\end{document}

\documentclass[12pt]{article}
\usepackage{fullpage}
\usepackage{graphicx}

\usepackage[pdftex,
bookmarks,
bookmarksopen,
pdfauthor={David Clayton},
pdftitle={Data input}]
{hyperref}

\title{Data input vignette\\Reading genotype data in {\tt snpStats}}
\author{David Clayton}
\date{\today}

\usepackage{Sweave}
\SweaveOpts{echo=TRUE, pdf=TRUE, eps=FALSE}

\begin{document}
\setkeys{Gin}{width=1.0\textwidth}

%\VignetteIndexEntry{Data input}
%\VignettePackage{snpStats}

\maketitle

<<lib,echo=FALSE>>=
require(snpStats)
@ 
\section*{Memory limitations}
Before we start it is important to emphasise that the {\tt SnpMatrix}
objects that hold genotype data in {\tt snpStats} 
are resident in memory, and limitations
of the computer and of the R language impose limits on the maximum
size of datasets that can be held at any one time. Each genotype
reading uses only a single byte of memory so that large datasets can
be read given the large memory capacity of modern
computers. Originally, 
R imposed a limit of  $2^{31}-1 \sim 2\times 10^9$ elements in a single
array. This limit applied in both the 32-bit and 64-bit versions
  of R, these versions differing only in the {\em total} memory that could
  be used. For example, this would correspond to one million loci for two
thousand subjects and would occupy two gigabytes of machine
memory. However, version 3 of R removed the restriction on single
arrays in the 64-bit version,  and this was implemented for
{\tt SnpMatrix} and {\tt XSnpMatrix} objects in version
1.19.2 of {\tt snpStats}. However, experience of this code is limited
and some caution is advised.
 
\section*{Reading pedfiles}
A commonly encountered format for storing genotype data is the
``pedfile'' format, which originated some years ago  in the LINKAGE
package. Pedfiles are text files containing one line per genotyped
sample, with fields separated by ``white space'' ( TAB
characters or SPACEs). The first six fields contain:
\begin{enumerate}
\item a pedigree or family identifier, unique to the family of which
  this subject is a member,
\item a further identifier, unique (within the family) to each family member,
\item the member identifier of the father of the subject if the father
  is also present in the data, otherwise an arbitrary code (usually
  {\tt 0}),
\item similarly, an identifier for the mother of the subject,
\item the sex of the subject ({\tt 1 =} Male, {\tt 2 =} Female), and
\item a binary trait indicator ({\tt 1 =} Absent, {\tt 2 =} Present).
\end{enumerate}
Missing values in the last two fields are usually coded as zero. 

The first few rows and columns of a sample file is shown below:
\begin{verbatim}
IBD054	430	0	0	1	0	1  3	3  1	4  1	4  2
IBD054	412	430	431	2	2	1  3	1  3	4  1	4  2
IBD054	431	0	0	2	0	3  3	3  3	1  1	2  2
IBD058	438	0	0	1	0	3  3	3  3	1  1	2  2
IBD058	470	438	444	2	2	3  3	3  3	1  1	2  2
\end{verbatim}
Thus, the subject of line~2 has a father whose data appears on line~1
and a mother whose data is on line~3. The grandparents do not appear
on the file. This subject is affected by the trait, but the trait
status of her parents is not known. The genotypes of this subject 
at the first four loci are {\tt 1/3}, {\tt 1/3}, {\tt 4/1} and {\tt 4/2}.
Note that {\tt snpStats} will only deal with diallelic data and,
although alleles are coded 1 to 4 in this file, only two of these
occur with in any one locus. In fact these data are from the sample
dataset distributed with the HAPLOVIEW program (Barrett et al., 2005)
which uses the numbers 1--4 to denote the four nucleotides:  {\tt 1 =}
A, {\tt 2 =} C, {\tt 3 = } G, {\tt 4 =} . The pedfile contains data for
20 loci on 120 subjects, and is accompanied by
a second file which describes the loci, the first four lines being:
\begin{verbatim}
IGR1118a_1	274044
IGR1119a_1	274541
IGR1143a_1	286593
IGR1144a_1	287261
\end{verbatim}
(this file is rather simple, containing just the locus name and its
position on a chromosome). 

The (gzipped) pedfile and the locus information file are stored in the 
{\tt extdata} sub-directory of the {\tt snpStats} package as, respectively,
{\tt sample.ped.gz} and {\tt sample.info}. Since the precise location
of these files may vary between installations, we first obtain full paths 
to these files  using the {\tt system.file} function
<<sysfile>>=
pedfile <- system.file("extdata/sample.ped.gz", package="snpStats")
pedfile
infofile <- system.file("extdata/sample.info", package="snpStats")
@ 
The data can then be read in using the {\tt read.pedfile} function
<<redpedfile>>=
sample <- read.pedfile(pedfile, snps=infofile)
@ 
The result, {\tt sample}, is a list with three elements. The first is an
object of class {\tt SnpMatrix} containing the genotype data. We shall
show summaries for the first few loci
<<sample1>>=
sample$genotypes
col.summary(sample$genotypes)$MAF
head(col.summary(sample$genotypes))
@ 
The second list element is a dataframe containing the first six fields of the
pedfile. We'll just display the start of this:
<<sample2>>=
head(sample$fam)
@ 
Note that the zero values in the pedfile have been read as
{\tt NA}; this is optional, but default, behaviour of the
function. Here the pedigree-member identifiers have been used as
subject identifiers, since these are not duplicated while pedigree
identifiers (the first choice) were duplicated (if both sets of
identifiers are duplicated, they are combined).
Finally, the third list element is a dataframe containing the
information read from the {\tt sample.info} file, to which have been
added the two alleles found at each locus:
<<sample3>>=
head(sample$map)
@ 
Here we have used the default settings of {\tt read.pedfile}. In
particular, it is not mandatory to supply a locus description file and
there
are further arguments which allow additional flexibility. These
options are described in the on-line help page.
\section*{PLINK files}
Binary PED (BED) files written by the PLINK toolset (Purcell et al., 2007) 
may also be
read as {\tt SnpMatrix} objects. Files of type {\tt .bed} are written
by the {\tt plink --make-bed} command and are accompanied by two text
files: a {\tt .fam} file containing the first six fields of a standard
pedfile as described above, and a {\tt .bim} file which describes the
loci. The package data directory also 
contains {\tt .bed}, {\tt .fam}
and {\tt .bim} files for the sample dataset of the last section; the
following commands recover the full file paths for these files and 
read the files:
<<plink>>=
fam <- system.file("extdata/sample.fam", package="snpStats")
bim <- system.file("extdata/sample.bim", package="snpStats")
bed <- system.file("extdata/sample.bed", package="snpStats")
sample <- read.plink(bed, bim, fam)
@ 
The output object is similar to that produced by {\tt read.pedfile}, a
list with three elements:
<<plinkout>>=
sample$genotypes
col.summary(sample$genotypes)$MAF
head(sample$fam)
head(sample$map)
@ 
Usually the three input files have the same filename stub with
{\tt .bed}, {\tt .fam} and {\tt .bim} extensions added. In this case
it is sufficient to just supply the filename stub to {\tt read.plink}.

A useful feature of {\tt read.plink} is the ability to select a
subset of data from a large PLINK dataset. This is demonstrated
in our small example below
<<plinkselect>>=
subset <- read.plink(bed, bim, fam, select.snps=6:10)
subset$genotypes
col.summary(subset$genotypes)$MAF
subset$map
@ 
Note that, in order to select certain SNPs, the input PLINK
file must be in SNP-major order {\it i.e.\,} all individuals for the
first SNP, all individuals for the second SNP, and so on. This is the
default mode in PLINK. However, to select certain individuals, the
input PLINK file must be in individual-major order.
\section*{Long format data}
The least compact, but perhaps most flexible, input format is the
``long'' format in which each genotype call takes up a single
line. Such data can be read using the function {\tt read.snps.long}.
A simple example is provided by the small gzipped data file {\tt
  sample-long.gz} provided with the package:
<<longfile>>=
longfile <- system.file("extdata/sample-long.gz", package="snpStats")
longfile
@ 
The first 5 lines of the file are listed as follows:
<<longlist>>=
cat(readLines(longfile, 5), sep="\n")
@ 
The first field gives the SNP identifier ({\tt snp1} to {\tt snp18}),
the second gives the sample, or subject, identifier  ({\tt subject1} to 
{\tt subject100}), the third field gives the genotype call ({\tt
  1=A/A}, {\tt 2=A/B}, {\tt 3=B/B}), and the last field gives a
confidence measure for the call (here always {\tt 1.000}).
To read in this file and inspect the data:
<<readlong>>=
gdata <- read.long(longfile, 
   fields=c(snp=1, sample=2, genotype=3, confidence=4),
   gcodes=c("1", "2", "3"), 
   threshold=0.95)
gdata
summary(gdata)
@ 
A few remarks:
\begin{enumerate}
\item In our example, the entire file has been read. However, subsets
  of data may be extracted by specifying the required SNP or sample
  identifiers.
\item Any calls for which the call confidence is less than {\tt
    threshold} is set to {\tt NA} (this did not affect any calls in
  this simple example).
\item Here, calls were represented by a single genotype code. It is
  also possible to read calls as pairs of alleles. The function then
  returns a list whose first argument is the {\tt SnpMatrix} object,
  and whose second object is a dataframe containing the allele
  codes. This option is demonstrated below, using an alternative
  coding of the same data (all SNPs are {\tt CT} SNPs):
\end{enumerate}
<<readlongallele>>=
allelesfile <- system.file("extdata/sample-long-alleles.gz", package="snpStats")
cat(readLines(allelesfile, 5), sep="\n")
gdata <- read.long(allelesfile, 
   fields=c(snp=1, sample=2, allele.A=3, allele.B=4, confidence=5),
   threshold=0.95)
gdata
gdata$genotypes
gdata$alleles
@
Note that the assignment of alleles depends on the order in which they
were encountered.

This function has many options and the online help page needs to be
read carefully.
\section*{Other formats}
\subsection*{Imputation}
A further source of input data is programs which can {\em impute}
genotype data for a set of study individuals, using genome-wide
SNP-chip data for the study subjects plus HapMap or 1,000 genomes
project datasets. {\tt snpStats} provides the functions {\tt
  read.beagle}, {\tt read.impute}, and {\tt read.mach} to read in
files produced by the leading imputation programs. For more details of
such data, see the imputation and meta-analysis vignette.
\subsection*{VCF format}
The 1,000 genomes data are released in the VCF format. {\tt snpStats}
does not yet include a function to read data files in this format, 
but the {\tt GGtools}
package does contain such a function ({\tt vcf2sm}).
\subsection*{X, Y and mitocondrial SNPs}
The {\tt SnpMatrix} class is designed for diploid SNP
genotypes. SNPs which can be haploid are stored in objects of the {\tt
  XSnpMatrix} class, which has an addition slot, named {\tt diploid}. Since,
for the X chromosome, ploidy depends on sex and may vary from row to
row, this (logical) vector has the same number of elements as the
number of rows in the SNP data matrix. Most input routines do not
allow for reading an {\tt XSnpMatrix} and simply read into a {\tt
  SnpMatrix}, coding haploid calls as (homozygous) diploid. Such
objects may then be coerced into the {\tt XSnpMatrix} class using {\tt
  as(\ldots, "XSnpMatrix")} or {\tt new("XSnpMatrix, \ldots,
  diploid=\ldots)}. If {\tt as} is used, ploidy is inferred from
homozygosity while, if {\tt new} is used, it must be supplied (if all
rows have the same ploidy, this argument can be a scalar). In either
case, calls presumed to be haploid but coded as heterozygous will be set
to {\tt NA}.
\section*{Reference}
Barrett JC, Fry B, Maller J, Daly MJ.(2005)  Haploview: analysis
and visualization of LD and haplotype maps. {\it Bioinformatics}, 2005 Jan 15,
[PubMed ID: 15297300]\\[2mm]
Purcell S, Neale B, Todd-Brown K, Thomas L, Ferreira MAR, 
Bender D, Maller J, Sklar P, de Bakker PIW, Daly MJ and Sham PC (2007) 
PLINK: a toolset for whole-genome association and population-based 
linkage analysis. {\it American Journal of Human Genetics}, {\bf 81}
\end{document}

%\documentclass[a4paper,12pt]{article}
\documentclass[12pt]{article}
\usepackage{fullpage}
% \usepackage{times}
%\usepackage{mathptmx}
%\renewcommand{\ttdefault}{cmtt}
\usepackage{graphicx}

\usepackage[pdftex,
bookmarks,
bookmarksopen,
pdfauthor={David Clayton and Chris Wallace},
pdftitle={snpStats Vignette}]
{hyperref}

\title{snpStats vignette\\Example of genome-wide association testing}
\author{David Clayton and Chris Wallace}
\date{\today}

\usepackage{Sweave}
\SweaveOpts{echo=TRUE, pdf=TRUE, eps=FALSE}

\begin{document}
\setkeys{Gin}{width=1.0\textwidth}

%\VignetteIndexEntry{snpStats introduction}
%\VignettePackage{snpStats}

\maketitle

\section*{The {\tt snpMatrix} and {\tt snpStats} packages}
The package ``{\tt snpMatrix}'' was written to provide data classes
and methods to
facilitate the analysis of whole genome association studies in R.
In the data classes it implements,
each genotype call is stored as a single byte and, at this density,
data for single chromosomes derived from large studies and new
high-throughput gene chip platforms can be handled in memory by modern
PCs and workstations.
The object--oriented programming model introduced with version 4 of
the S-plus package, usually termed ``S4 methods'' was used to
implement these classes.

{\tt snpStats} arose out of the need to store, and analyse, SNP 
genotype data in which subjects cannot be assigned to the three
possible genotypes with certainty. This necessitated a change in the
way in which data are stored internally, although {\tt snpStats} can
still handle conventionally called 
genotype data stored in the original {\tt snpMatrix} storage
mode. {\tt snpStats} currently lacks some facilities which were
present in {\tt snpMatrix} (although, hopefully, the important  gaps will 
soon be filled) but it also includes several important new
facilities. This vignette currently exploits none of the new
facilities; these are mainly used in the vignette which deals with
imputation and meta-analysis. 

For population-based studies, both quantitative
and qualitative phenotypes may be analysed but, at present, rather
more limited facilities are available for family--based studies. 
Flexible functions are provided which can carry out single SNP tests
which control for potential confounding by quantitative and
qualitative covariates. Tests involving several SNPs taken together
as ``tags'' are also supported. 
The original {\tt snpMatrix} package was described by Clayton and Leung 
(2007) {\it Human Heredity}, {\bf 64}: 45--51. Since this publication many new
facilities have been introduced; some of 
these are explored in further vignettes.


\section*{Getting started}
We shall start by loading  the 
the packages and the data to be used in the first part of this
exercise, which concerns a population--based case--control study:

<<init>>=
require(snpStats)
require(hexbin)
data(for.exercise)
@ 

In addition to the {\tt snpStats} package, we have also loaded the
{\tt hexbin} package which reduces file sizes and legibility of plots
with very many data points.

The data have been created artificially from publicly available
datasets. The SNPs have been selected from those genotyped by the
International HapMap Project\footnote{\tt http://www.hapmap.org} to represent
the typical density found on a whole genome association chip, (the Affymetrix
500K platform\footnote{\tt
  http://www.affymetrix.com/support/technical/sample\_data/500k\_hapmap\_genotype\_data.affx})
for a moderately sized chromosome (chromosome 10). A (rather too)
small study of 500 cases and
500 controls has been simulated allowing for recombination using beta
software from Su and Marchini.  Re-sampling of cases was weighted in such a
way as to simulate three ``causal'' locus on this chromosome, with
multiplicative effects of 1.3, 1.4 and 1.5 for each copy of the risk allele at
each locus. It should be noted that this is a somewhat optimistic
scenario!

You have loaded three objects:
\begin{enumerate}
\item {\tt snps.10}, an object of class ``{\tt SnpMatrix}''
  containing a matrix of SNP genotype calls. Rows of the matrix
  correspond to subjects and columns correspond to SNPs:
<<>>=
show(snps.10)
@ 

\item {\tt snp.support}, a conventional R
data frame containing information about the
SNPs typed. To see its contents:
<<>>=
summary(snp.support)
@

Row names of this data frame correspond with column names of {\tt
  snps.10} and comprise the (unique) SNP identifiers.
\item {\tt subject.support}, another conventional R data frame
  containing further
  information about the subjects. The row names coincide with the row
  names of {\tt snps.10} and
  comprise the (unique) subject identifiers. In this simulated study
  there are only two variables:
<<>>=
summary(subject.support)
@

The variable {\tt cc} identifies cases ({\tt cc=1}) and controls
({\tt cc=0}) while {\tt stratum}, coded 1 or 2, identifies a
stratification of the study population --- more on this later.
\end{enumerate}
In general, analysis of a whole--genome association study will require
a subject support data frame, a SNP support data frame for each
chromosome, and a SNP data file for each chromosome\footnote{
Support files are usually read in with general tools such as {\tt
  read.table}. The {\tt snpStats} package contains a number of tools
for reading SNP genotype data into an object of class ``{\tt
  SnpMatrix}''.}.

A short summary of the contents of {\tt snps.10} is provided by the
{\tt summary} function. This operation actually produces two ``summaries of
summaries''. First, summary 
statistics are calculated for each row (sample), and their results
summarised. Then summary statistics are calculated for each column
(SNP) and their results summarised.
<<>>=
summary(snps.10)
@ 
The row-wise and column-wise summaries are
calculated with the functions {\tt row.summary} and {\tt col.summary}.
For example, to calculate summary statistics for each SNP (column):
<<>>=
snpsum <- col.summary(snps.10)
summary(snpsum)
@
The second command duplicates the latter part of the result of {\tt
  summary(snps.10)}, and the contents of {\tt snpsum} are fairly
self-explanatory. 
We could look at a couple of summary statistics in more detail:
<<plot-snpsum,fig=TRUE>>=
par(mfrow = c(1, 2))
hist(snpsum$MAF)
hist(snpsum$z.HWE)
@

The latter should represent a $z$-statistic. {\it i.e.} a statistic
normally distributed with mean zero and unit standard deviation under
the hypothesis of Hardy--Weinberg equilibrium (HWE). Quite clearly there is
extreme deviation from HWE, but this can be accounted for by the
manner in which this synthetic dataset was created.

The function {\tt row.summary} is useful for detecting samples that
have genotyped poorly. This calculates call rate and mean heterozygosity
across all SNPs for each subject in turn:
<<sample-qc>>=
sample.qc <- row.summary(snps.10)
summary(sample.qc)
@
(note that the last command yields the same as the first part of 
{\tt summary(snps.10)}).
The plot of heterozygosity against call rate is useful in detecting
poor quality samples:
<<plot-outliners-qc,fig=TRUE>>=
par(mfrow = c(1, 1))
plot(sample.qc)
@ 
There is one clear outlier.

\section*{The analysis}
We'll start by removing the `outlying' sample above (the sample with 
Heterozygosity near zero):
<<outliers>>=
use <- sample.qc$Heterozygosity>0
snps.10 <- snps.10[use, ]
subject.support <- subject.support[use, ]
@

Then we'll see if there is any difference between call rates
for cases and controls. First generate logical arrays for selecting
out cases or controls:\footnote{
These commands assume that the subject support frame has the same
number of rows as the SNP matrix and that they are in the same
order. Otherwise a slightly more complicated derivation is necessary.}
<<if-case-control>>=
if.case <- subject.support$cc == 1
if.control <- subject.support$cc == 0
@
Now we recompute the genotype column summaries separately for cases and
controls:
<<sum-case-control>>=
sum.cases <- col.summary(snps.10[if.case, ])
sum.controls <- col.summary(snps.10[if.control, ])
@
and plot the call rates, using hexagonal binning and 
superimposing a line of slope 1 through the origin:
<<plot-summaries,fig=TRUE>>=
hb <- hexbin(sum.controls$Call.rate, sum.cases$Call.rate, xbin=50)
sp <- plot(hb)
hexVP.abline(sp$plot.vp, 0, 1, col="black")
@
There is no obvious difference in call rates. This is not a surprise,
since  no such difference was built into the simulation. In the same
way we could look for differences between
allele frequencies, superimposing a line of slope 1 through the origin:
<<plot-freqs,fig=TRUE>>=
sp <- plot(hexbin(sum.controls$MAF, sum.cases$MAF, xbin=50))
hexVP.abline(sp$plot.vp, 0, 1, col="white")
@

This is not a very effective way to look for associations, but if
the SNP calling algorithm has been run separately for cases and
controls this plot can be a useful diagnostic for things going
wrong ({\it e.g.} different labelling of clusters).

It should be stressed that, for real data, the plots described above
would usually have many more outliers. Our simulation did not model
the various
biases and genotype failures that affect real studies.

The fastest tool for carrying out simple tests for association taking
the SNP one at a time is {\tt single.snp.tests}. The output from this
function is a data frame with one line of data for each SNP. Running
this in our data and summarising the results:
<<tests>>=
tests <- single.snp.tests(cc, data = subject.support, snp.data = snps.10)
@
Some words of explanation are required. In the call, the {\tt
  snp.data=} argument is mandatory and provides the name of the matrix
providing the genotype data. The {\tt data=} argument gives the name
of the data frame that contains the remaining arguments --- usually the
subject support data frame\footnote{This is not mandatory --- we could
have made {\tt cc} available in the global environment. However we
would then have to be careful that the values are in the right order;
by specifying the data frame, order is forced to be correct by
checking the order of the row names for the {\tt data} and {\tt
  snp.data} arguments.}.

Let us now see what has been calculated:
<<sum-tests>>=
summary(tests)
@

We have, for each SNP,
 chi-squared tests on 1 and 2 degrees of freedom (df), together
with $N$, the number of subjects for whom data were available. The
 1 df test is the familiar Cochran-Armitage test for codominant effect
 while the 2 df test is the conventional Pearsonian test for the
 $3\times 2$ contingency table. The large number of {\tt NA} values
 for the latter test reflects the fact that, for these SNPs, the minor
 allele frequency was such that one homozygous genotype did not occur
 in the data.

We will probably wish to restrict our attention to SNPs that pass
certain criteria. For example
<<use>>=
use <- snpsum$MAF > 0.01 & snpsum$z.HWE^2 < 200
@
(The Hardy-Weinberg filter is ridiculous and reflects the strange
characteristics of these simulated data. In real life you might want
to use something like 16, equivalent to a 4SE cut-off). To see how
many SNPs pass this filter
<<sum-use>>=
sum(use)
@
We will now throw way the discarded test results and save the positions
of the remaining SNPs
<<subset-tests>>=
tests <- tests[use]
position <- snp.support[use, "position"]
@

We now calculate $p$-values  for the
Cochran-Armitage tests and plot minus logs (base 10) of the $p$-values
against position
<<plot-tests,fig=TRUE>>=
p1 <- p.value(tests, df=1)
plot(hexbin(position, -log10(p1), xbin=50))
@

Clearly there are far too many ``significant'' results, an impression
which is made even clearer by the quantile-quantile (QQ) plot:
<<qqplot,fig=TRUE>>=
chi2 <- chi.squared(tests, df=1)
qq.chisq(chi2,  df = 1)
@


The three numbers returned by this command are the number of tests considered,
the number of outliers falling beyond the plot boundary, and the slope of a
line fitted to the smallest 90\% of values ({\it i.e.} the multiple by
which the chi-squared test statistics are over-dispersed).  
The ``concentration band'' for the
plot is shown in grey. This region is defined by upper and lower probability
bounds for each order statistic.  The default is to use the 2.5\% and 95.7\%
bounds\footnote{Note that this is not a simultaneous confidence region; the
probability that the plot will stray outside the band at some point exceeds
95\%.}.

This over-dispersion of chi-squared values was built into our
simulation. The data were constructed by re-sampling individuals from
{\em two} groups of HapMap subjects, the CEU sample (of European
origin) and the JPT$+$CHB sample (of Asian origin). The 55\% of the
cases were of European ancestry as compared with  only 45\% of the
controls. We can deal with this by stratification of the tests,
achieved by adding the {\tt stratum} argument to the call to {\tt
  single.snp.tests} (the remaining commands are as before)
<<more-tests,fig=TRUE>>=
tests <- single.snp.tests(cc, stratum, data = subject.support,
     snp.data = snps.10)
tests <- tests[use]
p1 <- p.value(tests, df = 1)
plot(hexbin(position, -log10(p1), xbin=50))
@
<<more-tests-qq,fig=TRUE>>=
chi2 <- chi.squared(tests, df=1)
qq.chisq(chi2, df = 1)
@


Most of the over-dispersion of test statistics has been removed (the
residual is probably due to ``cryptic relatedness'' owing to the way
in which the data were simulated).

Now let us find the names and positions of the most significant 10
SNPs. The first step is to compute an array which gives the positions
in which the first, second, third etc. can be found
<<ord>>=
ord <- order(p1)
top10 <- ord[1:10]
top10
@


We now list the 1~df $p$-values, the corresponding SNP names and their
positions on the chromosome:
<<top-10>>=
names <- tests@snp.names
p1[top10]
names[top10]
position[top10]
@


The most associated SNPs lie within two small regions of the genome. To
concentrate on the rightmost region (the most associated region on the left
contains just one SNP), we'll first sort the names of the SNPs into position
order along the chromosome and select those lying in the region approximately
one mega-base either side of the second most associated SNP:
<<top10-local>>=
posord <- order(position)
position <- position[posord]
names <- names[posord]
local <- names[position > 9.6e+07 & position < 9.8e+07]
@
The variable {\tt posord} now contains the permutation necessary to sort
SNPs into position order and {\tt names} and {\tt position} have now
been reordered in this manner. The variable {\tt local} contains the
names of the SNPs in the selected 2 mega-base region.

Next we shall estimate the size of the effect at the most associated SNPs for
each region (rs870041, rs10882596). In the following commands, we
extract each SNP from the matrix as a numerical variable (coded 0, 1, or 2)
and then, using the {\tt glm} function, carry out a logistic regression of
case--control status on this numerical coding of the SNP and upon stratum. The
variable {\tt stratum} must be included in the regression in order to allow
for the different population structure of cases and controls.  We
first make copies of the {\tt cc} and {\tt stratum}
variables in {\tt subject.support} in the current working environment
(where the other variables reside):
<<top1>>=
cc <- subject.support$cc
stratum <- subject.support$stratum
top <- as(snps.10[, "rs870041"], "numeric")
glm(cc ~ top + stratum, family = "binomial")
@
The coefficient of {\tt top} in this regression is estimated as 0.5100,
equivalent to a relative risk of $\exp(.5100) =1.665$.  
For the other top SNP we have:
<<top2>>=
top2 <- as(snps.10[, "rs10882596"], "numeric")
fit <- glm(cc ~ top2 + stratum, family = "binomial")
summary(fit)
@
This relative risk is $\exp(0.4575)=1.580$. Both estimates are
close to the values used to simulate the data.

You might like to repeat the analysis above using the 2 df
tests. The conclusion would have been much the same. A word of caution
however; with real data the 2 df test is less robust against
artifacts due to genotyping error. On the other hand, it is much more
powerful against recessive or near-recessive variants.

The {\tt snpStats} package includes its own functions to fit generalized
linear models. These are much faster than {\tt glm}, although not yet
as flexible. They allow for a each of series of SNPs to be entered
into a GLM, either on the left hand side ({\it i.e.} as the dependent
variable) or on the right-hand side (as a predictor variable). In the
latter case seveal SNPs can be entered in each model fit. 
For example, to fit the same GLM as before, in which each SNP is
entered in turn on the right-hand side of a logistic regression
equation,  for each of the SNPs in the 2 megabase ``local'' region:
<<estimates>>=
localest <- snp.rhs.estimates(cc~stratum, family="binomial", sets=local,
                              data=subject.support, snp.data=snps.10)
@ 
This function call has computed \Sexpr{length(local)} GLM fits! 
The parameter estimates for the
first five, and for the second best SNP analyzed above (rs10882596)
are shown by
<<list-estimates>>=
localest[1:5]
localest["rs10882596"]
@ 
The parameter estimate for rs1088259 and its standard error agree closely 
with the values obtained earlier, using the {\tt glm} function. 

The GLM code within {\tt snpStats} allows a
further speed-up which is not available in the standard {\tt glm}
function. If a variable is to be included in the model as a ``factor''
taking many levels then a more efficient algorithm can be invoked 
by using the
{\tt strata} function in the model formula. For example, the following
command fits the same model for all the \Sexpr{sum(use)} 
SNPs we have decided to use in these analyses:
<<fast-estimates>>=
allest <- snp.rhs.estimates(cc~strata(stratum), family="binomial", sets=use,
                              data=subject.support, snp.data=snps.10)
length(allest)
@ 
As expected, the parameter estimates and standard errors 
are unchanged, for example:
<<check-estimates>>=
allest["rs10882596"]
@ 
Note that {\tt strata()} can only be used once in a model formula.
\section*{Multi-locus tests}
There are two other functions  for carrying out association
tests ({\tt snp.lhs.tests} and {\tt snp.rhs.tests}) in the
package. These are somewhat slower, but much more flexible. For
example, the former function allows one to test for  differences in allele
frequencies between more than two groups. An important use of the
latter function is to carry out tests using
{\em groups} of SNPs rather than single SNPs. We shall explore this use
in the final part of the exercise.

A prerequisite to multi-locus analyses is to
decide on how SNPs should be grouped in order to ``tag'' the genome
rather more completely than by use of single markers. Hopefully, the {\tt
  snpMatrix} package will eventually contain tools to compute such
groups, for example, by using HapMap data. The function {\tt
  ld.snp}, which we encountered earlier, will be an essential tool in
this process. However this work is not complete and, for now, we
demonstrate the testing tool by grouping the  \Sexpr{sum(use)} SNPs we have
decided to use into 20kB blocks. The following commands compute such a
grouping, tabulate the block size, and remove empty blocks:
<<blocks>>=
blocks <- split(posord, cut(position, seq(100000, 135300000, 20000)))
bsize <- sapply(blocks, length)
table(bsize)
blocks <- blocks[bsize>0]  
@
You can check that this has worked by listing the column positions 
of the first 20 SNPs together with the those contained in the
first five blocks
<<twentyfive>>=
posord[1:20]
blocks[1:5]
@


Note that these positions refer to the reduced set of SNPs after
application of the filter on MAF and HWE. Therefore, before proceeding
further we create a new matrix of SNP genotypes containing only these
27,828:
<<blocks-use>>=
snps.use <- snps.10[, use]
remove(snps.10)
@

The command to carry out the tests on these groups, controlling for
the known population structure differences is
<<mtests,keep.source=TRUE>>=
mtests <- snp.rhs.tests(cc ~ stratum, family = "binomial", 
     data = subject.support, snp.data = snps.use, tests = blocks)
summary(mtests)
@


The first argument, together with the second,
specifies  the model which corresponds to the null
hypothesis. In this case we have allowed for the variation in ethnic
origin ({\tt stratum}) between cases and controls.
We complete the analysis by extracting 
the $p$--values and plotting minus their logs (base 10):
<<plot-mtests,fig=TRUE>>=
pm <- p.value(mtests)
plot(hexbin(-log10(pm), xbin=50))
@

The same associated region is picked out, albeit with a rather larger
$p$-value; in this case the multiple df test cannot be powerful as the 1 df
test since the simulation ensured that the ``causal'' locus was
actually one of the SNPs typed on the Affymetrix platform.
QQ plots are somewhat more difficult since the tests are on differing
degrees of freedom. This difficulty is neatly circumvented by noting
that, under the null hypothesis,  $-2\log p$ is distributed as
chi-squared on 2~df:
<<qqplot-mtests,fig=TRUE>>=
qq.chisq(-2 * log(pm), df = 2)
@
\end{document}

%\documentclass[a4paper,12pt]{article}
\documentclass[12pt]{article}
\usepackage{fullpage}
% \usepackage{times}
%\usepackage{mathptmx}
%\renewcommand{\ttdefault}{cmtt}
\usepackage{graphicx}

\usepackage[pdftex,
bookmarks,
bookmarksopen,
pdfauthor={David Clayton},
pdftitle={TDT and snpStats Vignette}]
{hyperref}

\title{LD vignette\\Measures of linkage disequilibrium}
\author{David Clayton}
\date{\today}

\usepackage{Sweave}

\SweaveOpts{echo=TRUE, pdf=TRUE, eps=FALSE}

\begin{document}
\setkeys{Gin}{width=1.0\textwidth}

%\VignetteIndexEntry{LD statistics}
%\VignettePackage{snpStats}

<<lib,echo=FALSE>>=
require(snpStats)
require(hexbin)
@ 

\maketitle

\section*{Calculating linkage disequilibrium statistics}
We shall first load some illustrative data. 
<<data>>=
data(ld.example)
@ 
The data are drawn from the International HapMap Project and concern 603 SNPs 
over a 1mb region of
chromosome 22 in sample of Europeans ({\tt ceph.1mb}) and a sample of
Africans ({\tt yri.1mb}):
<<showgt>>=
ceph.1mb
yri.1mb
@ 
The details of these SNP are stored  in the dataframe {\tt support.ld}:
<<showsp>>=
head(support.ld)
@ 
The function for calculating measures of linkage disequilibrium (LD) in
{\tt snpStats} is {\tt ld}. The following two commands call this
function to calculate the D-prime and
R-squared measures of LD between pairs of SNPs for the European and
African samples:
<<ldstats>>=
ld.ceph <- ld(ceph.1mb, stats=c("D.prime", "R.squared"), depth=100)
ld.yri <- ld(yri.1mb, stats=c("D.prime", "R.squared"), depth=100)
@ 
The argument {\tt depth} specifies the maximum separation between
pairs of SNPs to be considered, so that {\tt depth=1} would have
specified calculation of LD only between immediately adjacent SNPs.

Both {\tt ld.ceph} and {\tt ld.yri} are lists with two elements each,
named {\tt D.prime} and {\tt R.squared}. These elements are (upper
triangular) band
matrices, stored in a packed form defined in the {\tt Matrix}
package. They are too large to be listed, but the {\tt Matrix} package
provides an {\tt image} method, a convenient way to examine patterns
in the matrices. You should look at these carefully and
note any differences. 
<<image1,eval=FALSE>>=
image(ld.ceph$D.prime, lwd=0)
@ 
<<image1a,echo=FALSE,fig=TRUE>>=
print(image(ld.ceph$D.prime, lwd=0))
@ 
<<image2,eval=FALSE>>=
image(ld.yri$D.prime, lwd=0)
@ 
<<image2a,echo=FALSE,fig=TRUE>>=
print(image(ld.yri$D.prime, lwd=0))
@ 
The important things to note are 
\begin{enumerate}
\item there are fairly well-defined ``blocks'' of LD, and
\item LD is more pronounced in the Europeans than in the Africans.
\end{enumerate}
The second point is demonstrated by extracting the D-prime values from
the matrices (they are to be found in a slot named {\tt x}) and
calculating quartiles of their distribution:
<<quartiles>>=
quantile(ld.ceph$D.prime@x, na.rm=TRUE)
quantile(ld.yri$D.prime@x, na.rm=TRUE)
@

If preferred, {\tt image} can produce colour plots. We first create a
set of 10 colours ranging from yellow (for low values) to red (for high values) 
<<colors>>=
spectrum <- rainbow(10, start=0, end=1/6)[10:1]
@ 
and plot the image, with a colour key down its right hand side
<<imagecol,eval=FALSE>>=
image(ld.ceph$D.prime, lwd=0, cuts=9, col.regions=spectrum, colorkey=TRUE)
@ 
<<imagecola,echo=FALSE,fig=TRUE>>=
print(image(ld.ceph$D.prime, lwd=0, cuts=9, col.regions=spectrum, colorkey=TRUE))
@ 

The R-squared matrices provide similar pictures, although they are rather less
regular. To show this clearly, we focus on the 200 SNPs starting from
the 75-th, using the European data
<<use>>=
use <- 75:274
@ 
<<image3,eval=FALSE>>=
image(ld.ceph$D.prime[use,use], lwd=0)
@ 
<<image3a,echo=FALSE,fig=TRUE>>=
print(image(ld.ceph$D.prime[use,use], lwd=0))
@ 
<<image4,eval=FALSE>>=
image(ld.ceph$R.squared[use,use], lwd=0)
@ 
<<image4a,echo=FALSE,fig=TRUE>>=
print(image(ld.ceph$R.squared[use,use], lwd=0))
@ 
The R-squared values are smaller and there are ``holes'' in the LD blocks; 
SNPs within an LD block do not necessarily have large R-squared
between them. This is further demonstrated in the next section.
\section*{D-prime, R-squared, and distance}
To examine the relationship between LD and physical distance, we first
need to construct a similar matrix holding the physical
distances. This is carried out, by first calculating each
off-diagonal, and then combining them into a band matrix
<<distance>>=
pos <- support.ld$Position
diags <- vector("list", 100)
for (i in 1:100) diags[[i]] <- pos[(i+1):603] - pos[1:(603-i)]
dist <- bandSparse(603, k=1:100, diagonals=diags)
@ 
The values in the body of the band matrix are contained in a slot
named {\tt x}, so the following commands extract the physical
distances and the corresponding LD statistics for the Europeans:
<<values>>=
distance <- dist@x
D.prime <- ld.ceph$D.prime@x
R.squared <- ld.ceph$R.squared@x
@ 
These are very long vectors so we use the {\tt hexbin} package to
produce abreviated plots. We first demonstrate the relationship
between D-prime and R-squared
<<drplot,fig=TRUE>>=
plot(hexbin(D.prime^2, R.squared))
@ 
We see that the square of D-prime provides an upper bound for
R-squared; a high D-prime indicates the {\em potential} for two SNPs
to be highly correlated, but they need not be. The following commands
examine the relationship between the two LD measures and physical distance
<<dpplot1,fig=TRUE>>=
plot(hexbin(distance, D.prime, xbin=10))
@ 
<<dpplot2,fig=TRUE>>=
plot(hexbin(distance, R.squared, xbin=10))
@ 
Although the data are very noisy, the first plot is consistent with an
approximately exponential decline in mean D-prime with distance, as
predicted by the Malecot model. 
\section*{A view of the calculations}
To understand the calculations let us consider the first and fifth
SNPs in the Europeans. We shall first converting these to character
data for legibility, and then tabulate the two-SNP genotypes, saving
the $3\times 3$ table of genotype frequencies as {\tt tab33}:
<<two>>=
snp1 <- as(ceph.1mb[,1], "character")
snp5 <- as(ceph.1mb[,5], "character")
tab33 <- table(snp1, snp5)
tab33
@ 
These two SNPs have a moderately high D-prime, but a very low R-squared:
<<twoDR>>=
ld.ceph$D.prime[1,5]
ld.ceph$R.squared[1,5]
@ 
The LD measures cannot be directly calculated from the $3\times 3$
table above, but from a $2\times 2$ table of {\em haplotype
frequencies}. In only eight cells around the periphery of the table we can 
unambiguously count haplotypes and these give us the following table
of haplotype frequencies:
\begin{center}
  \begin{tabular}{crr}
    & \multicolumn{2}{c}{\Sexpr{rownames(support.ld)[5]}}\\
      \cline{2-3}
      \Sexpr{rownames(support.ld)[1]} & A & B\\
      \hline
      A& 
      \Sexpr{2*tab33[1,1] + tab33[1,2] + tab33[2,1]}&
      \Sexpr{2*tab33[1,3] + tab33[1,2] + tab33[2,3]}\\
      B&
      \Sexpr{2*tab33[3,1] + tab33[2,1] + tab33[3,2]}&
      \Sexpr{2*tab33[3,3] + tab33[3,2] + tab33[2,3]}
      \\
      \hline
      
  \end{tabular}
\end{center}
However, in the central cell of the $3\times 3$ table ({\it i.e.\,} 
  {\tt tab33[2,2]}) we have
\Sexpr{tab33[2,2]} doubly heterozygous subjects, whose genotype could
correspond either to the pair of haplotypes {\tt A-A/B-B} or to the
pair of haplotypes {\tt A-B/B-A}. These are said to have {\em unknown
  phase}. The expected split between these
possible phases is determined by a further measure of LD --- the
odds ratio. If the odds ratio is $\theta$, we expect a proportion
$\theta/(1+\theta)$ of the doubly heterozygous subjects to be {\tt
  A-A/B-B}, and a proportion  $1/(1+\theta)$ to be  {\tt A-B/B-A}. 

We next use {\tt ld} to obtain an estimate of this odds
ratio\footnote{Here {\tt ld} is called with two arguments of class
  {\tt SnpStats} and, since only the odds ratio is to be calculated, it
  returns the odds ratio rather than a list.}
and, using this, we partition the doubly
heterozygous individuals between the two possible phases: 
<<digits,echo=FALSE>>=
options(digits=4)
@ 
<<OR>>=
OR <- ld(ceph.1mb[,1], ceph.1mb[,5], stats="OR")
OR
AABB <- tab33[2,2]*OR/(1+OR)
ABBA <- tab33[2,2]*1/(1+OR)
AABB
ABBA
@ 
<<twoxtwo,echo=FALSE>>=
a <- format(2*tab33[1,1] + tab33[1,2] + tab33[2,1] + AABB, digits=4)
b <- format(2*tab33[1,3] + tab33[1,2] + tab33[2,3] + ABBA, digits=4)
c <- format(2*tab33[3,1] + tab33[2,1] + tab33[3,2] + ABBA, digits=4)
d <- format(2*tab33[3,3] + tab33[3,2] + tab33[2,3] + AABB, digits=4)
@ 
We are now able to construct the table of haplotype frequencies:
\begin{center}
  \begin{tabular}{crr}
    & \multicolumn{2}{c}{\Sexpr{rownames(support.ld)[5]}}\\
      \cline{2-3}
      \Sexpr{rownames(support.ld)[1]} & A & B\\
      \hline
      A& \Sexpr{a}& \Sexpr{b}\\
      B& \Sexpr{c}& \Sexpr{d}
      \\
      \hline
  \end{tabular}
\end{center}
It is easy to confirm that the odds ratio in this table, 
$(\Sexpr{a}\times\Sexpr{d})/(\Sexpr{b}\times\Sexpr{c})$, 
corresponds closely with that given by the {\tt ld} function. 
Having obtained the $2\times 2$ table of haplotype frequencies, 
any LD statistic may be calculated. 

Of course, there is a circularity here; we needed to know the odds
ratio in order to be able to construct the $2\times 2$ table from
which it is calculated! That is why these calculations are not
simple. The usual method involves iterative solution using an EM
algorithm:  an initial guess at the odds ratio is used, as in the
calculations above, to compute a new estimate, and these calculations 
are repeated
until the estimate stabilizes. However, in {\tt snpStats} the estimate
is calculated in one step, by solving a cubic equation.
\section*{The extent of LD around a point}
Often we wish to guage how far LD extends from a given point (for
example, from a SNP which is associated with disease incidence). For
illustrative purposes we shall consider the region surroung the 168-th
SNP, rs2385786. We first calculate D-prime values for the 100 SNPs on
either side of rs2385786, and their positions:
<<extent>>=
lr100 <- c(68:167, 169:268)
D.prime <- ld(ceph.1mb[,168], ceph.1mb[,lr100], stats="D.prime")
where <- pos[lr100]
@ 
We now plot D.prime against position, adding a simple smoother:
<<eplot,fig=TRUE>>=
plot(where, D.prime)
lines(where, smooth(D.prime))
@ 
Although the data are somewhat noisy (the sample size is small), the
region of LD is fairly clearly delineated.
\section*{Selecting tag SNPs}
Several ways have been suggested to select a set of ``tag'' SNPs which
can be used to test for associations in a given region. That described
below is based upon a heirarchical cluster analysis. We shall apply it
to the region of high LD identified in the previous section, which
lies between positions  $1.579\times 10^7$ and $1.587\times 10^7$. 

The following commands identify which SNPs lie in this region, and
extracts the relevant part of the $R^2$ matrix, as a symmetric matrix
rather than as an upper triangular matrix.
<<ldregion>>=
use <- pos>=1.579e7 & pos<=1.587e7
r2 <- forceSymmetric(ld.ceph$R.squared[use, use])
@ 
The next step is to convert $(1-R^2)$ into a distance matrix, stored as
required for the hierachical clustering function {\tt hclust}, and to
carry out a complete linkage cluster analysis 
<<dist>>=
D <- as.dist(1-r2)
hc <- hclust(D, method="complete")
@ 
To plot the dendrogram, we must first adjust the character size for legibility:
<<clplot,fig=TRUE>>=
par(cex=0.5)
plot(hc)
@ 
The interpretation of this dendrogram is that, if we were to draw a
horizontal line at a ``height'' of 0.5, then this would divide the
SNPs into clusters in such a way that the value of $(1-R^2)$ between
any pair of SNPs in a cluster would be no more than 0.5 (so that $R^2$
would be at least 0.5). This can be carried out using the {\tt cutree}
function, which returns the cluster membership of each SNP:
<<clusters>>=
clusters <- cutree(hc, h=0.5)
head(clusters)
table(clusters)
@ 
It can be seen that there are \Sexpr{max(clusters)} clusters. 
To have a reasonable chance of picking up an association with the SNPs
in this 80kb region, we would need to type a SNP from each one of these
clusters. Of these, \Sexpr{sum(table(clusters)==1)} SNPs
would only tag themselves!

A threshold $R^2$ of 0.5 might seem rather low. However, this is a
``worst case'' figure and most values of $R^2$ would be substantially
better than this, particularly if an effort is made to choose tag SNPs
which are in the center of clusters rather than on their edges. Also,
this process has only considered tagging by single SNPs; it can be
that two or more tag SNPs, taken together, can provide substantially
better prediction than any one of them alone.
\end{document}

\documentclass[12pt]{article}
\usepackage{fullpage}
\usepackage{graphicx}

\usepackage[pdftex,
bookmarks,
bookmarksopen,
pdfauthor={David Clayton},
pdftitle={Fst calculations}]
{hyperref}


\title{Fst\\The algorithm used in {\tt snpStats}}
\author{David Clayton}
\date{\today}

\usepackage{Sweave}
\SweaveOpts{echo=TRUE, pdf=TRUE, eps=FALSE}

\begin{document}
\setkeys{Gin}{width=1.0\textwidth}

%\VignetteIndexEntry{Fst}
%\VignettePackage{snpStats}

\maketitle

<<lib,echo=FALSE>>=
require(snpStats)
@ 
\section*{$F$ statistics for diversity of groups}
There is a very large literature on this topic and the author does not claim
any great expertise. The purpose of this vignette is simply to document the 
method of calculation implemented in {\tt snpStats}.

We shall start by introducing some notation. Let:\\[5mm]
\begin{tabular}[h]{ll}
  $N_g$ &  Number of chromosomes in group $g$\\
  $N_{sg}$ & Number of chromsomes in group $g$ observed for SNP $s$\\
  $N_s$ & Number of chromosomes observed for SNP $s$, all groups\\
  $p_{sg}$ & Allele (relative) frequency for SNP $s$ in group $g$\\
  $p_s$ & Overall allele frequency for SNP $s$\\
\end{tabular}\\[5mm]
and let:
\begin{eqnarray*}
  Y_s &=& \frac{N_s}{N_s-1}p_s(1-p_s)\\
  X_{sg} &=& \frac{N_{sg}}{N_{sg}-1} p_{sg}(1-p_{sg})\\
  X_s &=& \sum_g W_g X_{sg}
\end{eqnarray*}
where $W_g$ are group-specific weights (see below).

The value returned for the F statistic for SNP $s$ is
\[
F_s = 1 - \frac{X_s}{Y_s} = \frac{Y_s - X_s}{Y_s}.
\]

A natural combined value over all SNPs is obtained by summing numerators and 
denominators of the SNP-specific values Denoting summation by a $+$ subsscript,
\[
F = \frac{Y_+ - X_+}{Y_+},
\]
which can also be written as a weighted mean of the SNP-specific values, 
with $Y_s$ as weights:
\[
F = \frac{1}{\sum_s Y_s} \sum_s Y_s F_s.
\]

There appear to be two ways of looking at this index, leading to different 
weights when group sizes are unequal. 

\subsection*{Pairwise differences}
One rationale suggests that the index can be written 
\[
\frac{D_T - D_W }{D_T }
\]
where $D_T$ is the probability that two chromosomes, sampled at random
from the total population, differ and $D_W$ is the probability that two
chromosomes, sampled at random from the same subpopulation,
differ. For a single SNP, $s$, $Y_s$ is an unbiased estimator for $D_T$
and $X_{sg}$ is an unbiased estimator for $D_W$ {\em within subgroup $g$}. 
With this rationale, the weights,
$\{W_g\}$ should reflect the numbers of distinct pairwise comparisons within
each group:
\[
W_g = \frac{N_g (N_g-1)}{\sum_g N_g(N_g-1)}
\]
\subsection*{The analysis of variance}
Another rationale would seem to be in terms of partition of the total
variance, specifically the ratio of the between-group variance to the
total variance (this seems to be the thrust of a series of papers by
Cockerham). $Y_s$ is then an unbiased estimate of the total variance of SNP 
$s$ and $X_{sg}$ is an unbiased estimator of its variance in group $g$. But 
the natural weights are then
\[
W_g =  \frac{N_g }{\sum_g N_g}.
\]
(Cockerham also seems to have considered
an unweighted analysis but that could be inefficient if group sizes differ 
substantially).
\section*{An example}
Here we show the results of these calculations using the HapMap data discussed 
in other vignettes. These data were constructed by re-sampling individuals from
two groups of HapMap subjects, the CEU sample (of European
origin) and the JPT$+$CHB sample (of Asian origin), these groups being 
identified by the variable {\tt stratum} in the subject support data frame.

We first use the pair-wise difference weights, first calculating the 
SNP-specific values, followed by the weighted average across all SNPs:
<<pairwise>>=
data(for.exercise)
f1 <- Fst(snps.10, subject.support$stratum, pairwise=TRUE)
weighted.mean(f1$Fst, f1$weight)
@
We now compare this result with that obtained with the alternative (AOV) 
weights: 
<<aov>>=
f2 <- Fst(snps.10, subject.support$stratum, pairwise=FALSE)
weighted.mean(f2$Fst, f2$weight)
@
Here there is little difference between the two values, since the group sizes 
are nearly the same:
<<groupsize>>=
table(subject.support$stratum)
@
In other cases the two weighting schemes could lead to different answers. In 
such situations, the preference of this author is for the analysis of variance 
weights and, accordingly, this has been set as the default action.

\end{document}

\name{ld.example}
\docType{data}
\alias{ld.example}
\alias{ceph.1mb}
\alias{yri.1mb}
\alias{support.ld}
\title{Datasets to illustrate calculation of linkage disequilibrium statistics}
\description{
  This R data file contains data from the International HapMap project,
  concerning 603 SNPs spanning a one megabase region on chromosome 22,
  in a sample of Europeans and a sample of Africans
}
\format{
  There are three objects in the file:
  \itemize{
    \item \code{ceph.1m}: A \code{snpMatrix} object containing the
    European genotype data
    \item \code{yri.1m}: A \code{snpMatrix} object containing the
    African genotype data
    \item \code{support.ld}: A dataframe containing details (chromosome
    position etc.} of the 603 SNPs
}
\source{
  \url{http://hapmap.ncbi.nlm.nih.gov}
}
\references{
  The International HapMap Consortium. The International HapMap Project.
  \emph{Nature} \bold{426}:789-796 (2003)
}
\keyword{datasets}

\name{for.exercise}
\docType{data}
\alias{for.exercise}
\alias{snps.10}
\alias{snp.support}
\alias{subject.support}
\title{Data for exercise in use of the snpStats package}
\description{
These data have been created artificially from publicly available
datasets. The SNPs have been selected from those genotyped by the
International HapMap Project
(\url{http://www.hapmap.org})
to represent
the typical density found on a whole genome association chip, (the Affymetrix
500K platform,
\url{http://www.affymetrix.com/support/technical/sample\_data/500k\_hapmap\_genotype\_data.affx}
for a moderately sized chromosome (chromosome 10). A study of 500 cases and
500 controls has been simulated allowing for recombination using beta
software from Su and Marchini
(\url{http://www.stats.ox.ac.uk/~marchini/software/gwas/hapgen.html}).
Re-sampling of cases was weighted in such a
way as to simulate three ``causal'' locus on this chromosome, with
multiplicative effects of 1.3, 1.4 and 1.5 for each copy of the risk allele.
}
\usage{data(for.exercise)}
\format{
  There are three data objects in the dataset:
  \itemize{
    \item \code{snps.10}: 
      An object of class "\code{SnpMatrix}"
      containing a matrix of SNP genotype calls. Rows of the matrix
      correspond to subjects and columns correspond to SNPs.
    \item \code{snp.support}: 
      A conventional R
      data frame containing information about the
      SNPs typed (the chromosome position and the nucleotides
      corresponding to the two alleles of the SNP).
    \item \code{subject.support}:
      A conventional R dataframe containing information about the study
      subjects. There are two variables; \code{cc} gives case/control
      status (1=case), and \code{stratum} gives ethnicity.
  }
}
\source{
  The data were obtained from the diabetes and inflammation laboratory
  (see \url{http://www-gene.cimr.cam.ac.uk})
}
\examples{
data(for.exercise)
snps.10
summary(snps.10)
summary(snp.support)
summary(subject.support)
}
\keyword{datasets}

\name{snpStats-package}
\alias{snpStats-package}
\alias{snpStats}
\docType{package}
\title{
\packageTitle{snpStats}
}
\description{
\packageDescription{snpStats}
}
\details{

The DESCRIPTION file:
\packageDESCRIPTION{snpStats}
\packageIndices{snpStats}
}
\author{
\packageAuthor{snpStats}

Maintainer: \packageMaintainer{snpStats}
}
\keyword{ package }

\name{read.pedfile}
\alias{read.pedfile}
\title{
Read a pedfile as \code{"SnpMatrix"} object
}
\description{
Reads diallelic data in linkage "pedfile" format, with one line of data
per sample (subject) containing six mandatory fields followed by pairs
of fields, one pair for each locus,  giving the two alleles observed.
}
\usage{
read.pedfile(file, n, snps, which, split = "\t| +", sep = ".", na.strings = "0", lex.order = FALSE)
}
\arguments{
  \item{file}{
    The input pedfile. This may be (but need not be) gzipped
}
  \item{n}{
    (Optional) The number of lines of data to be read. If not supplied
    the pedfile is read once and rewound to determine how many lines it
    contains 
}
  \item{snps}{
    (Optional) Either a character vector giving the names of the loci,
    or a single character variable giving the name of a locus
    information file from which these can be read. This file is assumed
    to be white-space delimited with one line per locus and no header
    line. If this argument is not supplied, locus names are generated as
    a numerical sequence, prefixed by \code{locus} and a separator character
}
  \item{which}{
    (Optional) If locus names are to be read from a file, this argument
    should specify which column contains the names. If not supplied, the
    first column giving unique locus names is used
}
  \item{split}{
    A "regexp" specifying how the input pedfile will be split into
    fields. The default value specifies either a TAB character or one or
    more spaces
}
  \item{sep}{
    The separator character used in constructing row and column names of
    the output \code{SnpMatrix} object
}
  \item{na.strings}{
    One or more strings to be set to \code{NA}. Any field taking one  of these
    values will be set to \code{NA}
  }
  \item{lex.order}{
    If TRUE, then alleles will be allocated to internal \code{1} and
    \code{2} values in lexographic order. Otherwise they are converted
    in the order in which they are encountered when reading the file
    (the default setting)
}
}
\details{
  Row names for the output \code{SnpMatrix} object and for the
  accompanying subject description dataframe are taken as the pedigree
  identifiers, when these provide the required unique identifiers. When
  these are duplicated, an attempt is made to use the pedigree-member
  identifiers instead but, when these too are duplicated,  
  row names are obtained by concatenating, with a separator character, the
  pedigree and pedigree-member identifiers.
}
\value{
A list, comprising
\item{genotypes}{The output genotype data as an object of class
  \code{"SnpMatrix"}. If either the pedigree or pedigree-member identifiers in
  the \code{ped} file are not duplicated, these are used for the row
  names of the output object. Otherwise these two fields are
  concatenated, separated by \code{sep}}
\item{fam}{A dataframe containing the first six fields in the
  pedfile. The row names will correspond with those of the \code{SnpMatrix}}
\item{map}{A dataframe giving the alleles at each locus. If locus names
  were obtained from a dataframe read from an existing file, then the
  allele information is simply appended to this frame. Otherwise a new
  dataframe is created. The row names will correspond with the column
  names of the \code{SnpMatrix }}
}
\note{
  This function is written entirely in R and may not be particularly
  fast. However, it imposes no restrictions on the allele codes
  recognized.

  Homozygous genotypes may be represented in the input file either (a)
  by coding both alleles to the same value, or (b) setting the second
  allele to "missing" (as specified by the \code{missing.allele}
  argument). No special provision is made to read \code{XSnpMatrix}
  objects; such data should first be read as a \code{SnpMatrix} and then
  coerced to an \code{XSnpMatrix} using \code{new} or \code{as}.
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{SnpMatrix-class}}, \code{\link{XSnpMatrix-class}}}
\examples{
##
## No example supplied yet
##
}
\keyword{manip}
\keyword{IO}
\keyword{file}
\keyword{utilities}

\name{read.impute}
\alias{read.impute}
\title{
Read genotypes imputed by the IMPUTE2 program
}
\description{
The IMPUTE2 program generates, for each SNP and each subject, 
posterior probabilities for the three genotypes. This function reads
such data as a \code{SnpMatrix} object, storing the posterior
probabilities to as much accuracy allowed by a one-byte coding
}
\usage{
read.impute(file, rownames = NULL, nsnp = NULL, snpcol = 2)
}
\arguments{
  \item{file}{
    The input file name. This file my be gzipped.
  }
  \item{rownames}{
    The row names for the output object. Note that these correspond to
    groups of three columns in the input file. If not supplied, names
    are generated as \code{Sample1}, \code{Sample2} etc.
  }
  \item{nsnp}{
    The number of SNPs to be read in. This corresponds with the number
    of lines in the input file. If not supplied, the function does a
    preliminary pass to determine the number of lines
  }
  \item{snpcol}{
    Which column of the input will be used as the SNP name. Default is
    column 2 
  }
}
\details{
No provision is made for data for the X chromosome. Such data must be
first read as a \code{SnpMatrix} and subsequently coerced to an
\code{XSnpMatrix} object
}
\value{
an object of class \code{SnpMatrix}
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{SnpMatrix-class}}}
\examples{
##---- No example available yet
}
\keyword{IO}
\keyword{file}

\name{snp.lhs.tests}
\alias{snp.lhs.tests}
\title{Score tests with SNP genotypes as dependent variable}
\description{
  Under the assumption of Hardy-Weinberg equilibrium, a SNP genotype is
  a binomial variate with two trials for an autosomal SNP or with one or
  two trials (depending on sex) for a SNP on the X chromosome.
  With each SNP in an input
  \code{"SnpMatrix"} as dependent variable, this function first fits a
  "base" logistic regression model and then carries out a score test for
  the addition of further term(s). The Hardy-Weinberg
  assumption can be relaxed by use of a "robust" option.
}
\usage{
snp.lhs.tests(snp.data, base.formula, add.formula, subset, snp.subset,
                data = sys.parent(), robust = FALSE, uncertain = FALSE, 
                control=glm.test.control(), score=FALSE)
}
\arguments{
  \item{snp.data}{The SNP data, as an object of class
    \code{"SnpMatrix"} or \code{"XSnpMatrix"} }
  \item{base.formula}{A \code{formula} object describing the base model,
    with dependent variable omitted }
  \item{add.formula}{A \code{formula} object describing the additional
    terms to be tested, also with dependent variable omitted}
  \item{subset}{An array describing the subset of observations to be
    considered} 
  \item{snp.subset}{An array describing the subset of SNPs to be
    considered. Default action is to test all SNPs.} 
  \item{data}{The data frame in which \code{base.formula},
    \code{add.formula} and \code{subset} are to be evaluated}
  \item{robust}{If \code{TRUE}, a test which does not assume
    Hardy-Weinberg equilibrium will be used }
  \item{uncertain}{If \code{TRUE}, uncertain genotypes are used and
    scored by their posterior expectations. Otherwise they are treated
    as missing. If set, this option forces \code{robust} variance estimates}
  \item{control}{An object giving parameters for the IRLS algorithm
    fitting of the base model and for the acceptable aliasing amongst
    new terms to be tested. See \code{\link{glm.test.control}}}
  \item{score}{Is extended score information to be returned?}
}
\details{
  The tests used are asymptotic chi-squared tests based on the vector of
  first and second derivatives of the log-likelihood with respect to the
  parameters of the additional model. The "robust" form is a generalized
  score test in the sense discussed by Boos(1992). 
  If a \code{data} argument is supplied, the \code{snp.data} and
  \code{data} objects are aligned by rowname. Otherwise all variables in
  the model formulae are assumed to be stored in the same order as the
  columns of the \code{snp.data} object. 
}
\value{
  An object of class \code{\link[=GlmTests-class]{snp.tests.glm}}
  or \code{\link[=GlmTestsScore-class]{GlmTests.score}}
  depending on whether \code{score} is set to \code{FALSE} or \code{TRUE}
  in the call.
}
\references{Boos, Dennis D. (1992) On generalized score tests. \emph{The
  American Statistician}, \strong{46}:327-333.}
\author{David Clayton \email{dc208@cam.ac.uk}}
\note{
  A factor (or
  several factors) may be included as arguments to the function
  \code{strata(...)} in the \code{base.formula}. This fits all
  interactions of the factors so included, but leads to faster
  computation  than fitting these in the normal way. Additionally, a
  \code{cluster(...)} call may be included in the base model
  formula. This identifies clusters of potentially correlated
  observations (e.g. for members of the same family); in this case, an
  appropriate robust estimate of the variance of the score test is used.
  No more than one
  \code{strata()} call may be used, and neither \code{strata(...)}  or
  \code{cluster(...)} calls may appear in the \code{add.formula}.
  A known bug is that the function fails when no \code{data} argument is
  supplied and the base model formula contains no variables
  (\code{~1}). A work-round is to create a data frame to hold the
  variables in the models and pass this as \code{data=}. 
}
\seealso{\code{\link{GlmTests-class}},
  \code{\link{GlmTestsScore-class}},
  \code{\link{glm.test.control}},\code{\link{snp.rhs.tests}}
    \code{\link{single.snp.tests}}, \code{\link{SnpMatrix-class}},
    \code{\link{XSnpMatrix-class}}}
\examples{
data(testdata)
snp.lhs.tests(Autosomes[,1:10], ~cc, ~region, data=subject.data)
snp.lhs.tests(Autosomes[,1:10], ~strata(region), ~cc,
   data=subject.data)
}
\keyword{htest}

\name{ibsDist}
\alias{ibsDist}
\title{Distance matrix based on identity by state (IBS)}
\description{
Expresses a matrix of IBS counts (see \code{\link{ibsCount}}) as a
distance matrix. The distance between two samples is returned as the
proportion of allele comparisons which are \emph{not} IBS.
}
\usage{
ibsDist(counts)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{counts}{A matrix of IBS counts as produced by the function
    \code{\link{ibsCount}}}
}
\value{
  An object of class \code{"dist"} (see \code{\link[stats]{dist}})
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{ibsCount}}, \code{\link[stats]{dist}}}
\examples{
data(testdata)
ibs <- ibsCount(Xchromosome)
distance <- ibsDist(ibs)
}
\keyword{cluster}

\name{plotUncertainty}
\alias{plotUncertainty}
\title{
Plot posterior probabilities of genotype assignment
}
\description{
The snpStats package allows for storage of uncertain genotype
assignments in a one byte "raw" variable. The probabilities of
assignment form a three-vector, subject to the linear constraint that
they sum to 1.0; their possible values are grouped into 253 different
classes. This function displays counts of these classes on a
two-dimensional isometric plot.
}
\usage{
plotUncertainty(snp, nlevels = 10, color.palette = heat.colors(nlevels))
}
\arguments{
  \item{snp}{
    One or more columns of a \code{SnpMatrix} object
}
  \item{nlevels}{
    Probability cells are coloured according to frequency. This argument
    gives the number of colours that can be used
}
  \item{color.palette}{
    The colour palette to be used
}
}
\details{
The plot takes the form of an equilateral triangle in which each apex
represents a certain assignment to one of the three genotypes. A point
within the triangle represents, by the perpendicular distance from each
side, the three probabilities. Each of the 253 probability classes is
represented by a hexagonal cell, coloured according to its frequency in
the data, which is also written within the cell
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\examples{
## No example available yet
}
\keyword{hplot}

\name{XSnpMatrix-class}
\docType{class}
\alias{XSnpMatrix-class}
\alias{initialize,XSnpMatrix-method}
\alias{[,XSnpMatrix,ANY,ANY,ANY-method}
\alias{[<-,XSnpMatrix,ANY,ANY,XSnpMatrix-method}
\alias{coerce,XSnpMatrix,character-method}
\alias{coerce,SnpMatrix,XSnpMatrix-method}
\alias{show,XSnpMatrix-method}
\alias{summary,XSnpMatrix-method}

\title{Class "XSnpMatrix"}
\description{This class extends the  \code{\link{SnpMatrix-class}} to
    deal with SNPs on the X and Y chromosomes and mitocondrial SNPs.}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("XSnpMatrix", x,
  diploid)}.
Such objects have an additional \code{slot} to objects of class
\code{"SnpMatrix"}
consisting of  a logical array of the same length as the number of
rows. This array indicates whether genotypes in that row are diploid
(\code{TRUE}) or haploid (\code{FALSE} as, for example,  SNPs on the X
chromosome for males).
}
\section{Slots}{
	 \describe{
    \item{\code{.Data}:}{Object of class \code{"matrix"} and storage mode
      \code{"raw"}} 
    \item{\code{diploid}:}{Object of class \code{"logical"} indicating
      sex of samples}
  }
}
\section{Extends}{
Class \code{"SnpMatrix"}, directly, with explicit coerce.
Class \code{"matrix"}, by class \code{"SnpMatrix"}.
Class \code{"structure"}, by class \code{"SnpMatrix"}.
Class \code{"array"}, by class \code{"SnpMatrix"}.
Class \code{"vector"}, by class "SnpMatrix", with explicit coerce.
Class \code{"vector"}, by class "SnpMatrix", with explicit coerce.
}
\section{Methods}{
  \describe{
    \item{[]}{\code{signature(x = "XSnpMatrix", i = "ANY", j = "ANY",
	drop = "missing")}: subset extraction}
    \item{[<-}{\code{signature(x = "XSnpMatrix", i = "ANY", j = "ANY", "XSnpMatrix")}: subset
      assignment operation to replace part of an object}
    \item{coerce}{\code{signature(from = "XSnpMatrix", to =
	"character")}:  map to codes 0, 1, 2, or NA}
    \item{coerce}{\code{signature(from = "SnpMatrix", to =
	"XSnpMatrix")}:
      maps a SnpMatrix to an XSnpMatrix. Ploidy is inferred from the
      genotype data since haploid genotypes should always be coded as
      homozygous. After inferring ploidy, heterozygous calls for haploid
      genotpes are set to \code{NA}}
    \item{show}{\code{signature(object = "XSnpMatrix")}: map to codes
      "A/A", "A/B", "B/B", "A", "B" or ""}
    \item{summary}{\code{signature(object = "XSnpMatrix")}: returns
      the distribution of ploidy, together with
      summaries of the data frames returned by
      \code{\link{row.summary}} and \code{\link{col.summary}}}
  }
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{SnpMatrix-class}}
}
\examples{
data(testdata)
summary(Xchromosome)

# display the first 10 snps of the first 10 samples
print(as(Xchromosome[1:10,1:10],'character'))

# convert the empty strings (no-calls) explicitly to "NC" before
# writing to an (anonymous and temporary) csv file
csvfile <- tempfile()
write.csv(file=csvfile, gsub ('^$', 'NC',
                              as(Xchromosome[1:10,1:10], 'character')
                             ), quote=FALSE)
unlink(csvfile)
}
\keyword{classes}

\name{SnpMatrix-class}
\docType{class}
\alias{SnpMatrix-class}
\alias{initialize,SnpMatrix-method} 
\alias{[,SnpMatrix,ANY,ANY,ANY-method}
\alias{rbind2,SnpMatrix,SnpMatrix-method}
\alias{cbind2,SnpMatrix,SnpMatrix-method}
\alias{coerce,SnpMatrix,numeric-method}
\alias{coerce,SnpMatrix,character-method}
\alias{coerce,matrix,SnpMatrix-method}
\alias{is.na,SnpMatrix-method}
\alias{cbind,SnpMatrix-method}
\alias{rbind,SnpMatrix-method}
\alias{show,SnpMatrix-method}
\alias{summary,SnpMatrix-method}
\alias{switch.alleles,SnpMatrix,ANY-method}


\title{Class "SnpMatrix"}
\description{ This class defines objects holding large arrays of single
  nucleotide polymorphism (SNP) genotypes generated using array
  technologies.}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("SnpMatrix", x)}
  where \code{x} is a matrix with storage mode \code{"raw"}.
	 Chips (usually corresponding to samples or
	 subjects) define rows of the matrix while polymorphisms (loci) define
	 columns. Rows and columns will usually have names which can be
	 used to link the data to further data concerning samples and SNPs
}
\section{Slots}{
	 \describe{
    \item{\code{.Data}:}{Object of class \code{"matrix"} and storage
      mode \code{raw}  
    Internally, missing data are coded 0 and SNP genotypes are coded
    1, 2 or 3. Imputed values may not be known exactly. Such
    uncertain calls are grouped by probability and represented by codes
    4 to 253}
  }
}
\section{Extends}{
Class \code{"matrix"}, from data part.
Class \code{"structure"}, by class \code{"matrix"}.
Class \code{"array"}, by class \code{"matrix"}.
Class \code{"vector"}, by class "matrix", with explicit coerce.
Class \code{"vector"}, by class "matrix", with explicit coerce.
}
\section{Methods}{
  \describe{
    \item{[]}{\code{signature(x = "SnpMatrix", i = "ANY", j = "ANY",
	drop = "missing")}: subset
      operations}
    \item{cbind2}{\code{signature(x = "SnpMatrix", y = "SnpMatrix")}:
      S4 generic function to provide cbind() for two or
      more matrices together by column. Row names must match and column
      names must not coincide. If the matrices are of the derived class
      \code{\link{XSnpMatrix-class}}, the \code{diploid} slot values must also
      agree}
    \item{coerce}{\code{signature(from = "SnpMatrix", to = "numeric")}:
      map to numeric values 0, 1, 2 or, for uncertain assignments, to
      the posterior expectation of the 0, 1, 2 code}
    \item{coerce}{\code{signature(from = "SnpMatrix", to =
	"character")}: map to codes "A/A", "A/B", "B/B", ""}
    \item{coerce}{\code{signature(from = "matrix", to = "SnpMatrix")}:
      maps numeric matrix (coded 0, 1, 2 or NA) to a SnpMatrix}
    \item{coerce}{\code{signature(from = "SnpMatrix", to =
	"XSnpMatrix")}:
      maps a SnpMatrix to an XSnpMatrix. Ploidy is inferred from the
      genotype data since haploid genotypes should always be coded as
      homozygous.  
      After inferring ploidy, heterozygous calls for haploid genotypes
      are set to \code{NA}}
    \item{is.na}{\code{signature(x = "SnpMatrix")}: returns a logical
      matrix indicating whether each element is \code{NA}}
    \item{rbind2}{\code{signature(x = "SnpMatrix", y = "snp.matrix")}:
      S4 generic function to provide rbind() for two or
      more matrices by row. Column names must match and duplicated row
      names prompt warnings}
    \item{show}{\code{signature(object = "SnpMatrix")}: shows the size
      of the matrix (since most objects will be too large to show in full)}
    \item{summary}{\code{signature(object = "SnpMatrix")}: returns
      summaries of the data frames returned by
      \code{\link{row.summary}} and \code{\link{col.summary}}}
    \item{is.na}{\code{signature(x = "SnpMatrix")}: returns a logical
      matrix of missing call indicators}
    \item{switch.alleles}{\code{signature(x = "SnpMatrix", snps
	="ANY")}: Recode specified columns of of the matrix to reflect
      allele switches}

	 }
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\note{This class requires at least version 2.3 of R}
\seealso{\code{\link{XSnpMatrix-class}}
}
\examples{
data(testdata)
summary(Autosomes)

# Just making it up - 3-10 will be made into NA during conversion
snps.class<-new("SnpMatrix", matrix(1:10))
snps.class
if(!isS4(snps.class)) stop("constructor is not working")

pretend.X <- as(Autosomes, 'XSnpMatrix')
if(!isS4(pretend.X)) stop("coersion to derived class is not S4")
if(class(pretend.X) != 'XSnpMatrix') stop("coersion to derived class is not working")

pretend.A <- as(Xchromosome, 'SnpMatrix')
if(!isS4(pretend.A)) stop("coersion to base class is not S4")
if(class(pretend.A) != 'SnpMatrix') stop("coersion to base class is not working")

# display the first 10 snps of the first 10 samples
print(as(Autosomes[1:10,1:10], 'character'))

# convert the empty strings (no-calls) explicitly to "NC" before
# writing to an (anonymous and temporary) csv file
csvfile <- tempfile()
write.csv(file=csvfile, gsub ('^$', 'NC',
                              as(Autosomes[1:10,1:10], 'character')
                             ), quote=FALSE)
unlink(csvfile)
}
\keyword{classes}

\name{row.summary}
\alias{row.summary}
\alias{col.summary}
\title{Summarize rows or columns of a snp matrix}
\description{
  These function calculates summary statistics of each row or column of
  call rates and heterozygosity for each row of a an
  object of class \code{"SnpMatrix"} or \code{"XSnpMatrix"}
}
\usage{
row.summary(object)
col.summary(object, rules = NULL, uncertain = TRUE)
}
\arguments{
\item{object}{genotype data as a \code{\link{SnpMatrix-class}} or
  \code{\link{XSnpMatrix-class}} object}
\item{rules}{An object of class
  \code{"ImputationRules"}. If
  supplied, the rules coded in this object are used, together with
  the snp genotype data in
  \code{object}, to generate imputed SNPs. The column summary of these
  imputed data are then returned}
\item{uncertain}{If \code{TRUE} uncertain genotypes are used in
  calculation of allele and genotype frequencies (by scoring as
  posterior expectations). Otherwise, and for Hardy-Weinberg tests,
  they are ignored}
}
\value{
  \item{row.summary}{
    returns a data frame with rows corresponding to rows of the input object
    and with columns/elements:
    \itemize{
      \item Call.rate: Proportion of SNPs called
      \item Certain.calls: Proportion of called SNPs with certain calls
      \item Heterozygosity: Proportion of called SNPs which are
	  heterozygous
	}
	Uncertain calls are ignored for calculating the heterozygosity.
    	
	
  }
  \item{col.summary}{
    returns a data frame with rows corresponding to columns of the input object
    and with columns/elements:
    \itemize{
      \item Calls: The number of valid calls
      \item Call.rate: The proportion of genotypes called
      \item Certain.calls: Proportion of called SNPs with certain calls
      \item RAF: The "risk" allele (allele \code{B}) frequency
      \item MAF: The minor allele frequency
      \item P.AA: The frequency of homozygous genotype 1 (A/A)
      \item P.AB: The frequency of heterozygous genotype 2 (A/B)
      \item P.BB: The frequency of homozygous genotype 3 (B/B)
      \item z.HWE: A z-test for Hardy-Weinberg equilibrium
    }
    For objects of class \code{"XSnpMatrix"}, the following
    additional columns are returned:
    \itemize{
      \item P.AY: The frequency of allele A in males
      \item P.BY: The frequency of allele B in males
      \item Calls.female: The number of valid calls in females (only
	  these calls are used in the z-test for HWE)
      }
  }
}
\note{The current version of \code{row.summary}
  does not deal with the X chromosome
  differently, so that males are counted as homozygous.}
\author{David Clayton \email{dc208@cam.ac.uk}}
\examples{
data(testdata)
rs <- row.summary(Autosomes)
summary(rs)
cs <- col.summary(Autosomes)
summary(cs)
cs <- col.summary(Xchromosome)
summary(cs)
}
\keyword{utilities}

\name{impute.snps}
\alias{impute.snps}
\title{Impute snps}
\description{
Given SNPs stored in an object of class \code{"SnpMatrix"} or
\code{"XSnpMatrix"} and a set of imputation rules in an object of
class \code{"ImputationRules"}, this function calculates imputed values.
}
\usage{
impute.snps(rules, snps, subset = NULL, as.numeric = TRUE)
}
\arguments{
  \item{rules}{The imputation rules; an object of class
    \code{"ImputationRules"}}
  \item{snps}{The object of class \code{"SnpMatrix"} or
    \code{"XSnpMatrix"} containing the observed SNPs}
  \item{subset}{A vector describing the subset of subjects
    to be used. If \code{NULL} (default), then use all subjects}
  \item{as.numeric}{If \code{TRUE}, the output is a numeric matrix
    containing posterior expectations of the imputed SNPs. Otherwise the
    output matrix is of the same class as \code{snps} and contains
    uncertain genotype calls}
}
\value{
  A matrix with imputed SNPs as columns. The imputed values are the
  estimated expected values of each SNP when coded 0, 1 or 2. 
}
\references{
  Wallace, C. et al. (2010) \emph{Nature Genetics}, \bold{42}:68-71
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{snp.imputation}}}
\examples{
# Remove 5 SNPs from a datset and derive imputation rules for them
data(for.exercise)
sel <- c(20, 1000, 2000, 3000, 5000)
to.impute <- snps.10[,sel]
impute.from <- snps.10[,-sel]
pos.to <- snp.support$position[sel]
pos.fr <- snp.support$position[-sel]
imp <- snp.imputation(impute.from, to.impute, pos.fr, pos.to)
# Now calculate the imputed values
imputed <- impute.snps(imp, impute.from)
}
\keyword{models}
\keyword{regression}

\name{mvtests}
\alias{mvtests}
\title{Multivariate SNP tests
}
\description{
  This function calculates multivariate score tests between a multivariate
  (or multinomial) phenotype and sets of SNPs
}
\usage{
mvtests(phenotype, sets, stratum, data = sys.parent(), snp.data, rules = NULL, complete = TRUE, uncertain = FALSE, score = FALSE)
}
\arguments{
  \item{phenotype}{
    Either a factor (for a multinomial phenotype) or a matrix (for a
    multivariate phenotype)
  }
  \item{sets}{
    A list of sets of SNPs to be tested against the phenotype
  }
  \item{stratum}{
    (Optional) a stratifying variable
  }
  \item{data}{
    A data frame in which \code{phenotype} and \code{stratum} reside. If
    absent these are assumed to be in the parent frame and correctly
    aligned with the rows of \code{snp.data}
  }
  \item{snp.data}{
    An object of class \code{SnpMatrix} containing the SNP data
  }
  \item{rules}{
    (Optional) A set of imputation rules. The function then carries out
    tess on imputed SNPs
  }
  \item{complete}{
    If \code{TRUE} each test will use only subjects who have complete
    data for the phenotype and all SNPs in the set to be tested. If
    \code{FALSE}, then complete data for the phenotype is required, but
    tests are based upon complete pairs of SNPs
  }
  \item{uncertain}{
    If \code{TRUE}, uncertain genotype calls will be used in the
    tests (scored by their posterior expectations).
    Otherwise such calls are treated as missing
  }
  \item{score}{
    If \code{TRUE}, the score vectors and their variance-covariance
    matrices are saved in the output object for further processing
  }
}
\details{
  Currently \code{complete=FALSE} is not implemented
}
\value{
  An object of class \code{\link[=GlmTests-class]{snp.tests.glm}}
  or \code{\link[=GlmTestsScore-class]{GlmTests.score}}
  depending on whether \code{score} is set to \code{FALSE} or \code{TRUE}
  in the call
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\note{
  This is an experimental version
}
\examples{
## No example yet
}
\keyword{htest}

\name{random.snps}
\alias{random.snps}
\title{
Generate random SnpMatrix 
}
\description{
This function is purely for testing purposes. It can generate SnpMatrix
objects which contain more than 2^31-1 elements.
}
\usage{
random.snps(nrows, ncols)
}
\arguments{
  \item{nrows}{The number of rows to be generated}
  \item{ncols}{The number of columns to be generated}
}
\details{
All SNPs should be in Hardy-Weinberg equilibrium with an allele
frequency of 0.5.

Note that, although the total number of elements can exceed 2^31-1, the
numbers of rows and columns are still subject to this limit.
}
\examples{
x <- random.snps(100,10)
col.summary(x)
}
\keyword{utilities}

\name{GlmEstimates-class}
\Rdversion{1.1}
\docType{class}
\alias{GlmEstimates-class}
\alias{[,GlmEstimates,ANY,missing,missing-method}
\alias{coerce,GlmEstimates,GlmTests-method}
\alias{show,GlmEstimates-method}

\title{Class "GlmEstimates"}
\description{A simple class to hold output from
  \code{\link{snp.lhs.estimates}} and
  \code{\link{snp.rhs.estimates}}. Its main purpose is to provide a
  \code{show} method}
\section{Objects from the Class}{
Objects from this class are simple lists. Each element of the list is a
list giving the results of a generalized linear model fit, with elements:
\describe{
  \item{Y.var}{Name of the Y variable}
  \item{beta}{The vector or parameter estimates (with their names)}
  \item{Var.beta}{The upper triangle of the variance-covariance matrix
    of estimates, stored as a simple vector}
  \item{N}{The number of "units" used in the model fit}
  }
}
\section{Extends}{
Class \code{"\linkS4class{list}"}, from data part.
Class \code{"\linkS4class{vector}"}, by class "list", distance 2.
}
\section{Methods}{
  \describe{
    \item{[}{\code{signature(x = "GlmEstimates", i = "ANY", j = "missing", drop = "missing")}: Subset }
    \item{coerce}{\code{signature(from = "GlmEstimates", to =
	"GlmTests")}: Calculate Wald tests }
    \item{show}{\code{signature(object = "GlmEstimates")}: Display}
	 }
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\note{Wald tests calculated by coercing an object of class
  \code{GlmEstimates} to class \code{GlmTests} are \emph{asymptotically}
  equivalent to the "score" tests calculated by
  \code{\link{snp.lhs.tests}} and \code{\link{snp.rhs.tests}}, and both types
  of test are \emph{asymptotically} correct. But the asymptotic properties of
  Wald tests are not as good as those of score tests and, in
  circumstances such as low sample size or low minor allele frequency,
  Wald and score tests may differ substantially. In general score tests
  are to be preferred, but Wald tests are provided in \code{snpStats}
  because they are widely used.
}
\seealso{
  \code{\link{snp.lhs.estimates}},
  \code{\link{snp.rhs.estimates}}
}
\examples{
showClass("GlmEstimates")
}
\keyword{classes}

\name{read.plink}
\alias{read.plink}
\title{Read a PLINK binary data file as a SnpMatrix}
\description{
The package PLINK saves genome-wide association data in groups of three
files, with the extensions \code{.bed}, \code{.bim}, and
\code{.fam}. This function reads these files and creates an object of
class \code{"SnpMatrix"}
}
\usage{
read.plink(bed, bim, fam, na.strings = c("0", "-9"), sep = "." , select.subjects = NULL, select.snps = NULL) 
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{bed}{The name of the
    file containing the packed binary SNP genotype data. It should have
    the extension \code{.bed}; if it doesn't, then this extension will
    be appended}
  \item{bim}{The file containing the SNP descriptions}
  \item{fam}{The file containing subject (and, possibly, family)
    identifiers. This is basically a tab-delimited "pedfile"}
  \item{na.strings}{Strings in .bam and .fam files to be recoded as NA}
  \item{sep}{A separator character for constructing unique subject
    identifiers}
  \item{select.subjects}{A numeric vector indicating a
    subset of subjects to be selected from the input file (see details)} 
  \item{select.snps}{Either a numeric or a character vector indicating a
   subset of SNPs to be selected from the input file (see details)}
}
\details{
  If the \code{bed} argument does not contain a filename with the file
  extension \code{.bed}, then this extension is appended to the
  argument. The remaining two arguments are optional; their default
  values are obtained by replacing the \code{.bed} filename extension by
  \code{.bim} and \code{.fam} respectively. See the PLINK documentation
  for the detailed specification of these files.

  The \code{select.subjects} or \code{select.snps} argument can be used
  to read a subset of the data. Use of \code{select.snps} requires that
  the \code{.bed} file is in SNP-major order (the default in
  PLINK). Likewise, use of \code{select.snps} requires that
  the \code{.bed} file is in individual-major order. Subjects are
  selected by their numeric order in the PLINK files, while SNPs are
  selected either by order or by name.  Note that
  the order of selected SNPs/subjects in the output objects
  will be the same as
  their order in the PLINK files. 

  Row names for the output \code{SnpMatrix} object and for the
  accompanying subject description dataframe are taken as the pedigree
  identifiers, when these provide the required unique identifiers. When
  these are duplicated, an attempt is made to use the pedigree-member
  identifiers instead but, when these too are duplicated,  
  row names are obtained by concatenating, with a separator character, the
  pedigree and pedigree-member identifiers.
}
\value{
  A list with three elements:
  \item{genotypes}{The output genotype data as an object of class}
  \code{"SnpMatrix"}. 
  \item{fam}{A dataframe corresponding to the \code{.fam} file,
  containing the first six fields in a standard pedfile.
  The row names will correspond with those of the \code{SnpMatrix}}
  \item{map}{A dataframe correponding to the \code{.bim} file. the row
    names correpond with the column names of the  \code{SnpMatrix}}
}
\note{
  No special provision is made to read \code{XSnpMatrix}
  objects; such data should first be read as a \code{SnpMatrix} and then
  coerced to an \code{XSnpMatrix} using \code{new} or \code{as}.
}
\references{PLINK: Whole genome association analysis toolset.
  \url{http://pngu.mgh.harvard.edu/~purcell/plink/}
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{write.plink}},
  \code{\link{SnpMatrix-class}}, \code{\link{XSnpMatrix-class}}}
\keyword{manip}
\keyword{IO}
\keyword{file}
\keyword{utilities}

\name{example-new}
\alias{example-new}
\docType{data}
\title{
An example of intensity data for SNP genotyping
}
\description{
The file \code{example-new.txt} contains some signal intensity data for
testing and comparing genotype scoring algorithms
}
\format{
  This is a text file containing data on 99 SNPs for 1550 DNA
  samples. One line of data appears for each SNP, starting with the
  SNP name and followed by 1550 pairs of intensity values. There is a
  header line containing variable names, with intensities labelled
  as \code{xxxxA} and \code{xxxxB}, where \code{xxxx} is the sample
  name.
}
\details{
See the package vignette "Comparing clustering algorithms".
}
\source{These data were originally distributed with the "Illuminus"
  genotype scoring software from the Wellcome Trust
  Sanger Institute: 
  \url{http://www.sanger.ac.uk/resources/software/illuminus/}
}
\keyword{datasets}

\name{pool2}
\alias{pool2}
\title{Pool results of tests from two independent datasets}
\description{
Generic function to pool results of tests from two independent
datasets. It is not designed to be called directly, but is called
recursively by \code{\link{pool}}
}
\usage{
pool2(x, y, score)
}
\arguments{
  \item{x, y}{Objects holding the (extended) test results. 
    These must be of class
    \code{\link[=SingleSnpTestsScore-class]{SingleSnpTests.score}}
    or
    \code{\link[=GlmTests-class]{snp.tests.glm}}}
  \item{score}{Is extended score information to
  be returned in the output object?}
}
\value{
  An object of same class as the input objects (optionally without the
  \code{.score}) extension. Tests are produced for the \emph{union} of
  SNPs tested in all the input objects. 
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{pool}},
  \code{\link{SingleSnpTestsScore-class}},
  \code{\link{GlmTests-class}},
  \code{\link{single.snp.tests}},
  \code{\link{snp.lhs.tests}},
  \code{\link{snp.rhs.tests}}
  }
\keyword{htest}

\name{testdata}
\docType{data}
\alias{testdata}
\alias{Autosomes}
\alias{Xchromosome}
\alias{Asnps}
\alias{Xsnps}
\alias{subject.data}
\title{Test data for the snpStats package}
\description{
  This dataset comprises several data frames from a fictional (and
  unrealistically small) study. The
  dataset started off as real data from a screen of non-synonymous SNPs
  for association with type 1 diabetes, but the original identifiers have
  been removed and a random case/control status has been generated.
}
\usage{data(testdata)}
\format{
  There are five data objects in the dataset:
  \itemize{
    \item \code{Autosomes}: An object of class \code{"SnpMatrix"}
    containing genotype calls for 400 subjects at 9445 autosomal SNPs
    \item \code{Xchromosome}: An object of class \code{"XSnpMatrix"}
    containing genotype calls for 400 subjects at 155 SNPs on the X
    chromosome
    \item \code{Asnps}: A dataframe containing information about the
    autosomal SNPs. Here it contains only one variable,
    \code{chromosome}, indicating the chromosomes on which the SNPs are
    located
    \item \code{Xsnps}: A dataframe containing information about the X
    chromosome SNPs. Here it is empty and is only included for
    completeness
    \item \code{subject.data}: A dataframe containing information about
    the subjects from whom each row of SNP data was obtained. Here it
    contains:
    \itemize{
      \item \code{cc}: Case-control status
      \item \code{sex}: Sex
      \item \code{region}: Geographical region of residence
    }
  }
}
\source{
  The data were obtained from the diabetes and inflammation laboratory
  (see \url{http://www-gene.cimr.cam.ac.uk})
}
\examples{
data(testdata)
Autosomes
Xchromosome
summary(Asnps)
summary(Xsnps)
summary(subject.data)
summary(summary(Autosomes))
summary(summary(Xchromosome))
}
\keyword{datasets}

\name{snp.pre.multiply}
\alias{snp.pre.multiply}
\alias{snp.post.multiply}
\title{Pre- or post-multiply a SnpMatrix object by a general matrix}
\description{
  These functions first standardize the input \code{SnpMatrix} in the
  same way as does the function \code{\link{xxt}}. The standardized
  matrix is then either pre-multiplied (\code{snp.pre.multiply}) or
  post-multiplied (\code{snp.post.multiply}) by a general matrix. Allele
  frequencies for standardizing the input SnpMatrix may be supplied
  but, otherwise, are calculated from the input SnpMatrix
}
\usage{
snp.pre.multiply(snps, mat, frequency=NULL, uncertain = FALSE)
snp.post.multiply(snps, mat, frequency=NULL, uncertain = FALSE)
}
\arguments{
  \item{snps}{An object of class \code{"SnpMatrix"} or \code{"XSnpMatrix"}}
  \item{mat}{A general (numeric) matrix}
  \item{frequency}{A numeric vector giving the allele (relative)
    frequencies to be used for standardizing the columns of \code{snps}.
    If \code{NULL}, allele frequencies will be calculated
    internally. Frequencies should refer to the second (\code{B}) allele
  }
  \item{uncertain}{If \code{TRUE}, uncertain genotypes are replaced by
    posterior expectations. Otherwise these are treated as missing values}
}
\details{
The two matrices must be conformant, as with standard matrix
multiplication. The main use envisaged for these functions is the
calculation of factor loadings in principal component analyses of large
scale SNP data, and the application of these loadings to other
datasets. The use of externally supplied allele frequencies for
standardizing the input SnpMatrix is required when applying loadings
calculated from one dataset to a different dataset
}
\value{
  The resulting matrix product
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{xxt}}}
\examples{
##--
##-- Calculate first two principal components and their loading, and verify
##--	
# Make a SnpMatrix with a small number of rows
data(testdata)
small <- Autosomes[1:20,]
# Calculate the X.X-transpose matrix
xx <- xxt(small, correct.for.missing=FALSE)
# Calculate the first two principal components and corresponding eigenvalues
eigvv <- eigen(xx, symmetric=TRUE)
pc <- eigvv$vectors[,1:2]
ev <- eigvv$values[1:2]
# Calculate loadings for first two principal components
Dinv <- diag(1/sqrt(ev))
loadings <- snp.pre.multiply(small,  Dinv \%*\% t(pc))
# Now apply loadings back to recalculate the principal components
pc.again <- snp.post.multiply(small, t(loadings) \%*\% Dinv)
print(cbind(pc, pc.again))
}
\keyword{array}
\keyword{multivariate}

\name{Fst}
\alias{Fst}
\title{
Calculate fixation indices
}
\description{
This function calculates the fixation index Fst for each SNP, together
with its weight in the overall estimate (as used by the Internation
HapMap Consortium). 
}
\usage{
Fst(snps, group, pairwise=FALSE)
}
\arguments{
  \item{snps}{an object of class \code{SnpMatrix} or
    \code{XSnpMatrix} containing the  SNP data}
  \item{group}{a factor (or object than can be coerced into a factor),
    of length equal to the number of rows of \code{snps}, giving the
    grouping or rows for which the Fst is to be calculated}	 
  \item{pairwise}{if \code{TRUE}, the within-group variances are weighted 
    according to the number of possible within-group pairwise comparisons 
    of chromosomes. If \code{FALSE}, the default value, 
    weights are simply the number of chromosomes in each group.}
}
\details{
See vignette.
}
\value{
  A list:
  \describe{
    \item{Fst}{Fst values for each SNP}
    \item{weight}{The weights for combining these into a single index}
    }
}
\note{Uncertain genotypes are treated as missing
}
\author{David Clayton \email{dc208@cam.ac.uk}
}
\examples{

## Analysis of some HapMap data

data(for.exercise)
f <- Fst(snps.10, subject.support$stratum)
weighted.mean(f$Fst, f$weight)

}
\keyword{univar}

\name{misinherits}
\alias{misinherits}
\title{Find non-Mendelian inheritances in family data}
\description{
For SNP data in families, this function locates all subjects whose
parents are in the dataset and tests each SNP for non-Mendelian
inheritances in these trios.
}
\usage{
misinherits(ped, id, father, mother, data = sys.parent(), snp.data)
}
\arguments{
  \item{ped}{Pedigree identifiers}
  \item{id}{Subject identifiers}
  \item{father}{Identifiers for subjects' fathers}
  \item{mother}{Identifiers for subjects' mothers}
  \item{data}{A data frame in which to evaluate the previous four arguments}
  \item{snp.data}{An object of class \code{"SnpMatrix"} containing the SNP
    genotypes to be tested}
}
\details{
    The first four arguments are usually derived from  a "pedfile". If a 
  data frame is supplied for the \code{data} argument, the first four
  arguments will be evaluated in this frame. Otherwise they will be evaluated 
  in the calling environment. If the arguments are missing, they will be 
  assumed to be in their usual positions in the pedfile data frame 
  i.e. in columns one to four.  If the pedfile data are obtained from
  a dataframe, the row names of the \code{data} and \code{snp.data}
  files will be used to align the pedfile and SNP data. Otherwise, these
  vectors will be assumed to be in the same order as the rows of
  \code{snp.data}. 
}
\value{
  A logical matrix. Rows are subjects with any non-Mendelian
  inheritances and columns are SNPs with any non-Mendelian
  inheritances. The body of the matrix details whether each subject has
  non-Mendelian inheritance at each SNP. If a subject has no recorded
  genotype for a specific SNP, the corresponding element of the output
  matrix is set to NA.
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{tdt.snp}}}
\examples{
data(families)
misinherits(data=pedData, snp.data=genotypes)
}
\keyword{manip}

\name{switch.alleles}
\alias{switch.alleles}
\title{Switch alleles in columns of a SnpMatrix or in test results}
\description{
  This is a generic function which can be applied to objects of class
  \code{"SnpMatrix"} or  \code{"XSnpMatrix"} (which hold SNP genotype
  data), or to objects of class 
  \code{"SingleSnpTestsScore"} or \code{"GlmTests"} (which hold
  association test results). In the former case, specified SNPs can be
  recoded as if the alleles were switched (so that \emph{AA} genotypes
  become \emph{BB} and 
  vice-versa while \emph{AB} remain unchanged). In the latter case,
  test results are modified \emph{as if} alleles had been switched. 
}
\usage{
switch.alleles(x, snps)
}
\arguments{
  \item{x}{The input object, of class  \code{"SnpMatrix"},
    \code{"XSnpMatrix"}, \code{"SingleSnpTestsScore"}, or
    \code{"GlmTests"}}
  \item{snps}{A vector of type integer, character or logical specifying
      the SNP to have its alleles switched}
}
\value{
  An object of the same class as the input object
}
\note{
  Switching alleles for SNPs has no effect on test results. These
  functions are required when carrying out meta-analysis, bringing
  together several sets of results. It is then important that alleles
  line up in the datasets to be combined. It is often more convenient
  (and faster) to apply this process to the test result objects rather
  than to the genotype data themselves.
  }
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{  \code{\link{SnpMatrix-class}},
  \code{\link{XSnpMatrix-class}},
  \code{\link{SingleSnpTests-class}},
  \code{\link{GlmTests-class}}
}
\examples{
data(testdata)
which <-  c("173774", "173811")
Asw <- switch.alleles(Autosomes, which)
col.summary(Autosomes[,which])
col.summary(Asw[,which])
}
\keyword{utilities}

\name{ld}
\alias{ld}
\title{
  Pairwise linkage disequilibrium measures
}
\description{
  This function calculates measures of linkage disequilibrium between
  pairs of SNPs. The two SNPs in each pair may both come from the same
  \code{SnpMatrix} object, or from two different \code{SnpMatrix}
  objects. Statistics which can be calculated are the
  log likelihood ratio, odds ratio, Yule's Q, covariance, D-prime,
  R-squared, and R.
}
\usage{
ld(x, y = NULL, depth = NULL, stats, symmetric = FALSE)
}
\arguments{
  \item{x}{
    An object of class \code{SnpMatrix} or \code{XSnpMatrix}
  }
  \item{y}{
    (Optional) Another object of the same class as \code{x}. If \code{y}
    is supplied, LD statistics are calculated between each column of
    \code{x} and each column of \code{y}. Otherwise, they are calculated
    between columns of \code{x}
  }
  \item{depth}{
    When \code{y} is not supplied, this parameter is mandatory and
    controls the maximum lag between columns of \code{x}
    considered. Thus, LD statistics are calculated between \code{x[,i]}
    and \code{x[,j]} only if \code{i} and \code{j} differ by no more
    than \code{depth}
  }
  \item{stats}{
    A character vector specifying the linkage disequilibrium measures to
    be calculated. This should contain one or more of the strings: \code{"LLR"},
    \code{"OR"}, \code{"Q"}, \code{"Covar"}, \code{"D.prime"},
    \code{"R.squared"}, ad \code{"R"}
  }
  \item{symmetric}{
    When no \code{y} argument is supplied this argument controls the format of
    the output band matrices. If \code{TRUE}, symmetric matrices are
    returned and, otherwise, an upper triangular matrices are returned
  }
}
\details{
For each pair of SNPs, phased haplotype frequencies are first estimated
by maximum likelihood using the method described by Clayton and Leung
(2007). The arrays of chosen LD statistics are then calculated and
returned, either as
band matrices (when \code{y} is not supplied), or as conventional
rectangular matrices (when \code{y} is supplied). Band matrices are
stored in compressed form as objects of class \code{dsCMatrix}
(symmetric) or \code{dgCMatrix} (upper triangular). These classes are
defined in the \code{"\link[Matrix:Matrix-class]{Matrix}"} package)
}
\value{
  If only one LD statistic is requested, the function returns either a
  matrix or a compressed band matrix. If more than one LD statistic is
  requested, a list of such objects is returned
}
\references{
  Clayton and Leung (2007) \emph{Human Heredity}, \bold{64}:45-51, 
  (this paper is included in package documentation)
}
\author{
David Clayton \email{dc208@cam.ac.uk}
}
\seealso{
\code{"\link[Matrix:Matrix-class]{Matrix-class}"}
}
\examples{
data(testdata)
ld1 <- ld(Autosomes[, 1:50], depth=10, stats=c("D.prime", "R.squared"))
ld2 <- ld(Autosomes[, 1:20], Autosomes[, 21:25], stats="R.squared") 
}
\keyword{misc}

\name{read.long}
\alias{read.long}
\title{
Read SNP genotype data in long format
}
\description{
This function reads SNP genotype data from a file in which each line
refers to a single genotype call. Replaces the earlier function
\code{read.snps.long}.
}
\usage{
read.long(file, samples, snps,
            fields = c(snp = 1, sample = 2, genotype = 3, confidence = 4,
                       allele.A = NA, allele.B = NA),
            split = "\t| +", gcodes, no.call = "", threshold = NULL,
            lex.order = FALSE, verbose = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{file}{
    Name(s) of file(s) to be read (can be gzipped)
}
  \item{samples}{
    Either a vector of sample identifiers, or the number of samples to
    be read. If a single file is to be read and this argument is
    omitted, the file will be scanned initially and all
    samples will be included
}
  \item{snps}{
    Either a vector of SNP identifiers, or the number of SNPs to
    be read. If a single file is to be read and this argument is
    omitted, the file will be scanned initially and all
    SNPs will be included
  }
  \item{fields}{
    A named vector giving the locations of the required fields. See
    Details below
  }
  \item{split}{
    A regular expression specifying how the input line will be split
    into fields. The default value specifies separation of fields by a
    TAB character, or by one or more blanks
}
  \item{gcodes}{
    When the genotype is read as a single field, this argument specifies
    how it is handled. See Details below.
}
  \item{no.call}{
    The string which indicates "no call" for either a genotype or (when
    the genotype is read as two allele fields) an allele
}
  \item{threshold}{
    A vector of length 2 giving the lower and higher acceptable limits
    for the confidence score
}
  \item{lex.order}{
    If \code{TRUE}, the alleles at each locus will be in lexographical
    order. Otherwise, ordering of alleles is arbitrary, depending on
    the order in which they are encountered
}
  \item{verbose}{
    If \code{TRUE}, this turns on output from the function. Otherwise
    only error and warning messages are produced
}
}
\details{
Each line on the input file represents a single call and is split into
fields using the function \code{strsplit}. The required fields are
extracted according to the \code{fields} argument. This \emph{must}
contain the locations of the sample and snp identifier
fields and \emph{either} the location of a genotype field \emph{or} the
locations of two allele fields.

If the \code{samples} and \code{snps} arguments contain vectors of
character strings, a \code{SnpMatrix} is created with these row and
column names and the  genotype values are "cherry-picked" from the input
file. If either, or both, of these arguments are specified simply as
numbers, then these
numbers determine the \emph{dimensions} of the \code{SnpMatrix}
created. In this case samples and/or SNPs are included in the
\code{SnpMatrix} on a first-come-first-served basis. If either
or both of these arguments are omitted, a preliminary scan of the input file
is carried out to find the missing sample and/or SNP identifiers. 
In this scan, 
when a sample or SNP identifier differs from that in the previous
line, but is identical to one previously found, then all the relevant
identifiers are assumed to have been found. This implies that
the file must be sorted, in some consistent order,
by sample and by SNP (although either one of these may vary fastest).

If the genotype is to be read as a single field, the \code{genotype}
element of the \code{fields} argument must be set to the appropriate
value, and the \code{allele.A} and \code{allele.B} elements should be
set to \code{NA}. Its handling is controlled
by the \code{gcodes} argument. If this is missing or \code{NA},  then
the genotype is assumed to be represented by a two-character field,
the two characters representing the two alleles. If \code{gcodes} is
a single string, then it is assumed to contain
a regular expression which will split the genotype field into two allele
fields. Otherwise, \code{gcode} must be an array of length three,
specifying the three genotype codes in the order "AA", "AB", "BB".

If the two alleles of the genotype are to be read from two separate
fields, the \code{genotype} element should be set to \code{NA} and the
\code{allele.A} and \code{allele.B} elements set to the appropriate
values. The \code{gcode} argument should be missing or set to \code{NA}.
}
\value{
If the genotype is read as a single field matching one of three
specified codes, the function returns an object of class
\code{SnpMatrix}. Otherwise it returns a list whose first element is the
\code{SnpMatrix} object and whose second element is a dataframe
containing the allele codes, with the SNP identifiers as row names. Note
that allele codes only occur in this file if they occur in a genotype
which was accepted. Thus, monomorphic SNPs have \code{allele.B} coded as
\code{NA}, and SNPs which never pass confidence score filters have both
alleles coded as \code{NA}.
}
\author{
David Clayton \email{dc208@cam.ac.uk}
}
\note{
  Unlike \code{read.snps.long},
  this function is written entirely in R and may not be particularly
  fast. However, it imposes no restrictions on the allele codes
  recognized.

  Homozygous genotypes are assumed to be represented in the input file 
  by coding both alleles to the same value. No special provision is made
  to read \code{XSnpMatrix} 
  objects; such data should first be read as a \code{SnpMatrix} and then
  coerced to an \code{XSnpMatrix} using \code{new} or \code{as}.
}
\seealso{
\code{\link{SnpMatrix-class}}, \code{\link{XSnpMatrix-class}}
}
\examples{
##
## No example supplied yet
##
}
\keyword{manip}
\keyword{IO}
\keyword{file}
\keyword{utilities}

\name{qq.chisq}
\alias{qq.chisq}
\title{Quantile-quantile plot for chi-squared tests}
\description{
This function plots ranked observed chi-squared test statistics against
the corresponding expected order statistics. It also estimates an
inflation (or deflation) factor, lambda, by the ratio of the trimmed
means of observed and expected values. This is useful for
inspecting the results of whole-genome association studies for
overdispersion due to population substructure and other sources of bias
or confounding.
}
\usage{
qq.chisq(x, df=1, x.max, main="QQ plot", 
    sub=paste("Expected distribution: chi-squared (",df," df)", sep=""), 
    xlab="Expected", ylab="Observed",
    conc=c(0.025, 0.975), overdisp=FALSE, trim=0.5,  
    slope.one=FALSE, slope.lambda=FALSE, pvals=FALSE,
    thin=c(0.25,50), oor.pch=24, col.shade="gray", ...)
}
\arguments{
  \item{x}{A vector of observed chi-squared test values}
  \item{df}{The degreees of freedom for the tests}
  \item{x.max}{If present, truncate the observed value (Y) axis at
    \code{abs(x.max)}. If \code{x.max} is negative, the y-axis will
    extend to \code{abs(x.max)} even if the observed data do not}
  \item{main}{The main heading}
  \item{sub}{The subheading}
  \item{xlab}{x-axis label (default "Expected")}
  \item{ylab}{y-axis label (default "Observed")}
  \item{conc}{Lower and upper probability bounds for concentration band
    for the plot. Set this to \code{NA} to suppress this}
  \item{overdisp}{If \code{TRUE}, an overdispersion factor, lambda, will be 
    estimated and used in calculating concentration  band} 
  \item{trim}{Quantile point for trimmed mean calculations for estimation of 
    lambda. Default is to trim at the median}
  \item{slope.one}{Is a line of slope one to be superimpsed?}
  \item{slope.lambda}{Is a line of slope lambda 
    to be superimposed?}
  \item{pvals}{Are P-values to be indicated on an axis drawn
     on the right-hand side of the plot?}
  \item{thin}{A pair of numbers indicating how points will be thinned
    before plotting (see Details).
    If \code{NA}, no thinning will be carried out}
  \item{oor.pch}{Observed values greater than \code{x.max} are plotted
    at \code{x.max}. This argument sets the plotting symbol to be used
    for out-of-range observations}
  \item{col.shade}{The colour with which the concentration band will be
    filled}
  \item{...}{Further graphical parameter settings to be passed to
    \code{points()}}
}
\details{
  To reduce plotting time and the size of plot files, the smallest
  observed and expected points are thinned so that only a reduced number of
  (approximately equally spaced) points are plotted. The precise
  behaviour is controlled by the parameter
  \code{thin}, whose value should be a pair of numbers.
  The first number must lie
  between 0 and 1 and sets the proportion of the X axis over which 
  thinning is to be applied. The second number should be an integer and
  sets the maximum number of points to be plotted in this section.
  
  The "concentration band" for the plot is shown in grey. This region is 
  defined by upper and lower probability bounds for each order statistic. 
  The default is to use the 2.5% and 97.5% bounds.
  Note that this is not a simultaneous confidence region; the probability 
  that the plot will stray outside the band at some point exceeds 95%.

  When required, the dispersion factor is estimated by the ratio 
  of the observed trimmed mean to its expected value under the chi-squared 
  assumption.
}
\value{
  The function returns the number of tests, the number of values omitted
  from the plot (greater than \code{x.max}), and the estimated
  dispersion factor, lambda. 
}
\note{
  All tests must have the same number of degrees of freedom. If this is
  not the case, I suggest transforming to p-values and then plotting 
  -2log(p) as chi-squared on 2 df. 

  }
\references{Devlin, B. and Roeder, K. (1999) Genomic control for
  association studies. \emph{Biometrics}, \bold{55}:997-1004}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{single.snp.tests}}, \code{\link{snp.lhs.tests}},
  \code{\link{snp.rhs.tests}} }
\examples{
## See example the single.snp.tests() function
}
\keyword{hplot}

\name{glm.test.control}
\alias{glm.test.control}
\title{Set up control object for GLM computations}
\description{
  Several commands depend on fitting a generalized linear model (GLM),
  using the standard 
  iteratively reweighted least squares (IRLS) algorithm.  This
  function sets various control parameters for this.
}
\usage{
glm.test.control(maxit = 20, epsilon = 1.e-5, R2Max = 0.99)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{maxit}{Maximum number of IRLS steps}
  \item{epsilon}{Convergence threshold for IRLS algorithm}
  \item{R2Max}{R-squared limit for aliasing of new terms}
}
\details{
Sometimes (although not always), an iterative scheme is necessary to fit
a generalized linear model (GLM). The \code{maxit} parameter sets
the maximum number of iterations to be carried out, while the
\code{epsilon} parameter sets the criterion for determining
convergence. Variables which are judged to be "aliased" are dropped.
A variable is judged to be aliased if RSS/TSS is less than (1-R2Max),
where
\itemize{
  \item{RSS }{is the residual (weighted) sum of squares from the
    regression of that variable on the variables which precede it in the
    model formula (and any stratification defined in a strata()
    call in th emodel formula), and}
  \item{TSS }{is the total (weighted) sum of squared deviations of this
    variable from its mean (or, when a strata() call is present,
    from its stratum-specific  means).}
}
The weights used in this calculation
are the "working" weights of the IRLS algorithm.
}
\value{
  Returns the parameters as a list in the expected order
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{snp.lhs.tests}}, \code{\link{snp.rhs.tests}}}
\keyword{utilities}

\name{snp.rhs.estimates}
\alias{snp.rhs.estimates}
\title{Fit GLMs with SNP genotypes as independent variable(s)}
\description{
This function fits a generalized linear model with phenotype as
dependent variable and with a series of SNPs (or small sets of SNPs) as
predictor variables.  Optionally, one or more potential confounders of
a phenotype-genotype association may be included in the model. 
In order to protect against misspecification
of the variance function,  "robust" estimates of the variance-covariance
matrix of estimates may be calculated in place of the usual
model-based estimates.
}
\usage{
snp.rhs.estimates(formula, family = "binomial", link, weights, subset,
data = parent.frame(), snp.data,
   rules = NULL, sets = NULL, robust = FALSE, uncertain = FALSE, control
= glm.test.control())
}
\arguments{
  \item{formula}{The model formula, with phenotype as dependent variable
  and any potential confounders as independent variables. Note that
  parameter estimates are not returned for these model terms}
  \item{family}{A string defining the generalized linear model
    family. This currently should (partially)  match one of
    \code{"binomial"}, \code{"Poisson"}, \code{"Gaussian"} or
      \code{"gamma"} (case-insensitive)}
  \item{link}{A string defining the link function for the GLM. This
  currently should (partially) match one of \code{"logit"},
  \code{"log"}, \code{"identity"} or \code{"inverse"}. The
  default action is to use the "canonical" link for the family selected}
  \item{data}{The dataframe in which the model formula is to be interpreted}
  \item{snp.data}{An object of class \code{"SnpMatrix"} or
    \code{"XSnpMatrix"} containing the SNP data}
  \item{rules}{Optionally, an object of class \code{"ImputationRules"}}
  \item{sets}{Either a vector of SNP names (or  numbers) for the SNPs
    to be added to the model formula, or a logical vector of length
    equal to the number of columns in \code{snp.data}
    or a list of short vectors
    defining sets of SNPs to be included (see \code{Details})}
  \item{weights}{"Prior" weights in the generalized linear model}
  \item{subset}{Array defining the subset of rows of  \code{data} to use}
  \item{robust}{If \code{TRUE}, robust tests will be carried out}
  \item{uncertain}{If \code{TRUE}, uncertain genotypes are used and
    scored by their posterior expectations. Otherwise they are treated
    as missing}
  \item{control}{An object giving parameters for the IRLS algorithm
    fitting of the base model and for the acceptable aliasing amongst
    new terms to be tested. See \code{\link{glm.test.control}}}
}
\details{
  Homozygous SNP genotypes are coded 0
  or 2 and heterozygous genotypes are coded 1. For SNPs on the X
  chromosome, males are coded as homozygous females. For X SNPs, it will
  often be
  appropriate to include sex of subject in the base model (this is
  not done automatically).
  The "robust" option causes Huber-White estimates of the
  variance-covariance matrix of the parameter estimates to be
  returned. These protect against mis-specification of the variance
  function in the GLM, for example if binary or count data are
  overdispersed,
    
  If a \code{data} argument is supplied, the \code{snp.data} and
  \code{data} objects are aligned by rowname. Otherwise all variables in
  the model formulae are assumed to be stored in the same order as the
  columns of the \code{snp.data} object.

  Usually SNPs to be fitted in models will be referenced by name. However,
  they can
  also be referenced by number, indicating the
  appropriate column in the input \code{snp.data}.  They can also
  be referenced by a logical selection vector of length equal to the
  number of columns in \code{snp.data}.

  If the \code{rules} argument is supplied, SNPs may be imputed using
  these rules and included in the model.
}
\value{
  An object of class \code{\link[=GlmEstimates-class]{GlmEstimates}}
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\note{
  A factor (or
  several factors) may be included as arguments to the function
  \code{strata(...)} in the \code{formula}. This fits all
  interactions of the factors so included, but leads to faster
  computation  than fitting these in the normal way. Additionally, a
  \code{cluster(...)} call may be included in the base model
  formula. This identifies clusters of potentially correlated
  observations (e.g. for members of the same family); in this case, an
  appropriate robust estimate of the variance of the parameter
  estimates is used.

  If uncertain genotypes (e.g. as a result of imputation) are used, the
  interpretation of the regression coefficients is questionable; the
  regression coefficient for an imperfectly measurement of a variable is
  not a biased (attenuated) estimate of the coefficient of the variable
  measured.
}
\seealso{\code{\link{GlmEstimates-class}},
    \code{\link{snp.lhs.estimates}},
    \code{\link{snp.rhs.tests}}, 
    \code{\link{SnpMatrix-class}}, \code{\link{XSnpMatrix-class}}}
\examples{
data(testdata)
test <- snp.rhs.estimates(cc~strata(region), family="binomial",
   data=subject.data, snp.data= Autosomes, sets=1:10)
print(test)
test2 <- snp.rhs.estimates(cc~region+sex, family="binomial",
   data=subject.data, snp.data= Autosomes, sets=1:10)
print(test2)
test.robust <- snp.rhs.estimates(cc~strata(region), family="binomial",
   data=subject.data, snp.data= Autosomes, sets=1:10, robust=TRUE)
print(test.robust)
}
\keyword{htest}

\name{test.allele.switch}
\alias{test.allele.switch}
\title{Test for switch of alleles between two collections}
\description{
When testing genotype data derived from different platforms or scoring
algorithms a common problem is switching of alleles. This function
provides a diagnostic for this. Input can either be two objects of class
\code{"SnpMatrix"} to be examined, column by column, for allele
switching, or a single \code{"SnpMatrix"} object together with an
indicator vector giving group membership for its rows.
}
\usage{
test.allele.switch(snps, snps2 = NULL, split = NULL, prior.df = 1)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{snps}{An object of class \code{"SnpMatrix"} or \code{"XSnpMatrix"}}
  \item{snps2}{A second object of the same class as \code{snps}}
  \item{split}{If only one \code{SnpMatrix} object supplied, a vector
    with the same number of elements as rows of \code{snps}. It must be
    capable of coercion to a factor with two levels.
  }
  \item{prior.df}{A degree of freedom parameter for the prior
    distribution of the allele frequency \code{prior.df} (see Details)}
}
\details{
This function calculates a Bayes factor for the comparison of the
hypothesis that the alleles have been switched with the hypothesis that
they have not been switched. This requires integration over the
posterior distribution of the allele frequency. The prior is taken as a
beta distribution with both parameters equal to \code{prior.df}so that
the prior is symmetric about 0.5. The default,
\code{prior.df=1} represents a uniform prior on (0,1). 
}
\value{
A vector containing the log (base 10) of the Bayes Factors for an allele
switch. 
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{SnpMatrix-class}}, \code{\link{XSnpMatrix-class}}}
\examples{
data(testdata)
#
# Call with two SnpMatrix arguments
#
cc <- as.numeric(subject.data$cc)
lbf1 <- test.allele.switch(Autosomes[cc==1,], Autosomes[cc==2,])
#
# Single matrix call (giving the same result)
#
lbf2 <- test.allele.switch(Autosomes, split=cc)
}
\keyword{utilities}

\name{xxt}
\alias{xxt}
\title{X.X-transpose for a standardized SnpMatrix}
\description{
The input SnpMatrix is first standardized by subtracting the mean (or
stratum mean) from
each call  and
dividing by the expected standard deviation under Hardy-Weinberg
equilibrium.
It is then post-multiplied by its transpose. This is a
preliminary step in the computation of principal components.
}
\usage{
xxt(snps, strata = NULL, correct.for.missing = FALSE, lower.only = FALSE,
  uncertain = FALSE)
}
\arguments{
  \item{snps}{The input matrix, of type \code{"SnpMatrix"}}
  \item{strata}{A \code{factor} (or an object which can be coerced into
    a  \code{factor}) with length equal to the number of rows of
    \code{snps} defining stratum membership}
  \item{correct.for.missing}{If \code{TRUE}, an attempt is made to
    correct for the effect of missing data by use of inverse probability
    weights. Otherwise, missing observations are scored zero in the
    standardized matrix}
  \item{lower.only}{If \code{TRUE}, only the lower triangle of the
    result is returned and the upper triangle is filled with
    zeros. Otherwise, the complete symmetric matrix is returned}
  \item{uncertain}{If \code{TRUE}, uncertain genotypes are replaced by
    posterior expectations. Otherwise these are treated as missing
    values}
}
\details{
  This computation forms the first step of the calculation of principal
  components for genome-wide SNP data. As pointed out by Price et al.
  (2006), when the data matrix has more rows than columns it is
  most efficient to calculate the eigenvectors of
  \var{X}.\var{X}-transpose, where \var{X} is a 
  \code{SnpMatrix} whose columns have been  standardized to zero mean and
  unit variance. For autosomes, the genotypes are given codes 0, 1 or 2
  after subtraction of the mean, 2\var{p}, are divided by the standard
  deviation 
  sqrt(2\var{p}(1-\var{p})) (\var{p} is the estimated allele
  frequency). For SNPs on the X chromosome in male subjects,
  genotypes are coded 0 or 2. Then 
  the mean is still 2\var{p}, but the standard deviation is 
  2sqrt(\var{p}(1-\var{p})). If the \code{strata} is supplied, a
  stratum-specific estimate value for \var{p} is used for
  standardization. 
  
  Missing observations present some difficulty. Price et al. (2006)
  recommended replacing missing observations by their means, this being
  equivalent to replacement by zeros in the standardized matrix. However
  this results in a biased estimate of the complete data
  result. Optionally this bias can be corrected by inverse probability
  weighting. We assume that the probability that any one call is missing
  is small, and can be predicted by a multiplicative model with row
  (subject) and column (locus) effects. The estimated probability of a
  missing value in a given row and column is then given by
  \eqn{m = RC/T}, where \var{R} is the row total number of
  no-calls, \var{C} is the column total of no-calls, and \var{T} is the
  overall total number of no-calls. Non-missing contributions to
  \var{X}.\var{X}-transpose are then weighted by \eqn{w=1/(1-m)} for
  contributions to the diagonal elements, and products of the relevant
  pairs of weights for contributions to off--diagonal elements.
}
\value{
A square matrix containing either the complete X.X-transpose matrix, or
just its lower triangle
}
\references{
Price et al. (2006) Principal components analysis corrects for
stratification in genome-wide association studies. \emph{Nature Genetics},
\bold{38}:904-9
}
\note{
In genome-wide studies, the SNP data will usually be held as a series of
objects (of
class \code{"SnpMatrix"} or\code{"XSnpMatrix"}), one per chromosome.
Note that the  \var{X}.\var{X}-transpose matrices
produced by applying the \code{xxt} function to each object in turn
can be added to yield the genome-wide result.

If the matrix is converted to a correlation matrix by pre- and
post-multiplying by the sqrt of the inverse of its diagonal, then this
is an unbiased estimate of twice the kinship matrix.
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\section{Warning}{
  The correction for missing observations can result in an output matrix
  which is not positive semi-definite. This should not matter in the
  application for which it is intended
  }
\examples{
# make a SnpMatrix with a small number of rows
data(testdata)
small <- Autosomes[1:100,]
# Calculate the X.X-transpose matrix
xx <- xxt(small, correct.for.missing=TRUE)
# Calculate the principal components
pc <- eigen(xx, symmetric=TRUE)$vectors
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{array}
\keyword{multivariate}

\name{snp.cor}
\alias{snp.cor}
\title{Correlations with columns of a SnpMatrix}
\description{
This function calculates Pearson correlation coefficients  between columns of a
\code{SnpMatrix} and columns of an ordinary matrix. The two matrices
must have the same number of rows. All valid pairs are used in the
computation of each correlation coefficient.
} 
\usage{
snp.cor(x, y, uncertain = FALSE)
}
\arguments{
  \item{x}{An \var{N} by \var{M} \code{SnpMatrix}}
  \item{y}{An \var{N} by \var{P} general matrix}
  \item{uncertain}{If \code{TRUE}, uncertain genotypes are replaced by
    posterior expectations. Otherwise these are treated as missing
    values}
}
\details{
  This can be used together with \code{\link{xxt}} and
  \code{\link[base]{eigen}} to calculate standardized loadings in the principal
  components
}
\value{
An  \var{M} by \var{P} matrix of correlation coefficients
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\note{
  This version cannot handle X chromosomes
}
\seealso{\code{\link{xxt}}}
\examples{
# make a SnpMatrix with a small number of rows
data(testdata)
small <- Autosomes[1:100,]
# Calculate the X.X-transpose matrix
xx <- xxt(small, correct.for.missing=TRUE)
# Calculate the principal components
pc <- eigen(xx, symmetric=TRUE)$vectors
# Calculate the loadings in first 10 components */
loadings <- snp.cor(small, pc[,1:10])
}
\keyword{array}
\keyword{multivariate}

\name{sample.ped.gz}
\docType{data}
\alias{sample.info}
\alias{sample.bed}
\alias{sample.bim}
\alias{sample.fam}
\alias{sample-long.gz}
\alias{sample-long-alleles.gz}
\alias{mach1.out.mlprob.gz}
\title{Sample datasets to illustrate data input}
\description{
  The first five files concern data on 20 diallelic loci on 120
  subjects. These data are 
  distributed with the Haploview package (Barrett et al., 2003). The
  sixth files contains a additional dataset of 18 SNPs in
  100 subjects, coded in "long" format, and the seventh file duplicates
  this dataset in an alternative long format. These seven files are used
  in the data input
  vignette. The final file is a sample imputed genotype
  dataset distributed with the MACH imputation package, and used in the
  imputation vignette.

  These files are stored in the \code{extdata} relative to the
  package base. Full file names can be obtained using the
  \code{system.file} function.
}
\format{
  The following files are described here:
  \itemize{
    \item \code{sample.ped.gz}: A gzipped pedfile
    \item \code{sample.info}: An accompanying locus information file
    \item \code{sample.bed}: The corresponding PLINK \code{.bed} file
    \item \code{sample.bim}: The PLINK \code{.bim} file
    \item \code{sample.fam}: The PLINK \code{.fam} file
    \item \code{sample-long.gz}: A sample of long-formatted data
    \item \code{sample-long-alleles.gz}: The same as above, but allele-coded
    \item \code{mach1.out.mlprob.gz}: An \code{mlprob} output file from
    the MACH genotype imputation program. This file contains, for each
    imputed genotype call, posterior probabilities for the three possible
    genotypes
  }
}
\source{
  \url{http://www.broadinstitute.org/scientific-community/science/programs/medical-and-population-genetics/haploview/downloads}
  \url{http://www.sph.umich.edu/csg/abecasis/MACH/download}
}
\references{
Barrett JC, Fry B, Maller J, Daly MJ.(2005)  Haploview: analysis
and visualization of LD and haplotype maps. \emph{Bioinformatics}, 2005 Jan 15,
[PubMed ID: 15297300]
}
\keyword{datasets}

\name{chi.squared}
\alias{chi.squared}
\alias{deg.freedom}
\alias{effect.sign}
\alias{p.value}
\alias{sample.size}
\alias{effective.sample.size}
\title{Extract test statistics and p-values}
\description{
Generic functions to extract values from the SNP
association test objects returned by various testing functions
}
\usage{
chi.squared(x, df)
deg.freedom(x)
effect.sign(x, simplify)
p.value(x, df)
sample.size(x)
effective.sample.size(x)
}
\arguments{
  \item{x}{An object of class \code{"SingleSnpTests"}, 
     \code{"SingleSnpTestsScore"}, or \code{"GlmTests"}}
   \item{df}{Either the numeric value 1 or 2 (not used when \code{x} is
     of class \code{"GlmTests"})}
   \item{simplify}{This switch is relevant when \code{x} is of class
     \code{"GlmTests"} and plays the same role as it does in
     \code{\link{sapply}}. If \code{simplify=TRUE}, where possible
     the output is returned as a simple numeric vector rather than as a
     list}
}
\details{
  These functions operate on objects created by 
  \code{\link{single.snp.tests}}, \code{\link{snp.lhs.tests}}, and
  \code{\link{snp.lhs.tests}}.

  The functions \code{chi.squared} and \code{p.value} return the
  chi-squared statistic and the corresponding \emph{p}-value.
  The argument \code{df} is only used for
  output from \code{single.snp.tests}, since this function calculates
  both 1 df and 2 df tests for each SNP. The functions
  \code{snp.lhs.tests} and \code{snp.rhs.tests} potentially calculate
  chi-squared tests on varying degrees of freedom, which can be
  extracted with \code{deg.freedom}. The function \code{effect.sign}
  indicates the direction of 
  associations. When applied to an output object from
  \code{snp.single.tests}, it returns \code{+1} if the association, as
  measured by the 1 df test, is positive and \code{-1} if the
  association is negative. Each test calculated by \code{GlmTests}
  are potentially tests of several parameters so that the effect sign
  can be a vector. Thus \code{effect.sign} returns a list of sign
  vectors unless, if \code{simplify=TRUE}, and it can be simplified as a
  single vector with one sign for each test.
  The function \code{sample.size} returns the number of observations
  actually used in the test, after exclusions due to missing data have
  been applied, and \code{effective.sample.size} returns the effective
  sample size which is less than the true sample size for tests on
  imperfectly imputed SNPs.
}
\value{
  A numeric vector containing the
  chi-squared test statistics or p-values. The output vector has a \code{names}
  attribute.
}
\note{The \code{df} and \code{simplify} arguments are not always
  required (or legal). See above}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{single.snp.tests}}, 
  \code{\link{snp.lhs.tests}}, \code{\link{snp.rhs.tests}},
  \code{\link{SingleSnpTests-class}},
  \code{\link{SingleSnpTestsScore-class}},
  \code{\link{GlmTests-class}}
}
\examples{
data(testdata)
tests <- single.snp.tests(cc, stratum=region, data=subject.data,
   snp.data=Autosomes, snp.subset=1:10)
chi.squared(tests, 1)
p.value(tests, 1)
}
\keyword{utilities}

\name{read.mach}
\alias{read.mach}
\title{
Read genotypes imputed by the MACH program
}
\description{
This routine reads imputed genotypes generated by the MACH program. With
the \code{--mle} and \code{--mldetails} options in force this program
generates a
\code{.mlprob} output file which contains probabilities of
assignments. These are stored as uncertain genotype calls in a
\code{SnpMatrix} object
}
\usage{
read.mach(file, colnames = NULL, nrow = NULL)
}
\arguments{
  \item{file}{The name of the \code{.mlprob} file. This may be gzipped
  }
  \item{colnames}{The column names. If absent, names are generated as
    \code{SNP1}, \code{SNP2}, etc.
  }
  \item{nrow}{If known the number of rows of data on the file. If not
    supplied, it is determined by a preliminary pass through the data
  }
}
\details{
No routine is explicitly available for data on chromosome X. Such data
should first be read as a \code{SnpMatrix} and then coerced to an
\code{XSnpMatrix} object
}
\value{
  An object of class \code{SnpMatrix}
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{SnpMatrix-class}}}
\examples{
##---- No example available yet
}
\keyword{IO}
\keyword{file}

\name{families}
\alias{families}
\alias{genotypes}
\alias{pedData}
\docType{data}
\title{Test data for family association tests}
\description{
  These data started life as real data derived from an affected sibling
  pair study of type 1 diabetes. However, original subject and SNP
  identidiers have been replaced by randomly chosen ones.
}
\usage{data(families)}
\format{
  There are two objects in the loaded data file:
  \itemize{
    \item \code{genotypes}: An object of class
    \code{"\link[=SnpMatrix-class]{snp.matrix}"} containing the SNP
    genotype data for both parents and affected offspring
    \item \code{pedData}: A data frame containing the standard six fields
    for a \emph{LINKAGE} pedfile. The are named \code{familyid},
    \code{member}, \code{father}, \code{mother} \code{sex}, and
    \code{affected}
  }
  The two objects are linked by common row names.
}
\details{
Coding in the \code{pedData} frame is as in the \emph{LINKAGE} package,
except that missing data are coded \code{NA} rather than zero
}
\examples{
data(families)
summary(genotypes)
summary(pedData)
}
\keyword{datasets}

\name{snp.lhs.estimates}
\alias{snp.lhs.estimates}
\title{Logistic regression with SNP genotypes as dependent variable}
\description{
  Under the assumption of Hardy-Weinberg equilibrium, a SNP genotype is
  a binomial variate with two trials for an autosomal SNP or with one or
  two trials (depending on sex) for a SNP on the X chromosome.
  With each SNP in an input
  \code{"SnpMatrix"} as dependent variable, this function fits a
  logistic regression model. The Hardy-Weinberg
  assumption can be relaxed by use of a "robust" option.
}
\usage{
snp.lhs.estimates(snp.data, base.formula, add.formula, subset, snp.subset,
                data = sys.parent(), robust = FALSE, uncertain = FALSE,
                control=glm.test.control())
}
\arguments{
  \item{snp.data}{The SNP data, as an object of class
    \code{"SnpMatrix"} or \code{"XSnpMatrix"} }
  \item{base.formula}{A \code{formula} object describing a base model
    containing those terms which are to be fitted but for which
    parameter estimates are not required
    (the dependent variable is omitted from the model formula) }
  \item{add.formula}{A \code{formula} object describing the additional
    terms in the model for which parameter estimates are required (again,
    the dependent variable is omitted)}
  \item{subset}{An array describing the subset of observations to be
    considered} 
  \item{snp.subset}{An array describing the subset of SNPs to be
    considered. Default action is to test all SNPs.} 
  \item{data}{The data frame in which \code{base.formula},
    \code{add.formula} and \code{subset} are to be evaluated}
  \item{robust}{If \code{TRUE}, Hardy-Weinberg equilibrium will is not
    assumed in calculating the variance-covariance matrix of parameter
    estimates}
  \item{uncertain}{If \code{TRUE}, uncertain genotypes are used and
    scored by their posterior expectations. Otherwise they are treated
    as missing. If set, this option forces \code{robust} variance estimates}
  \item{control}{An object giving parameters for the IRLS algorithm
    fitting of the base model and for the acceptable aliasing amongst
    new terms to be tested. See \code{\link{glm.test.control}}}
}
\details{
  The model fitted is the union of the \code{base.formula} and
  \code{add.formula} models, although parameter estimates (and their
  variance-covariance matrix) are only
  generated for the parameters of the latter.
  The "robust" option causes a Huber-White "sandwich" estimate of the
  variance-covariance matrix to be used in place of the usual inverse
  second derivative matrix of the log-likelihood (which assumes
  Hardy-Weinberg equilibrium).
  If a \code{data} argument is supplied, the \code{snp.data} and
  \code{data} objects are aligned by rowname. Otherwise all variables in
  the model formulae are assumed to be stored in the same order as the
  columns of the \code{snp.data} object. 
}
\value{
  An object of class \code{\link[=GlmEstimates-class]{GlmEstimates}}
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\note{
  A factor (or
  several factors) may be included as arguments to the function
  \code{strata(...)} in the \code{base.formula}. This fits all
  interactions of the factors so included, but leads to faster
  computation  than fitting these in the normal way. Additionally, a
  \code{cluster(...)} call may be included in the base model
  formula. This identifies clusters of potentially correlated
  observations (e.g. for members of the same family); in this case, an
  appropriate robust estimate of the variance-covariance matrix of
  parameter estimates is calculated. No more than one
  \code{strata()} call may be used, and neither \code{strata(...)}  or
  \code{cluster(...)} calls may appear in the \code{add.formula}.

  If uncertain genotypes (e.g. as a result of imputation) are used, the
  interpretation of the regression coefficients is questionable.

  A known bug is that the function fails when no \code{data} argument is
  supplied and the base model formula contains no variables
  (\code{~1}). A work-round is to create a data frame to hold the
  variables in the models and pass this as \code{data=}. 
}
\seealso{\code{\link{GlmEstimates-class}}, \code{\link{snp.lhs.tests}}}
\examples{
data(testdata)
test1 <-
snp.lhs.estimates(Autosomes[,1:10], ~cc, ~region, data=subject.data)
test2 <-
snp.lhs.estimates(Autosomes[,1:10], ~strata(region), ~cc,
   data=subject.data)
test3 <-
snp.lhs.estimates(Autosomes[,1:10], ~cc, ~region, data=subject.data, robust=TRUE)
test4 <-
snp.lhs.estimates(Autosomes[,1:10], ~strata(region), ~cc,
   data=subject.data, robust=TRUE)
test5 <- snp.lhs.estimates(Autosomes[,1:10], ~region+sex, ~cc, data=subject.data, robust=TRUE)
print(test1)
print(test2)
print(test3)
print(test4)
print(test5)
}
\keyword{htest}

\name{write.SnpMatrix}
\alias{write.SnpMatrix}
\title{Write a SnpMatrix object as a text file}
\description{
  This function is closely modelled on \code{write.table}. It writes an
  object of class \code{SnpMatrix} as a text file with one line for
  each row of the matrix. Genotpyes are written in numerical form,
  \emph{i.e.} as 0, 1 or 2 (where 1 denotes heterozygous) or,
  optionally, as a pair of alleles (each coded 1 or 2).
}
\usage{
write.SnpMatrix(x, file, as.alleles= FALSE, append = FALSE, quote = TRUE, sep = " ", eol = "\n", na = "NA", row.names = TRUE, col.names = TRUE)
}
\arguments{
  \item{x}{The object to be written }
  \item{file}{The name of the output file}
  \item{as.alleles}{If \code{TRUE}, write each genotype as two alleles}
  \item{append}{If \code{TRUE}, the output is appended to the designated
  file. Otherwise a new file is opened}
  \item{quote}{If \code{TRUE}, row and column names will be enclosed in quotes}
  \item{sep}{The string separating entries within a line}
  \item{eol}{The string terminating each line}
  \item{na}{The string written for missing genotypes}
  \item{row.names}{If \code{TRUE}, each row will commence with the row name}
  \item{col.names}{If \code{TRUE}, the first line will contain all the
    column names}
}
\value{
  A numeric vector giving the dimensions of the matrix written
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link[utils]{write.table}}, \code{\link{SnpMatrix-class}},
  \code{\link{XSnpMatrix-class}}}
\keyword{manip}
\keyword{IO}
\keyword{file}
\keyword{utilities}

\name{mean2g}
\alias{mean2g}
\alias{post2g}
\alias{g2post}
\title{
Raw coding of posterior probabilities of SNP genotype
}
\description{
An uncertain SNP genotype call is represented by the three posterior
probabilities of the three possible calls. In the class
\code{SnpMatrix}, an approximation to these is packed into a single
1-byte variable of type \code{raw}. These functions carry out this coding (and
decoding). 
}
\usage{
post2g(p, transpose = FALSE)
mean2g(m, maxE = FALSE)
g2post(g, transpose = FALSE)
}
\arguments{
  \item{p}{
    A matrix of posterior probabilities. If \code{transpose} is
    \code{FALSE} this is \code{Nx3}, otherwise \code{3xN} 
  }
  \item{m}{
    A vector of posterior means
  }
  \item{g}{
    A \code{raw} vector of genotype codes
  }
  \item{transpose}{
    A logical flag indicating transposition of the matric of posterior
    probabilities (see Description)
  }
  \item{maxE}{
    A logical flag selecting the maximum entropy option in
    \code{mean2g}
  }
}
\details{
\code{post2g} and \code{g2post} convert from posterior probabilities to
\code{raw} code and back respectively. If only the posterior expectation
of the genotype (when numerically coded 0, 1, or 2) is available, no
unique soultion exists in general and the behaviour of the function is
determined by the value of \code{maxE}. If \code{TRUE}, then the maximum
entropy solutions are returned while, if \code{FALSE}, an attempt is made
to return the least uncertain solution, by setting the posterior
probability of the \code{BB} genotype to zero when the posterior mean is
less than 1.0 and the probability of \code{AA} to zero when the mean is
greater than 1.0.
}
\value{
\code{post2g} and \code{mean2g} return a vector of type
\code{raw}. \code{g2post} returns a numeric matrix. 
}
\author{
David Clayton \email{dc208@cam.ac.uk}
}
\note{
These functions are provided mainly for users wishing to write their own
data input functions.
}
\examples{
data(testdata)
g <- Autosomes[1:10, 20] ## A vector of codes
p <- g2post(g)           ## Transform to probabilities ...
pg <- post2g(p)          ## ... and back to codes
m <- p[,2]+2*p[,3]       ## Posterior expectations
mg <- mean2g(m)          ## Transform to codes ...
pmg <- g2post(mg)        ## ... and transform to probabilities
## Write everything out
print(cbind(as(g, "numeric"), p, as.numeric(pg), m, as.numeric(mg), pmg))
}
\keyword{programming}

\name{filter.rules}
\alias{filter.rules}
\title{Filter a set of imputation rules}
\description{
Determine which imputation rules are broken by removal of some SNPs from
a study. This function is needed because, when if it emerges that
genotyping of some SNPs is not reliable, necessitating their removal from
study, we would also wish to remove any SNPs imputed on the basis of
these unreliable SNPs.
}
\usage{
filter.rules(rules, snps.excluded, exclusions = TRUE)
}
\arguments{
  \item{rules}{An object of class \code{"ImputationRules"} containing
    a set of imputation rules}
  \item{snps.excluded}{The names of the SNPs whose
    removal is to be investigated}
  \item{exclusions}{If \code{TRUE}, the names of the imputed SNPs which
    would be lost by removal of the SNPs listed in
    \code{snps.excluded}. If \code{FALSE}, the names of the imputed SNPs
  which would \emph{not} be lost are returned}
}
\value{
  A character vector containing the names of imputed SNPs to be removed
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{ImputationRules-class}},
    \code{\link{snp.imputation}}
}
\examples{
# No example yet
}
\keyword{models}
\keyword{regression}

\name{sm.compare}
\alias{sm.compare}
\title{
Compare two SnpMatrix objects
}
\description{
For quality control purposes, it is sometimes necessary to compare
genotype data derived from different sources. This function facilitates
this. 
}
\usage{
sm.compare(obj1, obj2, row.wise = TRUE, col.wise = TRUE)
}
\arguments{
  \item{obj1}{
    The first of the two \code{SnpMatrix} objects to be compared
  }
  \item{obj2}{
    The second \code{SnpMatrix} object
  }
  \item{row.wise}{
    Calculate comparison statistics aggregated in a row-wise manner
  }
  \item{col.wise}{
    Calculate column-wise comparison statistics
  }
}
\details{
  Initially row and column names of the two objects are compared to
  identify subsets of subjects and SNPs which they have in
  common. Then, every instance of a SNP genotype in the two objects are
  compared and agreements and disagreements counted by row and/or by
  column. 
}
\value{
  If only one of the row-wise and column-wise summaries are to be
  calculated, the return value is a matrix with rows defined by subjects
  or SNPs and columns giving counts of:
  \item{Agree}{Agreements (all)}
  \item{Disagree}{Disgreements (all)}
  \item{NA.agree}{Genotype coded \code{NA} in both objects}
  \item{NA.disagree}{Genotype coded \code{NA} in only one object}
  \item{Hom.agree}{Objects agree and genotype is homozygous}
  \item{Hom.switch}{Genotype coded as homozygous in both objects,
    but alleles switched}
  \item{Het.agree}{Genotype coded as heterozygous in both objects}
  \item{Het.Hom}{Genotype coded as heterozygous in one object and
    homozygous in the other}

  
  If both row-wise and column-wise summaries are computed (the default
  behaviour) , the function
  returns a list containing two matrices of the form described
  above. These are named \code{row.wise} and \code{col.wise}
}
\author{
David Clayton \email{dc208@cam.ac.uk}
}
\note{
  No special provision is yet made for objects of class
  \code{XSnpMatrix}, in which haploid calls are coded as homozygous.
}
\seealso{
  \code{\link{SnpMatrix-class}}, \code{\link{XSnpMatrix-class}}
}
\examples{
##
## No example yet available
##
}
\keyword{utilities}

\name{SingleSnpTests-class}
\docType{class}
\alias{SingleSnpTests-class}
\alias{[,SingleSnpTests,ANY,missing,missing-method}
\alias{chi.squared,SingleSnpTests,numeric-method}
\alias{names,SingleSnpTests-method}
\alias{p.value,SingleSnpTests,numeric-method}
\alias{sample.size,SingleSnpTests-method}
\alias{effective.sample.size,SingleSnpTests-method}
\alias{show,SingleSnpTests-method}
\alias{summary,SingleSnpTests-method}
\alias{coerce,SingleSnpTests,data.frame-method}
\alias{SingleSnpTestsScore-class}
\alias{[,SingleSnpTestsScore,ANY,missing,missing-method}
\alias{effect.sign,SingleSnpTestsScore,missing-method}
\alias{pool2,SingleSnpTestsScore,SingleSnpTestsScore,logical-method}
\alias{switch.alleles,SingleSnpTestsScore,ANY-method}


\title{Classes "SingleSnpTests" and "SingleSnpTestsScore"}
\description{These are classes to hold the objects created by
\code{\link{single.snp.tests}} and provide methods for extracting key
elements. The class \code{"SingleSnpTestsScore"} extends class
\code{"SingleSnpTests"}
to  include the score and score variance statistics in order to
provide methods for pooling results from several studies or parts of a study
}
\section{Objects from the Class}{
  Objects can be created by calls of the form
  \code{new("SingleSnpTests", ...)} and
  \code{new("SingleSnpTestsScore", ...)} but, more usually, will be created
  by calls to \code{\link{single.snp.tests}}
}
\section{Slots}{
  \describe{
    \item{\code{snp.names}:}{The names of the SNPs tested, as they
      appear as column names in the original \code{SnpMatrix}}
    \item{\code{chisq}:}{A two-column
      matrix holding the 1 and 2 df association tests}
    \item{\code{N}:}{The numbers of
      observations included in each test}
    \item{\code{N.r2}:}{For tests on imputed SNPs, the product of \code{N} and
	the imputation \eqn{r^2}. Otherwise a zero-length object}
    \item{\code{U}:}{(class \code{"SingleSnpTestsScore"}) Score statistics}
    \item{\code{V}:}{(class \code{"SingleSnpTestsScore"}) Score variances}
  }
}
\section{Methods}{
  \describe{
    \item{[]}{\code{signature(x =  "SingleSnpTests", i = "ANY")}:
      Subsetting operator}
    \item{[]}{\code{signature(x =  "SingleSnpTestsScore", i = "ANY")}:
      Subsetting operator}
    \item{chi.squared}{\code{signature(x = "SingleSnpTests", df =
	"numeric")}:
      Extract 1- and 2-df chi-squared test values}
    \item{effect.sign}{\code{signature(x = "SingleSnpTestsScore", simplify =
	"missing")}: Extract signs of associations tested by the 1df tests}
     \item{names}{\code{signature(x="SingleSnpTests")}: Extract names of
      test values (\code{snp.names} slot)}
    \item{p.value}{\code{signature(x = "SingleSnpTests", df =
	"numeric")}:
      Evaluate 1- and 2-df test p-values}
    \item{show}{\code{signature(object = "SingleSnpTests")}:
      List all tests and p-values}
    \item{coerce}{\code{signature(from = "SingleSnpTests", to = "data.frame")}:
      Conversion to data frame class}
    \item{sample.size}{\code{signature(object = "SingleSnpTests")}:
      Extract sample sizes for tests}
    \item{effective.sample.size}{\code{signature(object = "SingleSnpTests")}:
      Extract effective sample sizes for tests. For imputed tests, these
      are the real sample sizes multiplied by the corresponding
      R-squared values for imputation}
    \item{summary}{\code{signature(object = "SingleSnpTests")}:
      Summarize all tests and p-values}
    \item{pool2}{\code{signature(x = "SingleSnpTestsScore",
	y = "SingleSnpTestsScore", score = "logical")}: Combine two
      sets of test results. Used recursively by {\code{\link{pool}}}}
    \item{switch.alleles}{\code{signature(x = "SingleSnpTestsScore", snps =
	"ANY")}: Emulate, in the score vector and its (co)variances,
      the effect of switching of the alleles for the specified tests}
  }
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{
\code{\link{single.snp.tests}}, \code{\link{pool}}
}
\examples{
showClass("SingleSnpTests")
showClass("SingleSnpTestsScore")
}
\keyword{classes}

\name{ibsCount}
\alias{ibsCount}
\title{Count alleles identical by state}
\description{
  This function counts, for all pairs of subjects and across all SNPs,
  the total number of alleles which are identical by state (IBS)
}
\usage{
ibsCount(snps, uncertain = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{snps}{An input object of class \code{"SnpMatrix"} or
    \code{"XSnpMatrix"}}
  \item{uncertain}{If \code{FALSE}, uncertain genotypes are
    ignored. Otherwise contributions are weighted by posterior probabilities}
}
\details{
For each pair of subjects the function counts the total number of
alleles which are IBS. For autosomal SNPs, each locus
contributes 4 comparisons, since each subject carries two
copies. For SNPs on the X chromosome, the number of comparisons is also
4 for female:female comparisons, but is 2 for female:male 
and 1 for male:male comparisons.
}
\value{
  If there are \var{N} rows in the input matrix, the function returns an
  \var{N}*\var{N} matrix. The lower triangle contains the total number
  of comparisons and the upper triangle contains the number of these
  which  are IBS. The diagonal contains the number of valid calls for
  each subject. 
}
\note{
In genome-wide studies, the SNP data will usually be held as a series of
objects (of
class \code{"SnpMatrix"} or \code{"XSnpMatrix"}), one 
per chromosome. Note that the matrices
produced by applying the \code{ibsCount} function to each object in turn
can be added to yield the genome-wide result.
}
\author{David Clayton \email{dc208@cam.ac.uk} }
\seealso{\code{\link{ibsDist}} which calculates a distance matrix based
  on proportion of alleles which are IBS}
\examples{
data(testdata)
% taking a subset of Autosomes - it's a bit big for testing basic functionality
ibs.A <- ibsCount(Autosomes[,1:100])
ibs.X <- ibsCount(Xchromosome)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{cluster}

\name{tdt.snp}
\alias{tdt.snp}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{1-df and 2-df tests for genetic associations with SNPs (or
  imputed SNPs) in family data}
\description{
Given large-scale SNP data for families comprising both parents and one or 
more affected offspring, this function computes 1 df tests (the TDT test) and 
a 2 df test based on observed and expected transmissions of genotypes. 
Tests based on imputation rules can also be carried out.
}
\usage{
tdt.snp(ped, id, father, mother, affected, data = sys.parent(), snp.data, 
    rules = NULL, snp.subset, check.inheritance = TRUE, robust = FALSE,
    uncertain = FALSE, score = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{ped}{Pedigree identifiers}
  \item{id}{Subject identifiers}
  \item{father}{Identifiers for subjects' fathers}
  \item{mother}{Identifiers for subjects' mothers}
  \item{affected}{Disease status (TRUE if affected, FALSE otherwise)}
  \item{data}{A data frame in which to evaluate the previous five arguments}
  \item{snp.data}{An object of class \code{"SnpMatrix"} containing the SNP
    genotypes to be tested}
  \item{rules}{An object of class
    \code{"ImputationRules"}. If
    supplied, the rules coded in this object are used, together with
    \code{snp.data}, to calculate tests for imputed SNPs}
  \item{snp.subset}{A vector describing the subset of SNPs to be
    considered. Default action is to test all SNPs in \code{snp.data}
    or, in imputation mode,  as specified by \code{rules}}
  \item{check.inheritance}{If TRUE, each affected offspring/parent trio 
    is tested for Mendelian inheritance and excluded if the test fails. If
    FALSE, misinheriting trios are used but the "robust" variance option is
    forced}
  \item{robust}{If TRUE, forces the robust (Huber-White) variance option 
    (with \code{ped} determining independent "clusters")}
  \item{uncertain}{If TRUE, uncertain genotypes are handed by replacing
    score contributions by their posterior expectations. Otherwise these
    are treated as missing. Setting this option authomatically invokes
    use of \code{robust} variance estimates}
  \item{score}{If \code{TRUE}, the output object will contain, for each
    SNP,  the score vector and its variance-covariance matrix}
}
\details{
  Formally, the test statistics are score tests for the "conditioning on
  parental genotype" (CPG) likelihood. Parametrization of associations is
  the same as for the population-based tests calculated by 
  \code{\link{single.snp.tests}} so that results from family-based and 
  population-based studies can be combined using \code{\link{pool}}.

  When the function is used to calculate tests for imputed SNPs, the
  test is still an approximate score test. The current version does not use 
  the family relationships in the imputation. With this option, the robust 
  variance estimate is forced. 
  
  The first five arguments are usually derived from  a "pedfile". If a 
  data frame is supplied for the \code{data} argument, the first five 
  arguments will be evaluated in this frame. Otherwise they will be evaluated 
  in the calling environment. If the arguments are missing, they will be 
  assumed to be in their usual positions in the pedfile data frame 
  i.e. in columns one to 
  four for the identifiers  and column six for disease status 
  (with affected coded \code{2}). If the pedfile data are obtained from
  a dataframe, the row names of the \code{data} and \code{snp.data}
  files will be used to align the pedfile and SNP data. Otherwise, these
  vectors will be assumed to be in the same order as the rows of
  \code{snp.data}. 

  The \code{snp.subset} argument can be a logical,
  integer, or character vector.

  If imputed rather than observed SNPs are tested, or
  if \code{check.inheritance} is set to
  \code{FALSE}, the "robust" variance estimate is used regardless of the
  value supplied for the \code{robust} argument.
}
\value{
  An object of class
  \code{"SingleSnpTests"}. 
  If \code{score=TRUE}, the output object will be of the extended class
  \code{"SingleSnpTestsScore"}
  containing additional slots holding the score statistics and their
  variances (and covariances). This allows meta-analysis using the
  \code{\link{pool}} function.
}
\references{
  Clayton (2008) Testing for association on the X chromosome
  \emph{Biostatistics}, \bold{9}:593-600.)
}
\note{When the snps are on the X chromosome (i.e. when the \code{snp.data} 
  argument is of class \code{"XSnpMatrix"}), the tests are constructed 
  in the same way as was described by Clayton (2008) for population-based 
  association tests i.e. assuming that 
  genotype relative risks for males mirror thos of homozygous females
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{single.snp.tests}}, \code{\link{impute.snps}}, 
  \code{\link{pool}}, \code{\link{ImputationRules-class}},
  \code{\link{SingleSnpTests-class}},
  \code{\link{SingleSnpTestsScore-class}} 
}
\examples{
data(families)
tdt.snp(data=pedData, snp.data=genotypes)
}
\keyword{htest}

\name{snp.imputation}
\alias{snp.imputation}
\title{Calculate imputation rules}
\description{
  Given two set of SNPs typed in the same subjects, this function
  calculates rules which can be used to impute one set
  from the other in a subsequent sample. The function can also calculate
  rules for imputing each SNP in a single dataset from other SNPs in the
  same dataset
}
\usage{
snp.imputation(X, Y, pos.X, pos.Y, phase=FALSE, try=50, stopping=c(0.95, 4, 0.05),
               use.hap=c(1.0, 0.0), em.cntrl=c(50,0.01,10,0.01), minA=5)
}
\arguments{
  \item{X}{An object of class \code{"SnpMatrix"} or
    \code{"XSnpMatrix"} containing observations
    of the SNPs to be used for imputation ("predictor SNPs")}
  \item{Y}{An object of same class as \code{X} containing observations
  of the SNPs to be imputed in a future sample ("target SNPs"). If this
  argument is missing, then target SNPs are also drawn from \code{X}}
  \item{pos.X}{The positions of the  predictor SNPs. Can be missing if
    there is no \code{Y} argument and the columns of \code{X} are in
    genome position order}
  \item{pos.Y}{The positions of the target SNPs. Only required when
    a \code{Y} argument is present}
  \item{phase}{See "Details" below}
  \item{try}{The number of potential predictor SNPs to be
    considered in the stepwise regression procedure around each target
    SNP . The nearest \code{try} predictor SNPs to each target SNP
    will be considered}
  \item{stopping}{Parameters of the stopping rule for the stepwise
    regression (see below)}
  \item{use.hap}{Parameters to control use of the haplotype imputation
    method (see below)}
  \item{em.cntrl}{Parameters to control test for convergence of EM
    algorithm for fitting phased haplotypes (see below)}
  \item{minA}{A minimum data quantity measure for estimating pairwise
    linkage disequilibrium (see below)}
}
\details{
  The routine first carries out a series of step-wise least-square
  regression analyses in
  which each Y SNP is regressed on the nearest \code{try} predictor (X)
  SNPs. If
  \code{phase} is \code{TRUE}, the regressions will be calculated at the
  chromosome (haplotype) level, variances being simply  \eqn{p(1-p)} and
  covariances estimated from the estimated two-locus haplotypes (this option is
  not yet implemented). Otherwise, the
  analysis is carried out at the genotype level based on  
  conventional variance and covariance estimates using the
  \code{"pairwise.complete.obs"} missing value treatment
  (see \code{\link{cov}}). New
  SNPs are added to the regression until either (a) the value of
  \eqn{R^2} exceeds the first parameter of \code{stopping}, (b) the
  number of "tag" SNPs has reached the maximum set in the second parameter of
  \code{stopping}, or (c) the change in \eqn{R^2} does not achieve the
  target set by the third parameter of \code{stopping}. If the third
  parameter of \code{stopping} is \code{NA}, this last test is replaced
  by a test for improvement in the Akaike information criterion
  (AIC).

  After choosing the set of "tag" SNPs in this way, a prediction
  rule is generated either by calculating phased haplotype frequencies,
  either (a) under a log-linear model for linkage disequilibrium with
  only first order association terms fitted, or (b) under the
  "saturated" model.  
  These methods do not differ if there is only
  one tag SNP but, otherwise,  choice between  methods is controlled
  by the  \code{use.hap} parameters. 
  If the prediction,  as measure by  \eqn{R^2} achieved with the
  log-linear smoothing model exceeds a
  threshold (the first parameter of \code{use.hap})
  then this method is used. Otherwise, if the gain in \eqn{R^2}
  achieved by using the second method exceeds the second parameter of
  \code{use.hap}, then the second method is used.
  Current experience is that, the log-linear method is rarely
  preferred with reasonable choices for \code{use.hap}, and imputation
  is much faster when the second  method only is considered. 
  The current default ensures that this second method is used,
  but the other possibility might be considered if imputing
  from very small samples; however this code is not extensively tested
  and should be regarded as experimental.

  The argument \code{em.cntrl} controls convergence
  testing for the EM algorithm for fitting haplotype frequencies and the
  IPF algorithm for fitting the log-linear model. The
  first parameter is the maximum number of EM iterations, and the second
  parameter is the threshold for the change in log likelihood
  below which the iteration is judged to have converged. The third and
  fourth parameters give the maximum number of IPF iterations and the
  convergence tolerance. There should be no need to change the default
  values. 

  All SNPs selected for imputation must have sufficient data for
  estimating pairwise linkage disequilibrium with each other and with
  the target SNP. The statistic chosen is based on the four-fold tables
  of two-locus haplotype frequencies. If the frequencies in such a table
  are labelled \eqn{a, b, c} and \eqn{d} then, if \eqn{ad>bc} then
  \eqn{t = min(a,d)} and, otherwise,  \eqn{t = min(b,c)}. The cell
  frequencies \eqn{t} must exceed \code{minA} for all pairwise
  comparisons.  
}
\value{
  An object of class
  \code{"ImputationRules"}.
}
\references{
  Chapman J.M., Cooper J.D., Todd J.A. and Clayton D.G. (2003)
  \emph{Human Heredity}, \bold{56}:18-31.
  
  Wallace, C. et al. (2010) \emph{Nature Genetics}, \bold{42}:68-71
}
\note{The \code{phase=TRUE} option is not yet implemented} 
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{ImputationRules-class}},
  \code{\link{imputation.maf}}, \code{\link{imputation.r2}}}
\examples{
# Remove 5 SNPs from a datset and derive imputation rules for them
data(for.exercise)
sel <- c(20, 1000, 2000, 3000, 5000)
to.impute <- snps.10[,sel]
impute.from <- snps.10[,-sel]
pos.to <- snp.support$position[sel]
pos.fr <- snp.support$position[-sel]
imp <- snp.imputation(impute.from, to.impute, pos.fr, pos.to)
}
\keyword{models}
\keyword{regression}

\name{ImputationRules-class}
\docType{class}
\alias{ImputationRules-class}
\alias{show,ImputationRules-method}
\alias{summary,ImputationRules-method}
\alias{plot,ImputationRules,missing-method}
\alias{[,ImputationRules,ANY,missing,missing-method}

\title{Class "ImputationRules"}
\description{A class defining a list "rules" for imputation of
  SNPs. Rules are  estimated population
  haplotype probabilities for a target SNP and one or more predictor SNPs}
\section{Objects from the Class}{Objects are lists of
  \emph{rules}. Rules are named list elements each describing imputation
  of a SNP by a linear regression equation. Each element is itself a
  list with the following elements:
  \describe{
    \item{maf}{The minor allele frequency of the imputed SNP} 
    \item{r.squared}{The squared Pearson correlation coefficient between
    observed and predicted SNP duration derivation of the rule.}
    \item{snps}{The names of the SNPs to be included in the regression.}
    \item{hap.probs}{A numerical array containing estimated
      probabilities for haplotypes of the SNP to be imputed and all the
      predictor SNPs}
  }
  If any target SNP is monomorphic, the corresponding rule is
  returned as \code{NULL}. An object of class \code{ImputationRules}
  has an attribute, \code{Max.predictors}, which gives the maximum
  number of predictors used for any imputation.
}
\section{Methods}{
  \describe{
    \item{show}{\code{signature(object = "ImputationRules")}: prints
      an abreviated listing of the rules}
    \item{summary}{\code{signature(object = "ImputationRules")}:
      returns a table which shows the distribution of r-squared values
      achieved against the number of snps used for imputation}
    \item{plot}{\code{signature(x="ImputationRules", y="missing")}:
      plots the distribution of r-squared values as a stacked bar chart}
    \item{[]}{\code{signature(x = "ImputationRules", i = "ANY")}: subset
      operations}
  }
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{
  \code{\link{snp.imputation}}, \code{\link{impute.snps}},
  \code{\link{single.snp.tests}}
}
\examples{
showClass("ImputationRules")
}
\keyword{classes}

\name{pool}
\alias{pool}
\title{Pool test results from several studies or sub-studies}
\description{
  Given the same set of "score" tests carried out in several studies or
  in several different sub-samples within a study, this function pools
  the evidence by summation of the score statistics and score
  variances. It combines tests produced by
  \code{\link{single.snp.tests}} or by \code{\link{snp.lhs.tests}} and
  \code{\link{snp.rhs.tests}}. 
}
\usage{
pool(..., score = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{\dots}{Objects holding the (extended) test results. 
    These must be of class
    \code{\link[=SingleSnpTestsScore-class]{SingleSnpTests.score}}
    or \code{\link[=GlmTests-class]{snp.tests.glm}}
  }
  \item{score}{Is extended score information to
  be returned in the output object? Relevant only for
  \code{SingleSnpTestsScore} objects}
}
\details{
This function works by recursive calls to the generic function 
\code{pool2} which pools the results of two studies. 
}
\value{
  An object of same class as the input objects (optionally without the
  \code{.score}) extension. Tests are produced for the \emph{union} of
  SNPs tested in all the input objects. 
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{pool2}},
  \code{\link{SingleSnpTestsScore-class}},
  \code{\link{GlmTests-class}},
  \code{\link{single.snp.tests}},
  \code{\link{snp.lhs.tests}},
  \code{\link{snp.rhs.tests}}
}
\examples{
# An artificial example which simply doubles the size of a study
data(testdata)
sst <- single.snp.tests(snp.data=Autosomes, cc, data=subject.data,
         score=TRUE)
sst2 <- pool(sst, sst)
summary(sst2)
}
\keyword{htest}

\name{single.snp.tests}
\alias{single.snp.tests}
\title{1-df and 2-df tests for genetic associations with SNPs (or
  imputed SNPs)}
\description{
  This function carries out tests for association between phenotype and
  a series of single nucleotide polymorphisms (SNPs), within strata
  defined by a possibly confounding factor. SNPs are
  considered one at a time and both 1-df and 2-df tests are
  calculated. For a binary phenotype, the
  1-df test is the Cochran-Armitage test (or, when
  stratified, the Mantel-extension test). The function will also calculate
  the same tests for SNPs imputed by regression analysis.
}
\usage{
single.snp.tests(phenotype, stratum, data = sys.parent(), snp.data,
   rules=NULL, subset, snp.subset, uncertain = FALSE, score=FALSE)
}
\arguments{
  \item{phenotype}{A vector containing the values of the phenotype}
  \item{stratum}{Optionally, a factor defining strata for the analysis}
  \item{data}{A dataframe containing the \code{phenotype} and
    \code{stratum} data. The row names of this are linked with the row
    names of the \code{snps} argument to establish correspondence of
    phenotype and genotype data. If this argument is not supplied,
    \code{phenotype} and \code{stratum} are evaluated in the calling
    environment and should be in the same order as rows of \code{snps}}
  \item{snp.data}{An object of class \code{"SnpMatrix"} containing the SNP
    genotypes to be tested}
  \item{rules}{An object of class
    \code{"ImputationRules"}. If
    supplied, the rules coded in this object are used, together with
    \code{snp.data}, to calculate tests for imputed SNPs}
  \item{subset}{A vector or expression describing the subset of subjects
    to be used in the analysis. This is evaluated in the same
    environment as the \code{phenotype} and \code{stratum} arguments}
  \item{snp.subset}{A vector describing the subset of SNPs to be
    considered. Default action is to test all SNPs in \code{snp.data}
    or, in imputation mode,  as specified by \code{rules}}
  \item{uncertain}{If TRUE, uncertain genotypes are handled by replacing
    score contributions by their posterior expectations. Otherwise they
    are treated as missing. Setting this option authomatically invokes
    use of \code{robust} variance estimates}
   \item{score}{If \code{TRUE}, the output object will contain, for each
    SNP,  the score vector and its variance-covariance matrix}
}
\details{
  Formally, the test statistics are score tests for generalized linear
  models with canonical link. That is, they are inner products between
  genotype indicators and the deviations of phenotypes from their
  stratum means. Variances (and covariances) are those of the
  permutation distribution obtained by randomly permuting phenotype
  within stratum.

  When the function is used to calculate tests for imputed SNPs, the
  test is still a score test. The score statistics are calculated from
  the expected value, given observed SNPs, of the score statistic if the
  SNP to be tested were itself observed.
  
  The \code{subset} argument can either be a logical vector of length
  equal to the length of the vector of  phenotypes, an integer vector
  specifying positions in the \code{data} frame, or a character vector
  containing names of the selected rows in the \code{data}
  frame. Similarly, the \code{snp.subset} argument can be a logical,
  integer, or character vector.
}
\value{
  An object of class
  \code{"SingleSnpTests"}. 
  If \code{score} is set to \code{TRUE},
  the output object will be of the extended class
  \code{"SingleSnpTestsScore"}
  containing additional slots holding the score statistics and their
  variances (and covariances). This allows meta-analysis using the
  \code{\link{pool}} function.
}
\references{
  Chapman J.M., Cooper J.D., Todd J.A. and Clayton D.G. (2003)
  \emph{Human Heredity}, \bold{56}:18-31.\cr
  Clayton (2008) Testing for association on the X chromosome
  \emph{Biostatistics}, \bold{9}:593-600.)
}
\note{
  The 1 df imputation tests are described by Chapman et al. (2008)
  and the 2 df imputation tests are a simple extension of these.
  The behaviour of this function for objects of class
  \code{XSnpMatrix} is as described by Clayton (2008). Males are
  treated as homozygous females and corrected variance estimates are
  used.
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{snp.lhs.tests}}, \code{\link{snp.rhs.tests}},
  \code{\link{impute.snps}}, \code{\link{ImputationRules-class}},
  \code{\link{pool}}, 
	\code{\link{SingleSnpTests-class}},
    \code{\link{SingleSnpTestsScore-class}}
}
\examples{
data(testdata)
results <- single.snp.tests(cc, stratum=region, data=subject.data,
   snp.data=Autosomes, snp.subset=1:10)
print(summary(results))

# writing to an (anonymous and temporary) csv file
csvfile <- tempfile()
write.csv(file=csvfile, as(results, 'data.frame'))
unlink(csvfile)
# QQ plot 
qq.chisq(chi.squared(results, 1), 1)
qq.chisq(chi.squared(results, 2), 2)
}
\keyword{htest}

\name{imputation.maf}
\alias{can.impute}
\alias{imputation.maf}
\alias{imputation.r2}
\alias{imputation.nsnp}
\title{Extract statistics from imputation rules}
\description{
These functions extract key characteristics of regression-based
imputation rules stored as an object of class
\code{"ImputationRules"}. \code{imputation.maf} extracts the minor
allele frequencies of the imputed SNPs and \code{imputation.r2} extracts
the prediction \eqn{R^2}.
}
\usage{
can.impute(rules)
imputation.maf(rules)
imputation.r2(rules)
imputation.nsnp(rules)
}

\arguments{
  \item{rules}{An object of class \code{"ImputationRules"}}
}
\details{\code{can.impute} returns a logical vector identifying which
  rules allow a valid imputation.
  \code{imputation.maf} and \code{imputation.r2} extract the
  minor allele frequencies of the imputed SNPs and the \eqn{R^2} for
  prediction achieved when building each rule. \code{imputation.nsnp}
  returns the numbers of SNPs used in each imputation}
\value{
  Either a logical vector, or a numeric vector containing the extracted values
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{ImputationRules-class}},
  \code{\link{snp.imputation}}}
\examples{
# These functions are currently defined as
function (rules) sapply(rules, function(x) x$maf)
function (rules) sapply(rules, function(x) x$r2)
}
\keyword{manip}

\name{snp.rhs.tests}
\alias{snp.rhs.tests}
\title{ Score tests with SNP genotypes as independent variable}
\description{
This function fits a generalized linear model with phenotype as
dependent variable and, optionally, one or more potential confounders of
a phenotype-genotype association as independent variable. A series of
SNPs (or small groups of SNPs) are then tested for additional
association with phenotype. In order to protect against misspecification
of the variance function,  "robust" tests may be selected.
}
\usage{
snp.rhs.tests(formula, family = "binomial", link, weights, subset, data = parent.frame(),
   snp.data, rules=NULL, tests=NULL, robust = FALSE, uncertain=FALSE, 
   control=glm.test.control(), allow.missing=0.01, score=FALSE)
}
\arguments{
  \item{formula}{The base model formula, with phenotype as dependent variable}
  \item{family}{A string defining the generalized linear model
    family. This currently should (partially)  match one of
    \code{"binomial"}, \code{"Poisson"}, \code{"Gaussian"} or
      \code{"gamma"} (case-insensitive)}
  \item{link}{A string defining the link function for the GLM. This
  currently should (partially) match one of \code{"logit"},
  \code{"log"}, \code{"identity"} or \code{"inverse"}. The
  default action is to use the "canonical" link for the family selected}
  \item{data}{The dataframe in which the base model is to be fitted}
  \item{snp.data}{An object of class \code{"SnpMatrix"} or
    \code{"XSnpMatrix"} containing the SNP data} 
  \item{rules}{An object of class
    \code{"ImputationRules"}. If
    supplied, the rules coded in this object are used, together with
    \code{snp.data}, to calculate tests for imputed SNPs}
  \item{tests}{Either a vector of SNP names (or  numbers) for the SNPs
    to be tested, or a logical vector of length equal to the number of
    columns in \code{snp.data},
    or a list of short numeric or character
    vectors defining groups of SNPs to be
    tested (see \code{Details})}
  \item{weights}{"Prior" weights in the generalized linear model}
  \item{subset}{Array defining the subset of rows of  \code{data} to use}
  \item{robust}{If \code{TRUE}, robust tests will be carried out}
  \item{uncertain}{If \code{TRUE}, uncertain genotypes are used and
    scored by their posterior expectations. Otherwise they are treated
    as missing}
  \item{control}{An object giving parameters for the IRLS algorithm
    fitting of the base model and for the acceptable aliasing amongst
    new terms to be tested. See \code{\link{glm.test.control}}}
  \item{allow.missing}{The maximum proportion of SNP genotype that can
    be missing before it becomes necessary to refit the base model}
  \item{score}{Is extended score information to be returned?}
}
\details{
  The tests used are asymptotic chi-squared tests based on the vector of
  first and second derivatives of the log-likelihood with respect to the
  parameters of the additional model. The "robust" form is a generalized
  score test in the sense discussed by Boos(1992). The "base" model is
  first fitted, and a score test is performed for addition of one or
  more SNP genotypes to the model. Homozygous SNP genotypes are coded 0
  or 2 and heterozygous genotypes are coded 1. For SNPs on the X
  chromosome, males are coded as homozygous females. For X SNPs, it will
  often be
  appropriate to include sex of subject in the base model (this is
  not done automatically).
  
  If a \code{data} argument is supplied, the \code{snp.data} and
  \code{data} objects are aligned by rowname. Otherwise all variables in
  the model formulae are assumed to be stored in the same order as the
  columns of the \code{snp.data} object.

  Usually SNPs to be used in tests will be referenced by name. However,
  they can
  also be referenced by number, a positive number indicating the
  appropriate column in the input \code{snp.data}, and a negative number
  indicating (minus) a position in the \code{rules} list. They can also
  be referenced by a logical selection vector of length equal to the
  number of columns in \code{snp.data}. Sets of tests
  involving more than one SNP are referenced by a list and
  can use a mixture of observed and imputed
  SNPs. If the \code{tests} argument is missing, single SNP tests are
  carried out; if a \code{rules} is given, all \emph{imputed} SNP tests
  are calculated, otherwise all SNPs in the input \code{snp.data} matrix
  are tested. But note that, for single SNP tests, the function
  \code{\link{single.snp.tests}} will often achieve the same
  result much faster.
}
\value{
  An object of class \code{\link[=GlmTests-class]{GlmTests}}
  or \code{\link[=GlmTestsScore-class]{GlmTestsScore}}
  depending on whether \code{score} is set to \code{FALSE} or \code{TRUE}
  in the call.
}
\references{Boos, Dennis D. (1992) On generalized score tests. \emph{The
  American Statistician}, \strong{46}:327-333.}
\author{David Clayton \email{dc208@cam.ac.uk}}
\note{
  A factor (or
  several factors) may be included as arguments to the function
  \code{strata(...)} in the \code{formula}. This fits all
  interactions of the factors so included, but leads to faster
  computation  than fitting these in the normal way. Additionally, a
  \code{cluster(...)} call may be included in the base model
  formula. This identifies clusters of potentially correlated
  observations (e.g. for members of the same family); in this case, an
  appropriate robust estimate of the variance of the score test is used.
}
\seealso{\code{\link{GlmTests-class}},
  \code{\link{GlmTestsScore-class}},
  \code{\link{single.snp.tests}}, \code{\link{snp.lhs.tests}},
    \code{\link{impute.snps}}, \code{\link{ImputationRules-class}},
  \code{\link{SnpMatrix-class}}, \code{\link{XSnpMatrix-class}}}
\examples{
data(testdata)
slt3 <- snp.rhs.tests(cc~strata(region), family="binomial",
   data=subject.data, snp.data= Autosomes, tests=1:10)
print(slt3)
}
\keyword{htest}

\name{read.beagle}
\alias{read.beagle}
\title{
Read genotypes imputed by the BEAGLE program
}
\description{
The BEAGLE program generates, for each SNP and each subject, 
posterior probabilities for the three genotypes. This function reads
such data as a \code{SnpMatrix} object, storing the posterior
probabilities to as much accuracy allowed by a one-byte coding
}
\usage{
read.beagle(file, rownames=NULL, nsnp = NULL, header=TRUE)
}
\arguments{
  \item{file}{
    The input file name. This file my be gzipped.
  }
  \item{rownames}{
    The row names (sample identifiers) for the matrix
    }
  \item{nsnp}{
    The number of SNPs to be read in. This corresponds with the number
    of lines in the input file. If not supplied, the function does a
    preliminary pass to determine the number of lines
  }
  \item{header}{
    Set this \code{TRUE} if the file contains a header line (it won't
    for older versions of BEAGLE)
  }
}
\details{
  In later versions of BEAGLE, row names are listed on a header line.
  However, if the \code{rownames} argument is supplied, this will
  take precedence over the header line. If there is no header
  line and no row names are supplied, names
  are generated as \code{Sample1}, \code{Sample2} etc.
  
  No provision is made for data for the X chromosome. Such data must be
  first read as a \code{SnpMatrix} and subsequently coerced to an
\code{XSnpMatrix} object
}
\value{
an object of class \code{SnpMatrix}
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{SnpMatrix-class}}}
\examples{
##---- No example available yet
}
\keyword{IO}
\keyword{file}

\name{GlmTests-class}
\docType{class}
\alias{GlmTests-class}
\alias{[,GlmTests,ANY,missing,missing-method}
\alias{chi.squared,GlmTests,missing-method}
\alias{deg.freedom,GlmTests-method}
\alias{effect.sign,GlmTests,logical-method}
\alias{names,GlmTests-method}
\alias{coerce,GlmTests,data.frame-method}
\alias{p.value,GlmTests,missing-method}
\alias{sample.size,GlmTests-method}
\alias{show,GlmTests-method}
\alias{summary,GlmTests-method}

\alias{GlmTestsScore-class}
\alias{[,GlmTestsScore,ANY,missing,missing-method}
\alias{pool2,GlmTestsScore,GlmTestsScore,logical-method}
\alias{switch.alleles,GlmTestsScore,character-method}
  
\title{Classes "GlmTests" and "GlmTestsScore"}
\description{Classes of objects created by \code{\link{snp.lhs.tests}}
  and  \code{\link{snp.rhs.tests}}. The class
    \code{"GlmTestsScore"} extends the class
    \code{"GlmTests"} and is invoked by setting the argument
    \code{score=TRUE} when calling testing functions
    in order to save the scores and their variances
    (and covariances)
}
\section{Objects from the Class}{Objects of class \code{"GlmTests"}
  have four slots:
  \describe{
    \item{snp.names}{When only single SNPs are tested,
      a  character vector of SNP names. Otherwise a list
      of such vectors (one for each test)}
    \item{var.names}{A character vector containing names of variables
      tested against SNPs}
    \item{chisq}{A numerical vector of chi-squared test values}
    \item{df}{An integer vector of degrees of freedom for the tests}
    \item{N}{A integer vector of the number of samples contributing to
      each test}
  }
  The \code{"GlmTestsScore"} class extends this, adding a slot
  \code{score} containing a list with elements which are themselves
  lists with two elements:
  \describe{
    \item{U}{The vector (or matrix) of efficient scores}
    \item{V}{The upper triangle of the variance-covariance matrix of
      \code{U}, stored as a vector}
  }
}
\section{Methods}{
  \describe{
    \item{[]}{\code{signature(x = "GlmTests", i = "ANY", j =
	"missing", drop = "missing")}: Subsetting operator}
    \item{coerce}{\code{signature(from = "GlmTests", to = "data.frame")}:
      Simplify object}
    \item{chi.squared}{\code{signature(x = "GlmTests", df =
	"missing")}: Extract chi-squared test values}
    \item{deg.freedom}{\code{signature(x = "GlmTests")}: Extract
      degrees of freedom for tests}
    \item{names}{\code{signature(x="GlmTests")}: Extract (or generate)
      a name for each test}
    \item{p.value}{\code{signature(x = "GlmTests", df =
	"missing")}: Extract \emph{p}-values}
    \item{sample.size}{\code{signature(object = "GlmTests")}:
      Extract sample sizes for tests}
    \item{show}{\code{signature(object = "GlmTests")}: Show method}
    \item{summary}{\code{signature(object = "GlmTests")}: Summary
      method}

    \item{[]}{\code{signature(x = "GlmTestsScore", i = "ANY", j =
	"missing", drop = "missing")}: Subsetting operator}
    \item{effect.sign}{\code{signature(x = "GlmTestsScore", simplify =
	"logical")}: Extract signs of associations. If \code{simpify} is
      \code{TRUE} then a simple vector is returned if all tests are on 1df}
    \item{pool2}{\code{signature(x = "GlmTestsScore", y =
	"GlmTestsScore", score = "logical")}: Combine results from two
      sets of tests}
    \item{switch.alleles}{\code{signature(x = "GlmTestsScore", snps =
	"character")}: Emulate, in the score vector and its (co)variances,
      the effect of switching of the alleles of specified SNPs}
  }
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\note{Most of the methods for this class are shared with
the \code{\linkS4class{SingleSnpTests}} and
\code{\linkS4class{SingleSnpTestsScore}} classes
}
\seealso{\code{\link{snp.lhs.tests}},\code{\link{snp.rhs.tests}},
   \code{\linkS4class{SingleSnpTests}},
   \code{\linkS4class{SingleSnpTestsScore}}
}
\examples{
showClass("GlmTests")
}
\keyword{classes}

\name{write.plink}
\alias{write.plink}
\title{
Write files for analysis in the PLINK toolset
}
\description{
Given a \code{SnpMatrix} object, together with associated subject and
SNP support dataframes, this function writes \code{.bed}, 
\code{.bim}, and \code{.fam} files for processing in the PLINK toolset
}
\usage{
write.plink(file.base, snp.major = TRUE, snps,
   subject.data, pedigree, id, father, mother, sex, phenotype,
   snp.data, chromosome, genetic.distance, position, allele.1, allele.2,
   na.code = 0, human.genome=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{file.base}{A character string giving the base filename. The
    extensions \code{.bed}, \code{.bim}, and \code{.fam} are appended to
    this string to give the filenames of the three output files
  }
  \item{snp.major}{Logical variable controlling whether the \code{.bed}
    file is in SNP-major or subject-major order 
  }
  \item{snps}{The \code{SnpMatrix} or \code{XSnpMatrix} object to be written out
  }
  \item{subject.data}{(Optional) A subject support dataframe. If
    supplied, the next six arguments (which define the fields of the
    PLINK \code{.fam} file) will be evaluated in this
    environment, after matching row names with the row names of \code{snps}.
    Otherwise they will be evaluated in the calling environment; they then
    must be of the right length and in the correct order.
  }
  \item{pedigree}{A pedigree (family) identifier. Default is the row
    names of \code{snps}.
  }
  \item{id}{An identifier of an individual within family. Default is a
    vector of \code{na.code}.
  }
  \item{father}{The within-family identifier of the subject's
    father. Default is a vector of  \code{na.code}.
  }
  \item{mother}{The within-family identifier of the subject's
    mother. Default is a vector of  \code{na.code}.
  }
  \item{sex}{Sex of the individual. Default is a vector of
    \code{na.code}. This will be coerced to type \code{numeric}.
  }
  \item{phenotype}{The primary phenotype value. Default is a vector of
     \code{na.code}. This will be coerced to type \code{numeric}.
  }
  \item{snp.data}{(Optional) A SNP support dataframe. If
    supplied, the next five arguments (which define the  columns of the
    PLINK \code{.bim} file) will be evaluated in this
    environment,  after matching row names with the column names of
    \code{snps}. Otherwise they will be evaluated in the calling
    environment;  they then
    must be of the right length and in the correct order. 
  }
  \item{chromosome}{The chromosome on which the SNP is located. This
    should either be numeric, as specified by SPLINK, or character, with
    \code{"X"}, \code{"Y"}, \code{"XY"}, and \code{"MT"} for the
    non-numeric values. Default 
    is a vector of  \code{na.code}, or a vector of 23's if \code{snps}
    is a \code{XSnpMatrix}.
  }
  \item{genetic.distance}{The location of the SNP, expressed as a
    genetic distance. Default is a vector of  \code{na.code}.
    This will be coerced to type \code{numeric}.
  }
  \item{position}{The physical location of the SNP, expressed in base pairs.
    Default is a vector of  \code{na.code}.
    This will be coerced to type \code{numeric}.
  }
  \item{allele.1}{A character vector giving the first allele. Default is
    a vector of \code{"A"}s.
  }
  \item{allele.2}{A character vector giving the first allele. Default is
    a vector of \code{"B"}s.
  }
  \item{na.code}{The code to be written for \code{NA}
    in the \code{.fam} and \code{.bin} output files. It should be
    numeric (or capable of coercion to numeric).
  }
  \item{human.genome}{If TRUE, check the
    \code{chromosome} argument for valid values.
  }
}
\details{
  For more details of required codings in \code{.fam} and \code{.bim}
  files, see the PLINK documentation.
}
\value{
Returns \code{NULL}.
}
\references{
  PLINK: Whole genome association analysis toolset.
  \url{http://pngu.mgh.harvard.edu/~purcell/plink/}
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{read.plink}},
  \code{\link{SnpMatrix-class}}, \code{\link{XSnpMatrix-class}}
}
\examples{
data(testdata)
## the use of as.numeric() below is not necessary since this is done
## automatically. It just illustrates use of expressions for these arguments
## Note that cc and sex are variables within the subject.data frame
## This command writes files test.bed. test.fam and test.bim
write.plink(file.base="test", snps=Autosomes,
    subject.data=subject.data, phenotype = as.numeric(cc), sex=as.numeric(sex),
    snp.major=FALSE)
}
\keyword{manip}
\keyword{IO}
\keyword{file}
\keyword{utilities}

\name{convert.snpMatrix}
\alias{convert.snpMatrix}
\alias{convert.snpMatrix.dir}
\title{
Convert \code{snpMatrix} objects to \code{snpStats} objects
}
\description{
These functions convert \code{snpMatrix} objects to \code{snpStats}
objects. \code{convert.snpMatrix} converts a single object, while
\code{convert.snpMatrix.dir} converts all stored elements in a specified
directory. They really only change the class names since most of the
classes in \code{snpStats} are backwards-compatible with 
\code{snpMatrix}. The exception is the \code{ImputationRules} class; 
\code{imputation.rules} objects will need to be regenerated.
}
\usage{
convert.snpMatrix(object)

convert.snpMatrix.dir(dir = ".", ext = "RData")
}
\arguments{
  \item{object}{
    Object to be converted
  }
  \item{dir}{
    A directory containing saved \code{snpMatrix} objects
  }
  \item{ext}{
    The file extension for files containing such objects
  }
}
\value{
\code{convert.snpMatrix} returns the converted object. 
\code{convert.snpMatrix.dir} rewrites the files in place.
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\keyword{classes}

\name{read.snps.long}
\alias{read.snps.long}
\title{Read SNP data in long format (deprecated)}
\description{
  Reads SNP data when organized in free format
  as one call per line. Other than the one
  call per line requirement, there is considerable flexibility. Multiple
  input files can be read, the input fields can be in any order on the
  line, and irrelevant fields can be skipped. The samples and SNPs
  to be read  must be pre-specified, and define rows and columns of an
  output object of class \code{"SnpMatrix"}. This function has been
  replaced in versions 1.3 and later by the more flexible function
  \code{read.long}. 
}
\usage{
read.snps.long(files, sample.id = NULL, snp.id = NULL, diploid = NULL,
              fields = c(sample = 1, snp = 2, genotype = 3, confidence = 4),
              codes = c("0", "1", "2"), threshold = 0.9, lower = TRUE,
              sep = " ", comment = "#", skip = 0, simplify = c(FALSE,FALSE),
              verbose = FALSE, in.order=TRUE, every = 1000)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{files}{A character vector giving the names of the input files}
  \item{sample.id}{A character vector giving the identifiers of the
    samples to be read}
  \item{snp.id}{A character vector giving the names of the SNPs to be read}
  \item{diploid}{A logical array of the same length as
    \code{sample.id}, required if reading data into an \code{XSnpMatrix}
    rather than a \code{SnpMatrix}. This vector gives the expected
    ploidy for each row. If the same value suffices for all rows, then a
    scalar may be supplied}
  \item{fields}{A integer vector with named elements specifying the
    positions of the required fields in the input record. The fields are
    identified by the names \code{sample} and \code{snp} for the sample
    and SNP identifier fields, \code{confidence} for a call confidence
    score (if present) and either \code{genotype} if genotype calls
    occur as a single field, or \code{allele1} and \code{allele2} if the
    two alleles are coded in different fields}
  \item{codes}{Either the single string \code{"nucleotide"} denoting
    that coding in terms of nucleotides
    (\code{A}, \code{C}, \code{G} or \code{T}, case insensitive),
    or a character vector
    giving genotype or allele codes (see below)}
  \item{threshold}{A numerical value for the calling threshold on the
    confidence score}
  \item{lower}{If \code{TRUE}, then \code{threshold} represents a lower
    bound. Otherwise it is an upper bound}
  \item{sep}{The delimiting character separating fields in the input record}
  \item{comment}{A character denoting that any remaining input on a line
    is to be ignored}
  \item{skip}{An integer value specifying how many lines are to be
    skipped at the beginning of each data file}
  \item{simplify}{If \code{TRUE}, sample and SNP identifying strings
    will be shortened by removal of any common leading or trailing
    sequences when they are used as row and column names of the output
    \code{SnpMatrix} }
  \item{verbose}{If \code{TRUE}, a progress report is generated as
    every \code{every} lines of data are read}
  \item{in.order}{If \code{TRUE}, input lines are assumed to be in the
    correct order (see details)}
  \item{every}{See \code{verbose}}
}
\details{
  If nucleotide coding is not used, the \code{codes} argument
  should be a character array giving the valid codes. 
  For genotype coding of autosomal SNPs, this should be
  an array of length 3 giving the codes
  for the three genotypes, in the order homozygous(AA), heterozygous(AB),
  homozygous(BB). All other codes will be  treated
  as "no call". The default codes are \code{"0"},  \code{"1"},
  \code{"2"}.  For X SNPs, males are assumed to be coded as homozygous,
  unless an additional two codes are supplied (representing the 
  AY and BY genotypes). For allele coding, the
  \code{codes} array should be of length 2 and should specify the codes
  for the two alleles. Again, any other code is treated as
  "missing" and, for X SNPs, males should be coded either as
  homozygous or by omission of  the second allele.
  
  For nucleotide coding, nucleotides are assigned to the nominal alleles
  in alphabetic order. Thus, for a SNP with either  "T" and "A"
  nucleotides in the variant position,
  the nominal genotypes AA, AB and BB will refer to A/A,
  A/T and T/T.
  
  Although the function allows for reading into an object of class
  \code{XSnpMatrix} directly,
  it is usually preferable to read such data as a \code{"SnpMatrix"}
  (i.e. as autosomal) and to coerce it to an object of type
  \code{"XSnpMatrix"} later using \code{as(..., "X.SnpMatrix")} or
  \code{new("XSnpMatrix", ..., diploid=...)}. If \code{diploid}
  is coded \code{NA} for any subject the latter course \emph{must} be
  followed, since \code{NA}s are not accepted in the \code{diploid} argument.

  If the \code{in.order} argument is set \code{TRUE}, then
  the vectors \code{sample.id} and \code{snp.id} must be in the same
  order as they  vary on the input file(s) and this ordering must be
  consistent. However, there is
  no requirement that either SNP or sample should vary fastest as this is
  detected from the input. If \code{in.order} is \code{FALSE}, then no
  assumptions about the ordering of the input file are assumed and SNP
  and sample identifiers are looked up in hash tables as they are
  read. This option must be expected, therefore, to be somewhat slower.
  Each file may represent a separate sample or SNP, in which case the
  appropriate \code{.id} argument can be omitted; row or column names
  are then taken from the file names.  
}
\value{
  An object of class \code{"SnpMatrix"} or \code{"XSnpMatrix"}.
}
\note{
  The function will read gzipped files.
  
  If \code{in.order} is \code{TRUE},
  every combination of sample and snp listed in the
  \code{sample.id} and \code{snp.id} arguments  \emph{must} be present in the
  input file(s). Otherwise the function will search for any missing
  observation until reaching the end of the data, ignoring everything
  else on the way. 
}
\author{David Clayton \email{dc208@cam.ac.uk}}
\seealso{\code{\link{read.plink}}, 
  \code{\link{SnpMatrix-class}}, \code{\link{XSnpMatrix-class}}}
\keyword{manip}
\keyword{IO}
\keyword{file}
\keyword{utilities}

\name{pp}
\alias{pp}
\title{
Unpack posterior probabilities from one-byte codes
}
\description{
In \code{snpStats}, the three "posterior" probabilities corresponding to
the possible values of an uncertain genotype are packed into a single
byte code (with, of course, some loss in accuracy). This function, which
is provided as an aid to writing new functions, unpacks the posterior
probabilities from the single byte codes.  
}
\usage{
pp(x, transpose = FALSE)
}
\arguments{
  \item{x}{
    A vector, length N, which can be coerced into type \code{raw}
  }
  \item{transpose}{
    If \code{FALSE}, the result is an Nx3 matrix of posterior
    probabilities. If \code{TRUE}, a 3xN matrix is returned.
  }
}
\value{
  A numeric matrix
}
\author{
David Clayton \email{dc208@cam.ac.uk}
}
\examples{
##
## Read imputed data from a file produced by MACH 
##
path <- system.file("extdata/mach1.out.mlprob.gz", package="snpStats")
mach <- read.mach(path)
pp(mach[1:50, 10])
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
